@@ -2,7 +2,7 @@
 *                Exim Monitor                    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996. */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -17,21 +17,29 @@ XtAppContext X_appcon;
 
 XtActionsRec actionTable[] = {
   { "dialogAction",  (XtActionProc)dialogAction}};
-  
-int actionTableSize = sizeof(actionTable)/sizeof(XtActionsRec); 
 
-XtTranslations queue_trans;      
-XtTranslations text_trans;      
+int actionTableSize = sizeof(actionTable)/sizeof(XtActionsRec);
 
+XtTranslations queue_trans;
+XtTranslations text_trans;
+
+Widget dialog_ref_widget;
 Widget toplevel_widget;
 Widget log_widget;
 Widget queue_widget;
+Widget unhide_widget = NULL;
 
 
 FILE  *DEBUG = NULL;
 FILE  *LOG;
 
 int    action_output = TRUE;
+int    action_queue_update = TRUE;
+
+char   actioned_message[24];
+char  *action_required;
+
+int    body_max = 20000;
 
 int    eximon_initialized = FALSE;
 
@@ -44,6 +52,8 @@ ino_t  log_inode;
 long int log_position;
 int    log_width = 600;
 
+char  *menu_event = "Shift<Btn1Down>";
+int    menu_is_up = FALSE;
 int    min_height = 162;
 int    min_width  = 103;
 
@@ -54,6 +64,7 @@ char  *queue_buffer = NULL;
 int    queue_depth = 200;
 char  *queue_font = NULL;
 int    queue_max_addresses = 10;
+skip_item *queue_skip = NULL;
 int    queue_update = 60;
 int    queue_width = 600;
 
@@ -71,7 +82,6 @@ int    stripchart_varstart = 1;
 
 int    tick_queue_accumulator = 999999;
 
-
 char  *window_title = "exim monitor";
 
 /* End of em_globals.c */
