@@ -2,7 +2,7 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -46,9 +46,12 @@ the exim monitor program. */
 
 #include <errno.h>
 #include <dirent.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
+#include <pwd.h>
+#include <grp.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <unistd.h>
 
 /* The new standard is statvfs; some OS have statfs. Also arrange
 to be able to cut it out altogether for way-out OS that don't have
@@ -61,6 +64,9 @@ anything. */
 #else
 #ifdef HAVE_SYS_VFS_H
 #include <sys/vfs.h>
+#ifdef HAVE_SYS_STATFS_H
+#include <sys/statfs.h>
+#endif
 #endif
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
@@ -99,7 +105,7 @@ struct rlimit {
 #include <sys/time.h>
 #include <sys/resource.h>
 #endif
-   
+
 /* X11 includes */
 
 #include <X11/Xlib.h>
@@ -149,8 +155,9 @@ enum { dest_noop, dest_add, dest_remove };
 
 typedef struct dest_item {
   struct dest_item *next;
+  struct dest_item *parent;
   char address[1];
-} dest_item;     
+} dest_item;
 
 
 
@@ -160,15 +167,27 @@ typedef struct dest_item {
 
 typedef struct queue_item {
   struct queue_item *next;
-  struct queue_item *prev; 
-  struct dest_item  *destinations; 
-  int  input_time; 
-  int  update_time; 
-  char *sender; 
+  struct queue_item *prev;
+  struct dest_item  *destinations;
+  int  input_time;
+  int  update_time;
+  int  size;
+  char *sender;
   char name[17];
   char seen;
-  char frozen; 
-} queue_item;     
+  char frozen;
+} queue_item;
+
+
+/*************************************************
+*          Structure for queue skip items        *
+*************************************************/
+
+typedef struct skip_item {
+  struct skip_item *next;
+  time_t reveal;
+  char text[1];
+} skip_item;
 
 
 /*************************************************
@@ -179,7 +198,7 @@ typedef struct pipe_item {
   struct pipe_item *next;
   int fd;
   Widget widget;
-} pipe_item;     
+} pipe_item;
 
 
 
@@ -194,17 +213,25 @@ extern XtActionsRec actionTable[]; /* Actions table */
 extern XtTranslations queue_trans; /* translation table for queue text widget */
 extern XtTranslations text_trans;  /* translation table for other text widgets */
 
+extern Widget dialog_ref_widget;   /* for positioning dialog box */
 extern Widget toplevel_widget;
 extern Widget log_widget;          /* widget for tail display */
 extern Widget queue_widget;        /* widget for queue display */
+extern Widget unhide_widget;       /* widget for unhide button */
 
 
 extern FILE  *DEBUG;
 extern FILE  *LOG;
 
 extern int    action_output;       /* TRUE when wanting action command output */
+extern int    action_queue_update; /* controls auto updates */
 extern int    actionTableSize;     /* # entries in actionTable */
 
+extern char   actioned_message[];  /* For menu handling */
+extern char  *action_required;
+
+extern int    body_max;            /* Max size of body to display */
+
 extern int    eximon_initialized;  /* TRUE when initialized */
 
 extern int    log_buffer_size;     /* size of log buffer */
@@ -216,16 +243,19 @@ extern ino_t  log_inode;           /* the inode of the log file */
 extern long int log_position;      /* position in log file */
 extern int    log_width;           /* width of log tail window */
 
+extern char  *menu_event;          /* name of menu event */
+extern int    menu_is_up;          /* TRUE when menu displayed */
 extern int    min_height;          /* min window height */
 extern int    min_width;           /* min window width */
 
 extern pipe_item *pipe_chain;      /* for delivery displays */
 
-extern char  *qualify_domain;     
+extern char  *qualify_domain;
 extern char  *queue_buffer;        /* large buffer for header handling */
 extern int    queue_depth;         /* depth of queue window */
 extern char  *queue_font;          /* font for queue display */
 extern int    queue_max_addresses; /* limit on per-message list */
+extern skip_item *queue_skip;      /* for hiding bits of queue */
 extern int    queue_update;        /* update interval */
 extern int    queue_width;         /* width of queue window */
 
@@ -254,12 +284,14 @@ extern char  *window_title;        /* title of the exim monitor window */
 extern XtActionProc dialogAction(Widget, XEvent *, String *, Cardinal *);
 
 extern char *copystring(char *);
+extern void  create_dialog(char *, char *);
 extern void  create_stripchart(Widget, char *);
 extern void  debug(char *, ...);
 extern dest_item *find_dest(queue_item *, char *, int);
 extern queue_item *find_queue(char *, int);
 extern void  init(int, char **);
 extern void  menu_create(Widget, XEvent *, String *, Cardinal *);
+extern void  NonMessageDialogue(char *);
 extern void  queue_display(void);
 extern void  read_log(void);
 extern int   read_spool(char *);
@@ -267,14 +299,17 @@ extern int   read_spool_init(char *);
 extern void  read_spool_tidy(void);
 extern int   repaint_window(StripChartWidget, int, int);
 extern void  scan_spool_input(int, int);
+extern void  stripchart_init(void);
 extern int   strncmpic(char *, char *, int);
 extern char *strstric(char *, char *, int);
 extern void  str_remove(char *, char *, char *);
 extern char *str_subfield(char *, char *, int);
 extern char *substr(char *, int, int);
 extern void  substr_init(void);
+extern void  substr_reset(void);
 extern void  text_empty(Widget);
-extern void  text_show(Widget, char *, ...);
+extern void  text_show(Widget, char *);
+extern void  text_showf(Widget, char *, ...);
 extern void  text_position(Widget, int);
 extern void  xs_SetValues(Widget, Cardinal, ...);
 
