@@ -2,7 +2,7 @@
 *                  Exim monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996. */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module contains code to initialize things from the
@@ -33,67 +33,67 @@ work. */
 for (i = 0; i <= 1; i++)
 
   {
-  int first = 1; 
-  int count = 0; 
+  int first = 1;
+  int count = 0;
   char *p = s;
-   
+
   if (*p == '/') p++;   /* allow optional / at start */
 
   /* This loops for all the substrings, using the first flag
   to determine whether each is the first or second of the pairs. */
-      
+
   while (*p)
     {
-    char *pp; 
-    /* Handle continuations */ 
+    char *pp;
+    /* Handle continuations */
     if (*p == '\n')
       {
       while (*(++p) == ' ' || *p == '\t');
-      if (*p == '/') p++;  
-      }  
+      if (*p == '/') p++;
+      }
 
     /* Find the end of the string and count if first string */
-     
+
     pp = p;
-    while (*p && *p != '/') p++; 
-    if (first) count++; 
-  
+    while (*p && *p != '/') p++;
+    if (first) count++;
+
     /* Take action on the second time round. So as to give more
     information from the regexp error routine, set a pointer to
     the string we are compiling. */
-     
+
     if (i != 0)
       {
       char buffer[256];
       int indx = count + stripchart_varstart - 1;
       strncpy(buffer, pp, p-pp);
-      buffer[p-pp] = 0;   
-      if (first) 
+      buffer[p-pp] = 0;
+      if (first)
         {
-        regexp_compiling = buffer; 
+        regexp_compiling = buffer;
         stripchart_regexp[indx] = regcomp(buffer);
-        regexp_compiling = NULL; 
-        } 
+        regexp_compiling = NULL;
+        }
       else stripchart_title[indx] = string_copy(buffer);
       }
-   
+
     /* Advance past the delimiter and flip the first/second flag */
-     
+
     p++;
     first = !first;
     }
-   
+
   /* On the first pass, we now know the number of stripcharts. Get
   store for holding the pointers to the regular expressions and
   title strings. */
 
   if (i == 0)
     {
-    stripchart_number += count;  
+    stripchart_number += count;
     stripchart_regexp = (regexp **)malloc(stripchart_number * sizeof(regexp *));
-    stripchart_title = (char **)malloc(stripchart_number * sizeof(char *));  
-    }    
-  } 
+    stripchart_title = (char **)malloc(stripchart_number * sizeof(char *));
+    }
+  }
 }
 
 
@@ -111,6 +111,12 @@ char *s;
 s = getenv("ACTION_OUTPUT");
 if (s != NULL && strcmp(s, "no") == 0) action_output = FALSE;
 
+s = getenv("ACTION_QUEUE_UPDATE");
+if (s != NULL && strcmp(s, "no") == 0) action_queue_update = FALSE;
+
+s = getenv("BODY_MAX");
+if (s != NULL && (x = atoi(s)) != 0) body_max = x;
+
 s = getenv("EXIM_PATH");
 if (s != NULL) exim_path = string_copy(s);
 
@@ -124,17 +130,23 @@ if (s != NULL)
     if (x < 1024) x = 1024;
     log_buffer_size = x;
     }
-  }   
+  }
 
 s = getenv("LOG_DEPTH");
 if (s != NULL && (x = atoi(s)) != 0) log_depth = x;
 
+s = getenv("LOG_FILE_NAME");
+if (s != NULL) log_file = string_copy(s);
+
 s = getenv("LOG_FONT");
 if (s != NULL) log_font = string_copy(s);
 
 s = getenv("LOG_WIDTH");
 if (s != NULL && (x = atoi(s)) != 0) log_width = x;
 
+s = getenv("MENU_EVENT");
+if (s != NULL) menu_event = string_copy(s);
+
 s = getenv("MIN_HEIGHT");
 if (s != NULL && (x = atoi(s)) > 0) min_height = x;
 
@@ -166,14 +178,8 @@ if (s != NULL) spool_directory = string_copy(s);
 s = getenv("WINDOW_TITLE");
 if (s != NULL) window_title = string_copy(s);
 
-/* Set up the log file name */
-
-log_file = store_malloc((int)strlen(spool_directory) + 
-  (int)strlen("/log/mainlog") + 1);
-sprintf(log_file, "%s/log/mainlog", spool_directory); 
-
 /* Deal with stripchart configuration. First see if we are monitoring
-the size of a partition, then deal with log stripcharts in a separate 
+the size of a partition, then deal with log stripcharts in a separate
 function */
 
 s = getenv("SIZE_STRIPCHART");
@@ -182,7 +188,7 @@ if (s != NULL)
   stripchart_number++;
   stripchart_varstart++;
   size_stripchart = string_copy(s);
-  }    
+  }
 
 s = getenv("LOG_STRIPCHARTS");
 if (s != NULL) decode_stripchart_config(s);
