@@ -2,7 +2,7 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996. */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module contains code for scanning the smaill log,
@@ -28,7 +28,7 @@ static void show_log(char *s, ...)
 int length, newtop;
 va_list ap;
 XawTextBlock b;
-char buffer[1024];
+char buffer[log_buffer_len + 24];
 
 /* Initialize the text block structure */
 
@@ -98,7 +98,7 @@ always visible. */
 
 if (!scrolled)
   {
-  XawTextSetInsertionPoint(log_widget, size - 2);
+  XawTextSetInsertionPoint(log_widget, size - 1);
   top = XawTextTopPosition(log_widget);
   }
 }
@@ -126,50 +126,49 @@ char buffer[log_buffer_len];
 if (LOG != NULL)
   {
   fseek(LOG, log_position, SEEK_SET);
-  
+
   while (fgets(buffer, log_buffer_len, LOG) != NULL)
     {
     int i;
-    int length = (int)strlen(buffer);
-    char *id; 
+    char *id;
     char *p = buffer;
-  
+
     /* Skip totally blank lines */
-  
+
     while (*p == ' ' || *p == '\t') p++;
     if (*p == '\n') continue;
-  
+
     /* We should now have a complete log entry in the buffer; check
     it for various regular expression matches and take appropriate
     action. First, reset the substring handling to recover its store. */
-  
+
     substr_reset();
-  
+
     /* First, update any stripchart data values, noting that the zeroth
     stripchart is the queue length, which is handled elsewhere, and the
     1st may the a size monitor. */
-  
+
     for (i = stripchart_varstart; i < stripchart_number; i++)
       {
       if (regexec(stripchart_regexp[i], buffer)) stripchart_total[i]++;
       }
-  
+
     /* Munge the log entry and display shortened form on one line.
     We omit the date and show only the time. */
-    
+
     if (regexec(yyyymmdd_regexp, buffer))
-      { 
-      id = substr(buffer, 20, MESSAGE_ID_LENGTH);
-      show_log("%s", buffer+11);    
-      }
-    else 
       {
-      id = ""; 
-      show_log("%s", buffer);    
-      } 
-   
+      id = substr(buffer, 20, MESSAGE_ID_LENGTH);
+      show_log("%s", buffer+11);
+      }
+    else
+      {
+      id = "";
+      show_log("%s", buffer);
+      }
+
     /* Deal with frozen and unfrozen messages */
-    
+
     if (strstric(buffer, "frozen", FALSE) != NULL)
       {
       queue_item *qq = find_queue(id, queue_noop);
@@ -177,49 +176,75 @@ if (LOG != NULL)
         {
         if (strstric(buffer, "unfrozen", FALSE) != NULL)
           qq->frozen = FALSE;
-        else qq->frozen = TRUE;     
-        }   
-      }   
-      
-    /* Notice defer messages, and add the destination if it
-    isn't already on the list for this message. This
-    code picks up deferrals that don't cause msglog entries. */
-  
-    if ((p = strstric(buffer, "deferred:", TRUE)) != NULL)
-      { 
-      char *q; 
-      queue_item *qq;
-        
-      while (*(--p) == ' ');
-      q = p;
-      while (*(--p) != ' ');
-      
-      if (p[1] != '<')
-        {
-        *p = '<';
-        q[1] = '>';
-        q[2] = 0;
+        else qq->frozen = TRUE;
         }
-      else
+      }
+
+    /* Notice defer messages, and add the destination if it
+    isn't already on the list for this message, with a pointer
+    to the parent if we can. */
+
+    if ((p = strstr(buffer, "==")) != NULL)
+      {
+      queue_item *qq = find_queue(id, queue_noop);
+      if (qq != NULL)
         {
-        p++;
-        q[1] = 0;
-        }            
-      
-      if ((qq = find_queue(id, queue_noop)) != NULL) 
-        find_dest(qq, p, dest_add); 
+        dest_item *d;
+        char *q, *r;
+        p += 2;
+        while (isspace(*p)) p++;
+        q = p;
+        while (*p != 0 && !isspace(*p))
+          {
+          if (*p++ != '\"') continue;
+          while (*p != 0)
+            {
+            if (*p == '\\') p += 2;
+              else if (*p++ == '\"') break;
+            }
+          }
+        *p++ = 0;
+        if ((r = strstric(q, qualify_domain, FALSE)) != NULL &&
+          *(--r) == '@') *r = 0;
+        d = find_dest(qq, q, dest_add);
+        if (d->parent == NULL)
+          {
+          while (isspace(*p)) p++;
+          if (*p == '<')
+            {
+            dest_item *dd;
+            q = ++p;
+            while (*p != 0 && *p != '>') p++;
+            *p = 0;
+            if ((p = strstric(q, qualify_domain, FALSE)) != NULL &&
+              *(--p) == '@') *p = 0;
+            dd = find_dest(qq, q, dest_noop);
+            if (dd != NULL && dd != d) d->parent = dd;
+            }
+          }
+        }
       }
     }
-  }   
+  }
 
 
-/* We have to detect when the log file is changed, and switch
-to the new file. In practice, this means that some deliveries
-might go unrecorded, since they'll be written to the old file,
-but this happens in the middle of the night, and I don't think
-the hassle of keeping track of two log files is worth it. */
+/* We have to detect when the log file is changed, and switch to the new file.
+In practice, this means that some deliveries might go unrecorded, since they'll
+be written to the old file, but this usually happens in the middle of the
+night, and I don't think the hassle of keeping track of two log files is worth
+it.
 
-if (LOG == NULL || 
+The test for a changed log file is to look up the inode of the file by name and
+compare it with the saved inode of the file we currently are processing. This
+accords with the usual interpretation of POSIX and other Unix specs that imply
+"one file, one inode". However, it appears that on some Digital systems, if an
+open file is unlinked, a new file may be created with the same inode while the
+old file remains in existence. This can happen if the old log file is renamed,
+processed in some way, and then deleted. To work round this, also test for a
+link count of zero on the currently open file. */
+
+if (LOG == NULL ||
+    (fstat(fileno(LOG), &statdata) == 0 && statdata.st_nlink == 0) ||
     (stat(log_file, &statdata) == 0 && log_inode != statdata.st_ino))
   {
   FILE *TEST;
@@ -231,10 +256,10 @@ if (LOG == NULL ||
 
   if ((TEST = fopen(log_file, "r")) != NULL)
     {
-    fclose(TEST);
-    if (LOG == NULL) stat(log_file, &statdata); else fclose(LOG);
-    LOG = fopen(log_file, "r");
-    if (LOG != NULL) log_inode = statdata.st_ino;
+    if (LOG != NULL) fclose(LOG);
+    LOG = TEST;
+    fstat(fileno(LOG), &statdata);
+    log_inode = statdata.st_ino;
     }
   }
 
