@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996. */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -51,6 +51,26 @@ static Arg update_args[] = {
   {"bottom",    XawChainTop}
 };
 
+static Arg hide_args[] = {
+  {XtNfromVert, (XtArgVal) NULL},         /* must be first */
+  {XtNfromHoriz,(XtArgVal) NULL},         /* must be second */
+  {XtNlabel,    (XtArgVal) " Hide "},
+  {"left",      XawChainLeft},
+  {"right",     XawChainLeft},
+  {"top",       XawChainTop},
+  {"bottom",    XawChainTop}
+};
+
+static Arg unhide_args[] = {
+  {XtNfromVert, (XtArgVal) NULL},         /* must be first */
+  {XtNfromHoriz,(XtArgVal) NULL},         /* must be second */
+  {XtNlabel,    (XtArgVal) " Unhide "},
+  {"left",      XawChainLeft},
+  {"right",     XawChainLeft},
+  {"top",       XawChainTop},
+  {"bottom",    XawChainTop}
+};
+
 static Arg log_args[] = {
   {XtNfromVert, (XtArgVal) NULL},         /* must be first */
   {"editType",  XawtextEdit},
@@ -58,7 +78,7 @@ static Arg log_args[] = {
   {"string",    (XtArgVal)""},            /* dummy to get it going */
   {"scrollVertical", XawtextScrollAlways},
   {"scrollHorizontal", XawtextScrollAlways},
-  {"right",     XawChainRight}, 
+  {"right",     XawChainRight},
   {"top",       XawChainTop},
   {"bottom",    XawChainTop}
 };
@@ -68,35 +88,40 @@ static Arg queue_args[] = {
   {"editType",  XawtextEdit},
   {"string",    (XtArgVal)""},            /* dummy to get it going */
   {"scrollVertical", XawtextScrollAlways},
-  {"right",     XawChainRight}, 
+  {"right",     XawChainRight},
   {"top",       XawChainTop},
   {"bottom",    XawChainBottom}
 };
 
-static Arg get_size_args[] = {
-  {"width",     (XtArgVal)NULL},
-  {"height",    (XtArgVal)NULL}
-};
-
 static Arg get_sizepos_args[] = {
   {"width",     (XtArgVal)NULL},
   {"height",    (XtArgVal)NULL},
   {"x",         (XtArgVal)NULL},
   {"y",         (XtArgVal)NULL}
-};    
+};
 
 XtActionsRec menu_action_table[] = {
   { "menu-create",  menu_create } };
 
+/* Types of non-message dialog action */
+
+enum { da_hide };
+
 /* Miscellaneous local variables */
 
+static int dialog_action;
 static int tick_stripchart_accumulator = 999999;
 static int tick_interval = 2;
 static int screenwidth, screenheight;
 static int minposset = 0;
+static int x_adjustment = -1;
+static int y_adjustment = -1;
+static Dimension original_x, original_y;
 static Dimension minposx, minposy;
 static Dimension maxwidth, maxheight;
 static Widget outer_form_widget;
+static Widget hide_widget;
+
 
 
 
@@ -129,13 +154,14 @@ va_end(ap);
 *************************************************/
 
 /* Operations on messages are done in subprocesses; this handler
-just catches them when they finish. */
+just catches them when they finish. It causes a queue display update
+unless configured not to. */
 
 static void sigchld_handler(int sig)
 {
 while (waitpid(-1, NULL, WNOHANG) > 0);
 signal(sig, sigchld_handler);
-tick_queue_accumulator = 999999;    /* Cause display update */
+if (action_queue_update) tick_queue_accumulator = 999999;
 }
 
 
@@ -152,6 +178,33 @@ queue_display();
 tick_queue_accumulator = 0;
 }
 
+void hideAction(Widget w, XtPointer client_data, XtPointer call_data)
+{
+actioned_message[0] = 0;
+dialog_ref_widget = w;
+dialog_action = da_hide;
+create_dialog("Hide addresses ending with", "");
+}
+
+void unhideAction(Widget w, XtPointer client_data, XtPointer call_data)
+{
+skip_item *sk = queue_skip;
+while (sk != NULL)
+  {
+  skip_item *next = sk->next;
+  free(sk);
+  sk = next;
+  }
+queue_skip = NULL;
+
+XtDestroyWidget(unhide_widget);
+unhide_widget = NULL;
+
+scan_spool_input(TRUE, TRUE);
+queue_display();
+tick_queue_accumulator = 0;
+}
+
 void quitAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
 exit(0);
@@ -160,9 +213,22 @@ exit(0);
 
 /* Action when the "Size" button is pressed. This is a kludged up mess
 that I made work after much messing around. Reading the position of the
-toplevel widget gets the absolute position of the data portion of the window, 
+toplevel widget gets the absolute position of the data portion of the window,
 excluding the window manager's furniture. However, positioning the toplevel
-widget's window seems to position the top corner of the furniture. */
+widget's window seems to position the top corner of the furniture under the twm
+window manager, but not under fwvm and others. The two cases are distinguished
+by the values of x_adjustment and y_adjustment.
+
+For twm (adjustment >= 0), one has to fudge the miminizing function to ensure
+that we go back to exactly the same position as before.
+
+For fwvm (adjustment < 0), one has to fudge the "top left hand corner"
+positioning to ensure that the window manager's furniture gets displayed on the
+screen. I haven't found a way of discovering the thickness of the furniture, so
+some screwed-in values are used.
+
+This is all ad hoc, developed by floundering around as I haven't found any
+documentation that tells me what I really should do. */
 
 void resizeAction(Widget button, XtPointer client_data, XtPointer call_data)
 {
@@ -180,54 +246,106 @@ get_sizepos_args[3].value = (XtArgVal)(&y);
 XtGetValues(toplevel_widget, get_sizepos_args, 4);
 
 /* Get the position of the widget's window relative to its parent; this
-gives the thickness of the window manager's furniture. */
+gives the thickness of the window manager's furniture. At least it does
+in twm. For fwvm it gives zero. The size/movement function uses this data.
+I tried doing this before entering the main loop, but it didn't always
+work properly. Running it every time seems to be OK. */
 
 XGetWindowAttributes(X_display, XtWindow(toplevel_widget), &a);
+if (a.x != 0) x_adjustment = a.x;
+if (a.y != 0) y_adjustment = a.y;
 
 /* If at maximum size, reduce to minimum and move back to where it was
-when maximized, if that value is set, allowing for the furniture. */
+when maximized, if that value is set, allowing for the furniture in cases
+where the positioning includes the furniture. */
 
 if (width == maxwidth && height == maxheight)
   {
   if (minposset)
-    { 
-    XMoveResizeWindow(X_display, w, minposx - a.x, minposy - a.y, 
+    {
+    XMoveResizeWindow(X_display, w,
+      minposx - ((x_adjustment >= 0)? x_adjustment : 0),
+      minposy - ((y_adjustment >= 0)? y_adjustment : 0),
       min_width, min_height);
     minposset = 0;
-    }  
-  else XResizeWindow(X_display, w, min_width, min_height);    
+    }
+  else XResizeWindow(X_display, w, min_width, min_height);
   }
-  
-/* Else always expand to maximum. If currently at minimum size, and we have to 
-move it, remember where it was for coming back. */
- 
+
+/* Else always expand to maximum. If currently at minimum size, and we have to
+move it, remember where it was for coming back. If we don't have a value
+for the thickness of the furniture, the implication is that the coordinates
+position the application window, so we can't use (0,0) because that loses the
+furniture. Use screwed in values that seem to work with fvwm. */
+
 else
-  {       
+  {
   int xx = x;
-  int yy = y;  
+  int yy = y;
   if (x + maxwidth > screenwidth || y + maxheight + 10 > screenheight)
     {
-    if (x + maxwidth > screenwidth) xx = 0;
-    if (y + maxheight + 10 > screenheight) yy = 0; 
     if (width == min_width && height == min_height)
       {
       minposx = x;
       minposy = y;
-      minposset = 1; 
-      }    
+      minposset = 1;
+      }
+    if (x + maxwidth > screenwidth) xx = (x_adjustment >= 0)? 0 : 4;
+    if (y + maxheight + 10 > screenheight) yy = (y_adjustment >= 0)? 0 : 21;
     XMoveResizeWindow(X_display, w, xx, yy, maxwidth, maxheight);
     }
   else XResizeWindow(X_display, w, maxwidth, maxheight);
-  }    
-  
+  }
+
 /* Ensure the window is at the top */
 
-XRaiseWindow(X_display, w); 
+XRaiseWindow(X_display, w);
 }
 
 
 
 
+/*************************************************
+*          Handle input from non-msg dialogue    *
+*************************************************/
+
+/* The various cases here are: hide domain, (no more yet) */
+
+void NonMessageDialogue(char *s)
+{
+skip_item *sk;
+
+switch(dialog_action)
+  {
+  case da_hide:
+
+  /* Create the unhide button if not present */
+
+  if (unhide_widget == NULL)
+    {
+    unhide_args[0].value = (XtArgVal) log_widget;
+    unhide_args[1].value = (XtArgVal) hide_widget;
+    unhide_widget = XtCreateManagedWidget("unhide", commandWidgetClass,
+      outer_form_widget, unhide_args, XtNumber(unhide_args));
+    XtAddCallback(unhide_widget, "callback",  unhideAction, NULL);
+    }
+
+  /* Add item to skip queue */
+
+  sk = (skip_item *)malloc(sizeof(skip_item) + (int)strlen(s));
+  sk->next = queue_skip;
+  queue_skip = sk;
+  strcpy(sk->text, s);
+  sk->reveal = time(NULL) + 60 * 60;
+  scan_spool_input(TRUE, TRUE);
+  queue_display();
+  tick_queue_accumulator = 0;
+  break;
+  }
+}
+
+
+
 /*************************************************
 *              Regexp error handler              *
 *************************************************/
@@ -294,33 +412,34 @@ else if (tick_stripchart_accumulator >= stripchart_update)
   scan_spool_input(FALSE, FALSE);
   tick_stripchart_accumulator = 0;
   }
-  
+
 /* Scan any pipes that are set up for listening to delivery processes,
 and display their output if their windows are still open. */
 
 while (p != NULL)
   {
-  int count; 
+  int count;
   char buffer[256];
-   
+
   while ((count = read(p->fd, buffer, 254)) > 0)
     {
-    buffer[count] = 0;  
-    if (p->widget != NULL) text_show(p->widget, "%s", buffer); 
-    } 
-     
+    buffer[count] = 0;
+    if (p->widget != NULL) text_show(p->widget, buffer);
+    }
+
   if (count == 0)
     {
     close(p->fd);
     *pp = p->next;
-    free(p);  
-    tick_queue_accumulator = 999999;    /* Cause display update */
-    }   
-     
+    free(p);
+    /* If configured, cause display update */
+    if (action_queue_update) tick_queue_accumulator = 999999;
+    }
+
   else pp = &(p->next);
-   
+
   p = *pp;
-  }  
+  }
 
 /* Reset the timer for next time */
 
@@ -333,7 +452,7 @@ void action_hook(Widget w, XtPointer client_data, String name,
   XEvent *event, String *string, Cardinal *n)
 {
 printf("action hook, name = %s Widget=%d queue_widget=%d\n",
-  name, w, queue_widget);
+  name, (int)w, (int)queue_widget);
 }
 
 
@@ -346,23 +465,23 @@ printf("action hook, name = %s Widget=%d queue_widget=%d\n",
 
 int main(int argc, char **argv)
 {
-int i; 
+int i;
 struct stat statdata;
 Widget stripchart_form_widget,
        update_widget,
        quit_widget,
        resize_widget;
-       
+
 
 /* The exim global message_id needs to get set */
 
-message_id = message_id_external + 1; 
+message_id_external = message_id_option + 1;
+message_id = message_id_external + 1;
 
 /* Some store needs getting for big_buffer, which is used for
-constructing file names and things. It need not, however, be as big
-as it is for exim itself, where it may have to hold an enormous header. */
+constructing file names and things. */
 
-big_buffer = (char *)malloc(512);
+big_buffer = (char *)malloc(1024);
 
 /* Set up the version string and date and output them */
 
@@ -405,9 +524,9 @@ else
   {
   fseek(LOG, 0, SEEK_END);
   log_position = ftell(LOG);
-  stat(log_file, &statdata);
+  fstat(fileno(LOG), &statdata);
   log_inode = statdata.st_ino;
-  } 
+  }
 
 
 /* Now initialize the X world and create the top-level widget */
@@ -417,7 +536,7 @@ toplevel_widget = XtAppInitialize(&X_appcon, "Eximon", NULL, 0, &argc, argv,
 X_display = XtDisplay(toplevel_widget);
 xs_SetValues(toplevel_widget, 4,
   "title",     window_title,
-  "iconName", "eximon",
+  "iconName",  window_title,
   "minWidth",  min_width,
   "minHeight", min_height);
 
@@ -437,13 +556,14 @@ for popping up the menu. Note that the order of things in these
 tables is significant. Shift<thing> must come before <thing> as
 otherwise it isn't noticed. */
 
-/* Do we want these ?
-  <Key>R: search(backward)
-   Ctrl<Key>S: search(forward)
+/*
+   <FocusIn>:      display-caret(on)\n\
+   <FocusOut>:     display-caret(off)\n\
 */
 
-queue_trans = XtParseTranslationTable(
-  "Shift<Btn1Down>:menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
+
+sprintf(big_buffer,
+   "%s:   menu-create() XawPositionSimpleMenu(menu) MenuPopup(menu)\n\
    <Btn1Down>:     select-start()\n\
    <Btn1Motion>:   extend-adjust()\n\
    <Btn1Up>:       extend-end(PRIMARY,CUT_BUFFER0)\n\
@@ -452,7 +572,11 @@ queue_trans = XtParseTranslationTable(
    <Btn3Up>:       extend-end(PRIMARY,CUT_BUFFER0)\n\
    <Key>Up:        scroll-one-line-down()\n\
    <Key>Down:      scroll-one-line-up()\n\
-  ");
+   Ctrl<Key>R:     search(backward)\n\
+   Ctrl<Key>S:     search(forward)\n\
+   ", menu_event);
+
+queue_trans = XtParseTranslationTable(big_buffer);
 
 text_trans = XtParseTranslationTable(
   "<Btn1Down>:     select-start()\n\
@@ -463,6 +587,8 @@ text_trans = XtParseTranslationTable(
    <Btn3Up>:       extend-end(PRIMARY,CUT_BUFFER0)\n\
    <Key>Up:        scroll-one-line-down()\n\
    <Key>Down:      scroll-one-line-up()\n\
+   Ctrl<Key>R:     search(backward)\n\
+   Ctrl<Key>S:     search(forward)\n\
   ");
 
 
@@ -491,14 +617,14 @@ only if the OS supports statfs(). */
 
 if (size_stripchart != NULL)
   {
-#ifdef HAVE_STATFS   
+#ifdef HAVE_STATFS
   char *name = size_stripchart + (int)strlen(size_stripchart) - 1;
   while (name > size_stripchart && *name == '/') name--;
   while (name > size_stripchart && *name != '/') name--;
   create_stripchart(stripchart_form_widget, name);
 #else
   printf("Can't create size stripchart: statfs() function not available\n");
-#endif   
+#endif
   }
 
 /* Now create the configured input/output stripcharts; note
@@ -525,8 +651,9 @@ XtAddCallback(resize_widget, "callback",  resizeAction, NULL);
 log_args[0].value = (XtArgVal) quit_widget;
 log_widget = XtCreateManagedWidget("log", asciiTextWidgetClass,
   outer_form_widget, log_args, XtNumber(log_args));
-XawTextDisplayCaret(log_widget, FALSE);
-xs_SetValues(log_widget, 5,
+XawTextDisplayCaret(log_widget, TRUE);
+xs_SetValues(log_widget, 6,
+  "editType",  XawtextAppend,
   "translations", text_trans,
   "string",    log_display_buffer,
   "length",    log_buffer_size,
@@ -547,15 +674,23 @@ update_widget = XtCreateManagedWidget("update", commandWidgetClass,
   outer_form_widget, update_args, XtNumber(update_args));
 XtAddCallback(update_widget, "callback",  updateAction, NULL);
 
+/* The hide button */
+
+hide_args[0].value = (XtArgVal) log_widget;
+hide_args[1].value = (XtArgVal) update_widget;
+hide_widget = XtCreateManagedWidget("hide", commandWidgetClass,
+  outer_form_widget, hide_args, XtNumber(hide_args));
+XtAddCallback(hide_widget, "callback",  hideAction, NULL);
 
 /* Create an Ascii text widget for the queue display. */
 
 queue_args[0].value = (XtArgVal) update_widget;
 queue_widget = XtCreateManagedWidget("queue", asciiTextWidgetClass,
   outer_form_widget, queue_args, XtNumber(queue_args));
-XawTextDisplayCaret(queue_widget, FALSE);
+XawTextDisplayCaret(queue_widget, TRUE);
 
-xs_SetValues(queue_widget, 3,
+xs_SetValues(queue_widget, 4,
+  "editType",  XawtextAppend,
   "height",    queue_depth,
   "width",     queue_width,
   "translations", queue_trans);
@@ -582,16 +717,18 @@ printf("\nExim Monitor running\n");
 XtRealizeWidget(toplevel_widget);
 
 /* Find out the size of the initial window, and set that as its
-maximum. */
+maximum. While we are at it, get the initial position. */
 
-get_size_args[0].value = (XtArgVal)(&maxwidth);
-get_size_args[1].value = (XtArgVal)(&maxheight);
-XtGetValues(toplevel_widget, get_size_args, 2);
+get_sizepos_args[0].value = (XtArgVal)(&maxwidth);
+get_sizepos_args[1].value = (XtArgVal)(&maxheight);
+get_sizepos_args[2].value = (XtArgVal)(&original_x);
+get_sizepos_args[3].value = (XtArgVal)(&original_y);
+XtGetValues(toplevel_widget, get_sizepos_args, 4);
 
 xs_SetValues(toplevel_widget, 2,
   "maxWidth",  maxwidth,
   "maxHeight", maxheight);
-  
+
 /* Set up the size of the screen */
 
 screenwidth = XDisplayWidth(X_display, 0);
@@ -611,3 +748,4 @@ return 0;
 }
 
 /* End of em_main.c */
+
