@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996. */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -18,24 +18,21 @@ static Widget text_create(char *, int);
 
 static int highlighted_start, highlighted_end, highlighted_x, highlighted_y;
 
-static char actioned_message[24];
-static char *action_required;
-
 
 
 static Arg queue_get_arg[] = {
   { "textSink",   (XtArgVal)NULL },
   { "textSource", (XtArgVal)NULL },
   { "string",     (XtArgVal)NULL } };
-  
+
 static Arg dialog_arg[] = {
   { "label",      (XtArgVal)"dialog" },
   { "value",      (XtArgVal)"value" } };
-  
+
 static Arg get_pos_args[] = {
   {"x",           (XtArgVal)NULL },
   {"y",           (XtArgVal)NULL } };
-   
+
 static Arg menushell_arg[] = {
   { "label",      (XtArgVal)NULL } };
 
@@ -46,13 +43,13 @@ static Arg button_arg[] = {
   { "right",     XawChainLeft },
   { "top",       XawChainBottom },
   { "bottom",    XawChainBottom } };
-  
+
 static Arg text_arg[] = {
   { XtNfromVert, (XtArgVal) NULL },         /* must be first */
   { "editType",  XawtextEdit },
   { "string",    (XtArgVal)"" },            /* dummy to get it going */
-  { "scrollVertical", XawtextScrollWhenNeeded },
-  { "wrap",      XawtextWrapWord }, 
+  { "scrollVertical", XawtextScrollAlways },
+  { "wrap",      XawtextWrapWord },
   { "top",       XawChainTop },
   { "bottom",    XawChainBottom } };
 
@@ -87,10 +84,10 @@ static Arg item_7_arg[] = {
 static Arg item_8_arg[] = {
   { XtNfromVert,  (XtArgVal) NULL },        /* must be first */
   { "label",      (XtArgVal)" Remove message" } };
-  
+
 static Arg item_9_arg[] = {
   { XtNfromVert,  (XtArgVal) NULL },        /* must be first */
-  { "label",      (XtArgVal)"----------------" } };  
+  { "label",      (XtArgVal)"----------------" } };
 
 static Arg item_10_arg[] = {
   { XtNfromVert,  (XtArgVal) NULL },        /* must be first */
@@ -102,9 +99,13 @@ static Arg item_11_arg[] = {
 
 static Arg item_12_arg[] = {
   { XtNfromVert,  (XtArgVal) NULL },        /* must be first */
-  { "label",      (XtArgVal)" Edit sender" } };
+  { "label",      (XtArgVal)" Mark all delivered" } };
 
 static Arg item_13_arg[] = {
+  { XtNfromVert,  (XtArgVal) NULL },        /* must be first */
+  { "label",      (XtArgVal)" Edit sender" } };
+
+static Arg item_14_arg[] = {
   { XtNfromVert,  (XtArgVal) NULL },        /* must be first */
   { "label",      (XtArgVal)" Edit body" } };
 
@@ -117,9 +118,10 @@ static Arg item_13_arg[] = {
 static void popdownAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
 XawTextSinkDisplayText(queue_text_sink,
-  highlighted_x, highlighted_y, 
+  highlighted_x, highlighted_y,
   highlighted_start, highlighted_end, 0);
 XtDestroyWidget(w);
+menu_is_up = FALSE;
 }
 
 
@@ -137,13 +139,12 @@ sprintf(buffer, "%s/msglog/%s", spool_directory, (char *)client_data);
 
 f = fopen(buffer, "r");
 if (f == NULL)
-  text_show(text, "%s: %s\n", buffer, strerror(errno));
-else 
+  text_showf(text, "%s: %s\n", buffer, strerror(errno));
+else
   {
-  while (fgets(buffer, 256, f) != NULL)
-    text_show(text, "%s", buffer);
-  fclose(f);    
-  }  
+  while (fgets(buffer, 256, f) != NULL) text_show(text, buffer);
+  fclose(f);
+  }
 }
 
 
@@ -161,13 +162,22 @@ sprintf(buffer, "%s/input/%s-D", spool_directory, (char *)client_data);
 
 f = fopen(buffer, "r");
 if (f == NULL)
-  text_show(text, "Failed to open file: %s\n", strerror(errno));
-else 
+  text_showf(text, "Failed to open file: %s\n", strerror(errno));
+else
   {
+  int count = 0;
   while (fgets(buffer, 256, f) != NULL)
-    text_show(text, "%s", buffer);
-  fclose(f);    
-  }  
+    {
+    text_show(text, buffer);
+    count += (int)strlen(buffer);
+    if (count > body_max)
+      {
+      text_show(text, "\n*** Message length exceeds BODY_MAX ***\n");
+      break;
+      }
+    }
+  fclose(f);
+  }
 }
 
 
@@ -176,9 +186,9 @@ else
 *        Do something to a message               *
 *************************************************/
 
-/* The output is not shown in a window for non-delivery actions that succeed, 
+/* The output is not shown in a window for non-delivery actions that succeed,
 unless action_output is set. We can't, however, tell until we have run
-the command whether we want the output or not, so the pipe has to be set up in 
+the command whether we want the output or not, so the pipe has to be set up in
 all cases. */
 
 static void ActOnMessage(char *id, char *action, char *address_arg)
@@ -188,7 +198,10 @@ int pipe_fd[2];
 int delivery = strcmp(action + (int)strlen(action) - 2, "-M") == 0;
 int editing = strcmp(action + (int)strlen(action) - 4, "-Meb") == 0;
 char *quote = "";
+char *at = "";
+char *qualify = "";
 char buffer[256];
+queue_item *qq;
 Widget text = NULL;
 
 
@@ -199,43 +212,57 @@ when it is finished. */
 if (editing)
   {
   if ((pid = fork()) == 0)
-    { 
+    {
     sprintf(buffer, "xterm -n %s -title %s -e %s %s %s", id, id,
-      exim_path, action, id); 
+      exim_path, action, id);
     system(buffer);
     _exit(0);
-    } 
+    }
   return;
-  } 
-  
+  }
 
-/* The command for everything else is the same */
 
-if (address_arg[0] != 0) quote = "\'";
-sprintf(buffer, "%s %s %s %s%s%s", exim_path, action, id, 
-  quote, address_arg, quote);
+/* The command for everything else is the same; if the address arg is not empty
+and does not contain @ and there is a qualify domain, qualify it. */
+
+if (address_arg[0] != 0)
+  {
+  quote = "\'";
+  if (strchr(address_arg, '@') == NULL && qualify_domain != NULL &&
+      qualify_domain[0] != 0)
+    {
+    at = "@";
+    qualify = qualify_domain;
+    }
+  }
+sprintf(buffer, "%s %s %s %s%s%s%s%s", exim_path, action, id,
+  quote, address_arg, at, qualify, quote);
 
 /* If we know we are going to need the window, create it now. */
 
 if (action_output || delivery)
-  { 
+  {
   text = text_create(id, 200);
-  text_show(text, "%s\n", buffer);
-  } 
+  text_showf(text, "%s\n", buffer);
+  }
+
+/* Create the pipe for output. Remember, on most systems pipe[0] is
+for reading and pipe[1] is for writing! Solaris, with its two-way
+pipes is a trap! */
 
 if (pipe(pipe_fd) != 0)
   {
   if (text == NULL)
-    { 
+    {
     text = text_create(id, 200);
-    text_show(text, "%s\n", buffer);
-    } 
-  text_show(text, "*** Failed to create pipe ***\n"); 
+    text_showf(text, "%s\n", buffer);
+    }
+  text_show(text, "*** Failed to create pipe ***\n");
   return;
-  }  
+  }
 
-fcntl(pipe_fd[0], F_SETFL, O_NONBLOCK); 
-fcntl(pipe_fd[1], F_SETFL, O_NONBLOCK); 
+fcntl(pipe_fd[0], F_SETFL, O_NONBLOCK);
+fcntl(pipe_fd[1], F_SETFL, O_NONBLOCK);
 
 /* Delivering a message can take some time, and we want to show the
 output as it goes along. This requires subprocesses and is coded below. For
@@ -245,86 +272,93 @@ output at all. */
 
 if (!delivery)
   {
-  int count, rc; 
-  int save_stdout = dup(1); 
+  int count, rc;
+  int save_stdout = dup(1);
   int save_stderr = dup(2);
 
   close(1);
   close(2);
-   
-  dup2(pipe_fd[0], 1);
-  dup2(pipe_fd[0], 2);
-  close(pipe_fd[0]);     
+
+  dup2(pipe_fd[1], 1);
+  dup2(pipe_fd[1], 2);
+  close(pipe_fd[1]);
 
   rc = system(buffer);
-  
+
   close(1);
   close(2);
-    
-  if (action_output || rc != 0) 
-    { 
+
+  if (action_output || rc != 0)
+    {
     if (text == NULL)
-      { 
-      text = text_create(id, 200);
-      text_show(text, "%s\n", buffer);
-      } 
-    while ((count = read(pipe_fd[1], buffer, 254)) > 0)
       {
-      buffer[count] = 0;  
-      text_show(text, "%s", buffer); 
+      text = text_create(id, 200);
+      text_showf(text, "%s\n", buffer);
+      }
+    while ((count = read(pipe_fd[0], buffer, 254)) > 0)
+      {
+      buffer[count] = 0;
+      text_show(text, buffer);
       }
     }
 
-  close(pipe_fd[1]);
-  
+  close(pipe_fd[0]);
+
   dup2(save_stdout, 1);
   dup2(save_stderr, 2);
   close(save_stdout);
-  close(save_stderr);   
-  
+  close(save_stderr);
+
   /* If action was to change the sender, and it succeeded, we have to
   update the in-store data. */
-  
+
   if (rc == 0 && strcmp(action + (int)strlen(action) - 4, "-Mes") == 0)
     {
     queue_item *q = find_queue(id, queue_noop);
     if (q != NULL) q->sender = string_copy(address_arg);
-    }  
-    
-  /* Cause a display update and return */
-   
-  tick_queue_accumulator = 999999;
-  return; 
+    }
+
+  /* If configured, cause a display update and return */
+
+  if (action_queue_update) tick_queue_accumulator = 999999;
+  return;
   }
- 
+
+/* Message is to be delivered. Ensure that it is marked unfrozen,
+because nothing will get written to the log to show that this has
+happened. (Other freezing/unfreezings get logged and picked up from
+there.) */
+
+qq = find_queue(id, queue_noop);
+if (qq != NULL) qq->frozen = FALSE;
 
 /* New, asynchronous code runs in a subprocess for commands that
 will take some time. The main process does not wait. There is a
 SIGCHLD handler in the main program that cleans up any terminating
 sub processes. */
- 
+
 if ((pid = fork()) == 0)
   {
   close(1);
   close(2);
-   
-  dup2(pipe_fd[0], 1);
-  dup2(pipe_fd[0], 2);
-  close(pipe_fd[0]);     
-  
+
+  dup2(pipe_fd[1], 1);
+  dup2(pipe_fd[1], 2);
+  close(pipe_fd[1]);
+
   system(buffer);
-  
+
   close(1);
   close(2);
-  close(pipe_fd[1]);
+  close(pipe_fd[0]);
   _exit(0);
-  } 
+  }
 
 /* Main process - set up an item for the main ticker to watch. */
 
-if (pid < 0) text_show(text, "Failed to fork: %s\n", strerror(pid)); else
+if (pid < 0) text_showf(text, "Failed to fork: %s\n", strerror(pid)); else
   {
-  pipe_item *p = (pipe_item *)malloc(sizeof(pipe_item)); 
+  pipe_item *p = (pipe_item *)malloc(sizeof(pipe_item));
 
   if (p == NULL)
     {
@@ -333,13 +367,13 @@ if (pid < 0) text_show(text, "Failed to fork: %s\n", strerror(pid)); else
     }
 
   p->widget = text;
-  p->fd = pipe_fd[1]; 
-    
+  p->fd = pipe_fd[0];
+
   p->next = pipe_chain;
   pipe_chain = p;
-   
-  close(pipe_fd[0]); 
-  } 
+
+  close(pipe_fd[1]);
+  }
 }
 
 
@@ -393,7 +427,14 @@ s = XawDialogGetValueString(dialog_widget);
 XtPopdown((Widget)dialog_shell);
 XtDestroyWidget((Widget)dialog_shell);
 while (isspace(*s)) s++;
-if (s[0] != 0) ActOnMessage(actioned_message, action_required, s);
+if (s[0] != 0)
+  {
+  if (actioned_message[0] != 0)
+    ActOnMessage(actioned_message, action_required, s);
+  else
+    NonMessageDialogue(s);    /* When called from somewhere else */
+  }
+return NULL;
 }
 
 
@@ -403,21 +444,35 @@ if (s[0] != 0) ActOnMessage(actioned_message, action_required, s);
 *************************************************/
 
 /* The focus is grabbed exclusively, so nothing else can
-be done to the application until the box is filled in. */
+be done to the application until the box is filled in. This
+function is also used by the Hide button handler. */
 
-static void create_dialog(char *label, char *value)
+void create_dialog(char *label, char *value)
 {
 Arg warg[4];
-Dimension x, y;
+Dimension x, y, xx, yy;
 XtTranslations pop_trans;
 Widget text;
 
-/* Get the position of the menu so the dialog box can be put
+/* Get the position of a reference widget so the dialog box can be put
 near to it. */
 
 get_pos_args[0].value = (XtArgVal)(&x);
 get_pos_args[1].value = (XtArgVal)(&y);
-XtGetValues(menushell, get_pos_args, 2);
+XtGetValues(dialog_ref_widget, get_pos_args, 2);
+
+/* When this is not a message_specific thing, the position of the
+reference widget is relative to the window. Get the position of the top level
+widget and add to the position. */
+
+if (dialog_ref_widget != menushell)
+  {
+  get_pos_args[0].value = (XtArgVal)(&xx);
+  get_pos_args[1].value = (XtArgVal)(&yy);
+  XtGetValues(toplevel_widget, get_pos_args, 2);
+  x += xx;
+  y += yy;
+  }
 
 /* Create a transient shell for the dialog box. */
 
@@ -425,11 +480,11 @@ XtSetArg(warg[0], XtNtransientFor, queue_widget);
 XtSetArg(warg[1], XtNx, x + 50);
 XtSetArg(warg[2], XtNy, y + 50);
 XtSetArg(warg[3], XtNallowShellResize, True);
-dialog_shell = XtCreatePopupShell("forDialog", transientShellWidgetClass,          
+dialog_shell = XtCreatePopupShell("forDialog", transientShellWidgetClass,
    toplevel_widget, warg, 4);
 
 /* Create the dialog box. */
-   
+
 dialog_arg[0].value = (XtArgVal)label;
 dialog_arg[1].value = (XtArgVal)value;
 dialog_widget = XtCreateManagedWidget("dialog", dialogWidgetClass, dialog_shell,
@@ -467,6 +522,7 @@ static void addrecipAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
 strcpy(actioned_message, (char *)client_data);
 action_required = "-Mar";
+dialog_ref_widget = menushell;
 create_dialog("Recipient address to add?", "");
 }
 
@@ -480,15 +536,26 @@ static void markdelAction(Widget w, XtPointer client_data, XtPointer call_data)
 {
 strcpy(actioned_message, (char *)client_data);
 action_required = "-Mmd";
+dialog_ref_widget = menushell;
 create_dialog("Recipient address to mark delivered?", "");
 }
 
 
+/*************************************************
+*   Cause all addresses to be marked delivered   *
+*************************************************/
+
+static void markalldelAction(Widget w, XtPointer client_data, XtPointer call_data)
+{
+ActOnMessage((char *)client_data, "-Mmad", "");
+}
+
+
 /*************************************************
 *        Edit the message's sender               *
 *************************************************/
 
-static void editsenderAction(Widget w, XtPointer client_data, 
+static void editsenderAction(Widget w, XtPointer client_data,
   XtPointer call_data)
 {
 queue_item *q;
@@ -497,6 +564,7 @@ strcpy(actioned_message, (char *)client_data);
 q = find_queue(actioned_message, queue_noop);
 sender = (q == NULL)? "" : q->sender;
 action_required = "-Mes";
+dialog_ref_widget = menushell;
 create_dialog("New sender address?", sender);
 }
 
@@ -553,28 +621,28 @@ if (spool_read_header(buffer, TRUE) != spool_read_OK)
     struct stat statbuf;
     sprintf(big_buffer, "%s/input/%s", spool_directory, buffer);
     if (stat(big_buffer, &statbuf) == 0)
-      text_show(text, "Format error in spool file %s: size=%d\n", buffer,
-        statbuf.st_size);   
-    else text_show(text, "Format error in spool file %s\n", buffer);   
-    } 
-  else text_show(text, "Read error for spool file %s\n", buffer);
-  return; 
-  } 
+      text_showf(text, "Format error in spool file %s: size=%d\n", buffer,
+        statbuf.st_size);
+    else text_showf(text, "Format error in spool file %s\n", buffer);
+    }
+  else text_showf(text, "Read error for spool file %s\n", buffer);
+  return;
+  }
 
 if (sender_address != NULL)
   {
-  text_show(text, "%s sender: <%s>\n", sender_local? "Local" : "Remote",
+  text_showf(text, "%s sender: <%s>\n", sender_local? "Local" : "Remote",
     sender_address);
   free(sender_address);
   }
 
 if (recipients_list != NULL)
   {
-  int i; 
-  text_show(text, "Recipients:\n"); 
+  int i;
+  text_show(text, "Recipients:\n");
   for (i = 0; i < recipients_count; i++)
     {
-    text_show(text, "  %s %s\n", 
+    text_showf(text, "  %s %s\n",
       (tree_search(tree_nonrecipients, recipients_list[i]) == NULL)? " ":"*",
         recipients_list[i]);
     free(recipients_list[i]);
@@ -587,10 +655,11 @@ tree_free(tree_nonrecipients);
 
 for (h = header_list; h != NULL; h = next)
   {
-  next = h->next; 
-  text_show(text, "%c %s", h->type, h->text);
-  free(h); 
-  } 
+  next = h->next;
+  text_showf(text, "%c ", h->type);   /* Don't push h->text through a %s */
+  text_show(text, h->text);           /* expansion as it may be v large */
+  free(h);
+  }
 }
 
 
@@ -616,12 +685,12 @@ while (p != NULL)
   {
   Widget pp = p->widget;
   while (pp != NULL)
-    { 
-    if (pp == client_data) { p->widget = NULL; return; }
+    {
+    if (pp == (Widget)client_data) { p->widget = NULL; return; }
     pp = XtParent(pp);
     }
   p = p->next;
-  }  
+  }
 }
 
 
@@ -639,14 +708,14 @@ toplevel window. */
 
 textshell = XtCreatePopupShell("textshell", topLevelShellWidgetClass,
   toplevel_widget, NULL, 0);
-xs_SetValues(textshell, 4, 
+xs_SetValues(textshell, 4,
   "title",     name,
   "iconName",  name,
-  "minWidth",  100, 
+  "minWidth",  100,
   "minHeight", 100);
 
 /* Create a form widget, containing the text widget and the
-dismiss button widget. */  
+dismiss button widget. */
 
 form = XtCreateManagedWidget("textform", formWidgetClass,
   textshell, NULL, 0);
@@ -654,11 +723,12 @@ xs_SetValues(form, 1, "defaultDistance", 8);
 
 text = XtCreateManagedWidget("texttext", asciiTextWidgetClass,
   form, text_arg, XtNumber(text_arg));
-xs_SetValues(text, 3,
+xs_SetValues(text, 4,
+  "editType",        XawtextAppend,
   "width",           700,
-  "height",          height, 
+  "height",          height,
   "translations",    text_trans);
-XawTextDisplayCaret(text, FALSE);
+XawTextDisplayCaret(text, TRUE);
 
 /* Use the same font as for the queue display */
 
@@ -667,7 +737,7 @@ if (queue_font != NULL)
   XFontStruct *f = XLoadQueryFont(X_display, queue_font);
   if (f != NULL) xs_SetValues(text, 1, "font", f);
   }
-  
+
 button_arg[0].value = (XtArgVal)text;
 button = XtCreateManagedWidget("dismiss", commandWidgetClass,
   form, button_arg, XtNumber(button_arg));
@@ -692,15 +762,13 @@ be called, and set up button 2 in the text widgets to call it. */
 
 void menu_create(Widget w, XEvent *event, String *actargs, Cardinal *count)
 {
-char buffer[512];
-Window dummy;
 int line;
 int i;
 XawTextPosition p;
 XtTranslations menu_trans;
-Widget src, popupform, menu_line, item_1, item_2, item_3, item_4,
-  item_5, item_6, item_7, item_8, item_9, item_10, item_11, 
-  item_12, item_13;
+Widget src, menu_line, item_1, item_2, item_3, item_4,
+  item_5, item_6, item_7, item_8, item_9, item_10, item_11,
+  item_12, item_13, item_14;
 char *s;
 
 /* Get the sink and source and the current text pointer */
@@ -725,12 +793,21 @@ while (i-- > 0)
   }
 
 /* Now pointing either at 0 or 1st char after \n, or very 1st char.
-If 0, the click was beyond the end of the data; ignore. If on a
+If 0, the click was beyond the end of the data; just set up a dummy
+menu. (Not easy to ignore as several actions are specified for the
+mouse click and it expects this one to set up a menu.) If on a
 continuation line, move back to the main line. */
 
-if (s[p] == 0) return;
+if (s[p] == 0)
+  {
+  menushell_arg[0].value = (XtArgVal)"No message selected";
+  menushell = XtCreatePopupShell("menu", simpleMenuWidgetClass,
+    queue_widget, menushell_arg, XtNumber(menushell_arg));
+  XtAddCallback(menushell, "popdownCallback", popdownAction, NULL);
+  return;
+  }
 
-while (p > 0 && s[p+5] == ' ')
+while (p > 0 && s[p+11] == ' ')
   {
   line--;
   p--;
@@ -739,7 +816,7 @@ while (p > 0 && s[p+5] == ' ')
 
 /* Now pointing at first character of a main line. */
 
-strncpy(message_id, s+p+5, MESSAGE_ID_LENGTH);
+strncpy(message_id, s+p+11, MESSAGE_ID_LENGTH);
 message_id[MESSAGE_ID_LENGTH] = 0;
 
 /* Highlight the line being menued, and save its parameters so that it
@@ -751,7 +828,7 @@ highlighted_x = 17;
 highlighted_y = line * XawTextSinkMaxHeight(queue_text_sink, 1) + 2;
 
 XawTextSinkDisplayText(queue_text_sink,
-  highlighted_x, highlighted_y, 
+  highlighted_x, highlighted_y,
   highlighted_start, highlighted_end, 1);
 
 /* Create the popup shell and the other widgets that comprise the menu.
@@ -761,7 +838,7 @@ menushell_arg[0].value = (XtArgVal)message_id;
 menushell = XtCreatePopupShell("menu", simpleMenuWidgetClass,
   queue_widget, menushell_arg, XtNumber(menushell_arg));
 XtAddCallback(menushell, "popdownCallback", popdownAction, NULL);
- 
+
 menu_trans = XtParseTranslationTable(
   "<EnterWindow>:   highlight()\n\
    <LeaveWindow>:   unhighlight()\n\
@@ -833,18 +910,25 @@ XtAddCallback(item_11, "callback",  markdelAction, (XtPointer)message_id);
 item_12_arg[0].value = (XtArgVal)item_11;
 item_12 = XtCreateManagedWidget("item12", smeBSBObjectClass, menushell,
   item_12_arg, XtNumber(item_12_arg));
-XtAddCallback(item_12, "callback",  editsenderAction, (XtPointer)message_id);
+XtAddCallback(item_12, "callback",  markalldelAction, (XtPointer)message_id);
 
 item_13_arg[0].value = (XtArgVal)item_12;
 item_13 = XtCreateManagedWidget("item13", smeBSBObjectClass, menushell,
   item_13_arg, XtNumber(item_13_arg));
-XtAddCallback(item_13, "callback",  editbodyAction, (XtPointer)message_id);
-
+XtAddCallback(item_13, "callback",  editsenderAction, (XtPointer)message_id);
 
+item_14_arg[0].value = (XtArgVal)item_13;
+item_14 = XtCreateManagedWidget("item14", smeBSBObjectClass, menushell,
+  item_14_arg, XtNumber(item_14_arg));
+XtAddCallback(item_14, "callback",  editbodyAction, (XtPointer)message_id);
 
 /* Arrange that the menu pops up with the first item selected. */
 
 xs_SetValues(menushell, 1, "popupOnEntry", item_1);
+
+/* Flag that the menu is up to suppress queue updates. */
+
+menu_is_up = TRUE;
 }
 
 /* End of em_menu.c */
