@@ -2,7 +2,7 @@
 *               Exim Monitor                     *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996. */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -40,23 +40,33 @@ XawTextSetInsertionPoint(w, position);
 
 
 /*************************************************
-*               Display text                     *
+*                 Display text                   *
 *************************************************/
 
-void text_show(Widget w, char *s, ...)
+void text_show(Widget w, char *s)
 {
-va_list ap;
 XawTextBlock b;
-char buffer[1024];
-va_start(ap, s);
-vsprintf(buffer, s, ap);
-va_end(ap);
 b.firstPos = 0;
-b.ptr = buffer;
+b.ptr = s;
 b.format = FMT8BIT;
-b.length = (int)strlen(buffer);
+b.length = (int)strlen(s);
 XawTextReplace(w, 999999, 999999, &b);
 XawTextSetInsertionPoint(w, 99999);
 }
 
+
+/*************************************************
+*           Display text from format             *
+*************************************************/
+
+void text_showf(Widget w, char *s, ...)
+{
+va_list ap;
+char buffer[1024];
+va_start(ap, s);
+vsprintf(buffer, s, ap);
+va_end(ap);
+text_show(w, buffer);
+}
+
 /* End of em_text.c */
