@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996. */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include <stdio.h>
@@ -11,6 +11,7 @@
 #include <time.h>
 
 #include "../regexp/regexp.h"
+#include <sys/types.h>
 
 #include "macros.h"
 #include "structs.h"
@@ -22,13 +23,13 @@ void version_init(void)
 int i = 0;
 char today[20];
 
-version_string = "0.12";
+version_string = "1.26";
 
 strcpy(today, __DATE__);
 if (today[4] == ' ') i = 1;
 today[3] = today[6] = '-';
 
-version_date = (char *)store_malloc(32);
+version_date = (char *)malloc(32);
 version_date[0] = 0;
 strncat(version_date, today+4+i, 3-i);
 strncat(version_date, today, 4);
