@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -23,9 +23,25 @@ process, or -1 if things go wrong.
 
 A new umask is supplied for the process, and an optional new uid and gid are
 also available. These are used by the queryprogram router to set an
-unprivileged id. */
+unprivileged id.
 
-int child_open(char **argv, char **envp, int newumask, int newuid, int newgid,
+Arguments:
+  argv        the argv for exec in the new process
+  envp        the envp for exec in the new process
+  newumask    umask to set in the new process
+  newuid      point to uid for the new process or NULL for no change
+  newgid      point to gid for the new process or NULL for no change
+  fdptr       pointer to int into which the fd of the stdin of the new process
+                is placed
+  outfd       a fd for the stdout and stderr of the new process; if -1 is given
+                fds to /dev/null are set up
+
+Returns:      the pid of the created process or -1 if anything has gone wrong;
+              (via fdptr) the fd of the standard input of the new process
+*/
+
+pid_t
+child_open(char **argv, char **envp, int newumask, uid_t *newuid, gid_t *newgid,
   int *fdptr, int outfd)
 {
 int pfd[2];
@@ -45,7 +61,7 @@ if (outfd == 0)
 
 if (pipe(pfd) == 0)
   {
-  int pid = vfork();
+  pid_t pid = fork();
 
   /* The child process makes the reading end of the pipe into the
   standard input, and outfd into the standard output and error,
@@ -59,13 +75,13 @@ if (pipe(pfd) == 0)
 
     for (fd = mac_maxfd; fd >= 0; fd--)
       if (fd != pfd[pipe_read] && fd != outfd) close(fd);
-       
+
     if (pfd[pipe_read] != 0)
       {
       dup2(pfd[pipe_read], 0);  /* Make the standard input */
       close(pfd[pipe_read]);
       }
-       
+
     if (outfd > 0)
       {
       if (outfd != 1) dup2(outfd, 1);
@@ -78,17 +94,25 @@ if (pipe(pfd) == 0)
 
     if (geteuid() != root_uid) mac_seteuid(root_uid);
 
-    /* Set the required environment, and exec */
+    /* Set the required environment. If changing uid, ensure that
+    SIGUSR1 is ignored, as the process won't have the privilege to
+    write to the process log. */
 
-    if (newgid >= 0) setgid(newgid);
-    if (newuid >= 0) setuid(newuid);
+    if (newgid != NULL) setgid(*newgid);
+    if (newuid != NULL)
+      {
+      signal(SIGUSR1, SIG_IGN);
+      setuid(*newuid);
+      }
     umask(newumask);
 
+    /* Now do the exec */
+
     if (envp == NULL) execv(argv[0], argv);
       else execve(argv[0], argv, envp);
 
     /* Failed to execv */
-    
+
     _exit(errno);         /* Note: must be _exit(), NOT exit() */
     }
 
@@ -103,26 +127,83 @@ if (pipe(pfd) == 0)
   else                    /* fork failed */
     {
     close(pfd[pipe_write]);
-    return -1;
+    return (pid_t)(-1);
     }
   }
-else return -1;
+else return (pid_t)(-1);
 }
 
 
 
+/*************************************************
+*           SIGALRM handler for child_close      *
+*************************************************/
+
+static volatile BOOL child_sigalrm_seen;
+
+static void
+child_alarm_handler(int sig)
+{
+child_sigalrm_seen = TRUE;
+}
+
+
 /*************************************************
 *           Close down child process             *
 *************************************************/
 
-/* Wait for the given process to finish. */
+/* Wait for the given process to finish. Called when there is only one child in
+existence, or when picking up any others en route doesn't matter.
 
-int child_close(int pid)
+Arguments
+  pid:      the pid to wait for
+  timeout:  maximum time to wait; 0 means for as long as it takes
+
+Returns:    >= 0          process terminated by exiting; value is process
+                            ending status
+            < 0 & > -256  process was terminated by a signal; value is the
+                            negation of the signal number
+            -256          timed out
+            -257          other error in wait(); errno still set
+*/
+
+int
+child_close(pid_t pid, int timeout)
 {
-int rc, status;
-while ((rc = wait(&status)) != pid && rc >= 0);
-return (status >> 8) & 255;
+int yield;
+
+if (timeout > 0)
+  {
+  child_sigalrm_seen = FALSE;
+  signal(SIGALRM, child_alarm_handler);
+  alarm(timeout);
+  }
+
+for(;;)
+  {
+  int status;
+  pid_t rc = wait(&status);
+  if (rc == pid)
+    {
+    int lowbyte = status & 255;
+    if (lowbyte == 0) yield = (status >> 8) & 255;
+      else yield = -lowbyte;
+    break;
+    }
+  if (rc < 0)
+    {
+    yield = (errno == EINTR && child_sigalrm_seen)? -256 : -257;
+    break;
+    }
+  }
+
+if (timeout > 0)
+  {
+  alarm(0);
+  signal(SIGALRM, SIG_IGN);
+  }
+
+return yield;
 }
 
 /* End of child.c */
-
