@@ -2,28 +2,37 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
-/* Private structure for the private options. */
+/* Header for the aliasfile director */
+
+/* Structure for the private options. */
 
 typedef struct {
-  void *handle;
   char *file;
-  char *directory; 
+  char *query;
+  char *home_dir;
+  char *current_dir;
   char *errors_to;
   char *expand_uid;
   char *expand_gid;
-  int   uid;
-  int   gid;
+  uid_t uid;
+  gid_t gid;
   int   modemask;
   int   search_type;
   int  *owners;
   int  *owngroups;
+  BOOL  uid_set;
+  BOOL  gid_set;
+  BOOL  initgroups;
+  BOOL  expand;
   BOOL  optional;
   BOOL  forbid_file;
   BOOL  forbid_pipe;
-  BOOL  freeze_missing_include; 
+  BOOL  freeze_missing_include;
+  BOOL  rewrite;
+  BOOL  skip_syntax_errors;
 } aliasfile_director_options_block;
 
 /* Data for reading the private options. */
@@ -35,12 +44,10 @@ extern int aliasfile_director_options_count;
 
 extern aliasfile_director_options_block aliasfile_director_option_defaults;
 
-/* The main, initialization, tidyup, and dependency check entry points for the
-director */
+/* The main and initialization entry points */
 
 extern int aliasfile_director_entry(director_instance *, address_item *,
-  address_item **, address_item **, address_item **, BOOL);
+  address_item **, address_item **, address_item **, address_item **, BOOL);
 extern void aliasfile_director_init(director_instance *);
-extern void aliasfile_director_tidyup(director_instance *);
 
 /* End of directors/aliasfile.h */
