@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -14,24 +14,36 @@
 /* Options specific to the forwardfile director. */
 
 optionlist forwardfile_director_options[] = {
-  { "*expand_group",      opt_stringptr,
+  { "*expand_group",      opt_stringptr | opt_hidden,
       (void *)(offsetof(forwardfile_director_options_block, expand_gid)) },
-  { "*expand_user",       opt_stringptr,
+  { "*expand_user",       opt_stringptr | opt_hidden,
       (void *)(offsetof(forwardfile_director_options_block, expand_uid)) },
-  { "check_local_user",   opt_bool,
-      (void *)(offsetof(forwardfile_director_options_block, check_local_user)) },
+  { "*set_group",         opt_bool | opt_hidden,
+      (void *)(offsetof(forwardfile_director_options_block, gid_set)) },
+  { "*set_user",          opt_bool | opt_hidden,
+      (void *)(offsetof(forwardfile_director_options_block, uid_set)) },
+  { "check_ancestor",     opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, check_ancestor)) },
   { "check_group",        opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, check_group)) },
-  { "directory",          opt_stringptr,
-      (void *)(offsetof(forwardfile_director_options_block, directory)) },
+  { "check_local_user",   opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, check_local_user)) },
+  { "current_directory",  opt_stringptr,
+      (void *)(offsetof(forwardfile_director_options_block, current_dir)) },
+  { "directory",          opt_stringptr | opt_hidden,
+      (void *)(offsetof(forwardfile_director_options_block, file_dir)) },
   { "errors_to",          opt_stringptr,
       (void *)(offsetof(forwardfile_director_options_block, errors_to)) },
   { "file",               opt_stringptr,
       (void *)(offsetof(forwardfile_director_options_block, file)) },
+  { "file_directory",     opt_stringptr,
+      (void *)(offsetof(forwardfile_director_options_block, file_dir)) },
   { "filter",             opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, filter)) },
   { "forbid_file",        opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, forbid_file)) },
+  { "forbid_filter_log",  opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, forbid_filter_log)) },
   { "forbid_include",     opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, forbid_include)) },
   { "forbid_pipe",        opt_bool,
@@ -42,14 +54,22 @@ optionlist forwardfile_director_options[] = {
       (void *)(offsetof(forwardfile_director_options_block, freeze_missing_include)) },
   { "group",              opt_expand_gid,
       (void *)(offsetof(forwardfile_director_options_block, gid)) },
+  { "home_directory",     opt_stringptr,
+      (void *)(offsetof(forwardfile_director_options_block, home_dir)) },
+  { "initgroups",         opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, initgroups)) },
   { "modemask",           opt_octint,
       (void *)(offsetof(forwardfile_director_options_block, modemask)) },
   { "owners",             opt_uidlist,
       (void *)(offsetof(forwardfile_director_options_block, owners)) },
   { "owngroups",          opt_gidlist,
       (void *)(offsetof(forwardfile_director_options_block, owngroups)) },
+  { "rewrite",            opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, rewrite)) },
   { "seteuid",            opt_bool,
       (void *)(offsetof(forwardfile_director_options_block, use_seteuid)) },
+  { "skip_syntax_errors", opt_bool,
+      (void *)(offsetof(forwardfile_director_options_block, skip_syntax_errors)) },
   { "user",               opt_expand_uid,
       (void *)(offsetof(forwardfile_director_options_block, uid)) }
 };
@@ -63,7 +83,9 @@ int forwardfile_director_options_count =
 /* Default private options block for the forwardfile director. */
 
 forwardfile_director_options_block forwardfile_director_option_defaults = {
-  NULL,     /* directory */
+  NULL,     /* file_dir */
+  NULL,     /* home_dir */
+  NULL,     /* current_dir */
   NULL,     /* file */
   NULL,     /* errors_to */
   NULL,     /* expand_uid */
@@ -73,15 +95,22 @@ forwardfile_director_options_block forwardfile_director_option_defaults = {
   022,      /* modemask */
   NULL,     /* owners */
   NULL,     /* owngroups */
+  FALSE,    /* uid_set */
+  FALSE,    /* gid_set */
+  FALSE,    /* check_ancestor */
   TRUE,     /* check_local_user */
-  FALSE,    /* check_group */ 
-  FALSE,    /* filter */ 
+  FALSE,    /* check_group */
+  FALSE,    /* filter */
+  FALSE,    /* forbid_filter_log */
   FALSE,    /* forbid_file */
-  FALSE,    /* forbid_include */ 
+  FALSE,    /* forbid_include */
   FALSE,    /* forbid_pipe */
-  FALSE,    /* forbid_reply */ 
+  FALSE,    /* forbid_reply */
+  FALSE,    /* initgroups */
   FALSE,    /* use_seteuid */
-  TRUE      /* freeze_missing_include */
+  TRUE,     /* freeze_missing_include */
+  TRUE,     /* rewrite */
+  FALSE     /* skip_syntax_errors */
 };
 
 
@@ -101,30 +130,30 @@ forwardfile_director_options_block *ob =
 
 /* If a fixed uid field is set, then a gid field must also be set. */
 
-if (ob->uid >= 0 && ob->gid < 0)
-  log_write(LOG_PANIC_DIE, "Exim configuration error:\n  "
+if (ob->uid_set && !ob->gid_set && ob->expand_gid == NULL)
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "user set without group for the %s director", dblock->name);
 
 /* A file name is mandatory */
 
 if (ob->file == NULL)
-  log_write(LOG_PANIC_DIE, "Exim configuration error for %s director:\n  "
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
     "no file name specified", dblock->name);
 
 /* A directory setting is optional, but if it exists it must be absolute,
 though we can't check for certain until it has been expanded. */
 
-if (ob->directory != NULL && ob->directory[0] != '/' &&
-    ob->directory[0] != '$')
-  log_write(LOG_PANIC_DIE, "Exim configuration error for %s director:\n  "
+if (ob->file_dir != NULL && ob->file_dir[0] != '/' &&
+    ob->file_dir[0] != '$')
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
     "absolute path name required for the 'directory' option", dblock->name);
 
 /* Permit relative paths only if local user checking is set, or if the
 directory option (which must be absolute) is set. */
 
 if (!ob->check_local_user && ob->file[0] != '/' && ob->file[0] != '$' &&
-     ob->directory == NULL)
-  log_write(LOG_PANIC_DIE, "Exim configuration error for %s director:\n  "
+     ob->file_dir == NULL)
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
     "absolute file path required when check_local_user and directory are not set",
     dblock->name);
 
@@ -132,19 +161,19 @@ if (!ob->check_local_user && ob->file[0] != '/' && ob->file[0] != '$' &&
 user checking is set, or uid/gid are supplied. */
 
 if (ob->use_seteuid && !have_seteuid)
-  log_write(LOG_PANIC_DIE, "Exim configuration error for %s director:\n  "
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
     "seteuid requested but not configured as available",
     dblock->name);
 
-if (ob->use_seteuid && !ob->check_local_user && ob->uid < 0 &&
+if (ob->use_seteuid && !ob->check_local_user && !ob->uid_set &&
     ob->expand_uid == NULL)
-  log_write(LOG_PANIC_DIE, "Exim configuration error for %x director:\n  "
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
     "seteuid requested without check_local_user or explicit uid", dblock->name);
 
 /* A transport must *not* be specified */
 
 if (dblock->transport != NULL)
-  log_write(LOG_PANIC_DIE, "Exim configuration error for %s director:\n  "
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG2, "%s director:\n  "
     "a transport is not allowed for this director", dblock->name);
 }
 
@@ -162,6 +191,7 @@ int forwardfile_director_entry(
   address_item **addr_local,      /* add it to this if it's local */
   address_item **addr_remote,     /* add it to this if it's remote */
   address_item **addr_new,        /* put new addresses on here */
+  address_item **addr_succeed,    /* put old address here on success */
   BOOL verify)                    /* TRUE when verifying */
 {
 forwardfile_director_options_block *ob =
@@ -172,37 +202,90 @@ char *errors_to = addr->errors_address;
 char *filename;
 char *filebuf;
 char *error;
+char *s, *tag;
 struct stat statbuf;
 struct passwd *pw;
+struct passwd *upw = NULL;
+error_block *eblock = NULL;
 FILE *fwd;
-int  uid = ob->uid;
-int  gid = ob->gid;
-int  yield = OK;
-int  saved_euid = -1;
-int  saved_egid;
-int  extracted;
-BOOL uid_ok = FALSE;
-BOOL gid_ok = !ob->check_group;
-BOOL restore_uid = FALSE;
-BOOL delivered = FALSE;
-BOOL is_filter = FALSE;
+uid_t uid = ob->uid;
+gid_t gid = ob->gid;
+int   yield = OK;
+int   saved_euid = -1;
+int   saved_egid = -1;
+int   extracted = 0;
+BOOL  uid_set = ob->uid_set;
+BOOL  gid_set = ob->gid_set;
+BOOL  uid_ok = FALSE;
+BOOL  gid_ok = !ob->check_group;
+BOOL  restore_uid = FALSE;
+BOOL  stat_directory = TRUE;
+BOOL  delivered = FALSE;
+BOOL  is_filter = FALSE;
+BOOL  filter_header = FALSE;
 
-/* Do file existence tests */
+/* If the check_local_user option is set, check that the local_part is
+the login of a local user, and fail if not. Note: the third argument to
+direct_finduser() must be NULL here, to prevent a numeric string being
+taken as a numeric uid. If the user is found, set directory to the home
+directory, and the home expansion variable as well, so that it can be
+used while expanding ob->file_dir! */
 
-if (!match_exists(dblock->require_files))
+if (ob->check_local_user)
   {
+  if (!direct_finduser(addr->local_part, &pw, NULL))
+    {
+    DEBUG(2) debug_printf("%s director failed for %s (not a user)\n",
+      dblock->name, addr->local_part);
+    return FAIL;
+    }
+  else
+    {
+    directory = pw->pw_dir;
+    deliver_home = directory;
+    }
+  }
+
+/* If the file_directory option is set expand the string, and set it as the
+"home" directory. The expansion can contain $home if check_local_user
+is set. */
+
+if (ob->file_dir != NULL)
+  {
+  directory = expand_string(ob->file_dir);
+  if (directory == NULL)
+    {
+    log_write(0, LOG_MAIN, "%s director failed to expand %s: %s", dblock->name,
+      ob->file_dir, expand_string_message);
+    addr->special_action = SPECIAL_FREEZE;
+    return ERROR;
+    }
+  deliver_home = directory;
+  }
+
+/* Do file existence tests; these can include references to $home. */
+
+switch (match_exists(dblock->require_files))
+  {
+  case FAIL:
   DEBUG(9) debug_printf("%s director skipped: file existence failure\n",
     dblock->name);
-  return FAIL;   
+  return FAIL;
+
+  case DEFER:
+  addr->message = string_sprintf("file existence defer in %s director: %s",
+    dblock->name, strerror(errno));
+  return DEFER;
   }
 
 /* Get the required file name and expand it. If the expansion fails, log the
 incident and indicate an internal error. */
 
 filename = expand_string(ob->file);
+
 if (filename == NULL)
   {
-  log_write(LOG_MAIN, "%s director failed to expand %s: %s", dblock->name,
+  log_write(0, LOG_MAIN, "%s director failed to expand %s: %s", dblock->name,
     ob->file, expand_string_message);
   addr->special_action = SPECIAL_FREEZE;
   return ERROR;
@@ -211,107 +294,100 @@ if (filename == NULL)
 DEBUG(2) debug_printf("%s director: file = %s\n", dblock->name,
   filename);
 
-/* If the check_local_user option is set, check that the local_part is
-the login of a local user, and fail if not. Note: the second argument to
-direct_finduser() must be NULL here, to prevent a numeric string being
-taken as a numeric uid. If the user is found, set directory to the home
-directory. */
+/* If a directory is set and the file name is not absolute, construct the
+complete file name. Otherwise set a flag to prevent an attempt at statting the
+directory below. */
 
-if (ob->check_local_user)
-  {
-  if ((pw = direct_finduser(addr->local_part, NULL)) == NULL)
-    {
-    DEBUG(2) debug_printf("%s director failed for %s (not a user)\n",
-      dblock->name, addr->local_part);
-    return FAIL;
-    }
-  else directory = pw->pw_dir;
-  }
-
-/* If the directory option is set, and the file name is not absolute, expand
-the string */
-
-if (ob->directory != NULL && filename[0] != '/')
-  {
-  directory = expand_string(ob->directory);
-  if (directory == NULL)
-    {
-    log_write(LOG_MAIN, "%s director failed to expand %s: %s", dblock->name,
-      ob->directory, expand_string_message);
-    addr->special_action = SPECIAL_FREEZE;
-    return ERROR;
-    }
-  }
-
-/* If a directory is set, construct the complete file name. */
-
-if (directory != NULL)
+if (directory != NULL && filename[0] != '/')
   filename = string_sprintf("%s/%s", directory, filename);
+else stat_directory = FALSE;
 
 /* Check that the file name is absolute. Simple checks are done in the
 init function, but expansions mean that we have to do a final check here. */
 
 if (filename[0] != '/')
   {
-  log_write(LOG_MAIN, "%s is not an absolute path for the %s director",
+  log_write(0, LOG_MAIN, "%s is not an absolute path for the %s director",
     filename, dblock->name);
   addr->special_action = SPECIAL_FREEZE;
   return ERROR;
   }
 
-/* If there is no fixed uid set, see if there's a dynamic one that can
-be expanded and possibly looked up. This uid/gid pair is used (a) for reading 
-the file if seteuid is on and (b) for running the transports for generated file 
-and pipe addresses. It is not (necessarily) the same as the uids/gids that may 
-own the file. */
+/* If there is no fixed explicit uid set, see if there's a dynamic one that can
+be expanded and possibly looked up. This uid is used (a) for reading the file
+if seteuid is on and (b) for running the transports for generated file and pipe
+addresses. It is not (necessarily) the same as the uids that may own the file.
+Exim panics if an expanded string is not a number and can't be found in the
+password file. */
 
-if (uid < 0 && ob->expand_uid != NULL)
+if (!uid_set && ob->expand_uid != NULL)
   {
-  struct passwd *pw = direct_find_expanded_user(ob->expand_uid,
-    dblock->name, "director", &uid);
-  if (pw != NULL) gid = pw->pw_gid;
+  direct_find_expanded_user(ob->expand_uid, dblock->name, "director",
+    &upw, &uid);
+  uid_set = TRUE;
+  }
 
-  if (gid < 0 && ob->expand_gid != NULL)
-    (void) direct_find_expanded_group(ob->expand_gid, dblock->name,
-      "director", &gid);
+/* Likewise, set up an expanded gid if necessary. */
 
-  /* If no gid has been set, it is a disaster. */
+if (!gid_set && ob->expand_gid != NULL)
+  {
+  direct_find_expanded_group(ob->expand_gid, dblock->name, "director", &gid);
+  gid_set = TRUE;
+  }
 
-  if (gid < 0)
-    log_write(LOG_PANIC_DIE, "User set without group for %s director",
+/* If a uid was specified, a gid to go with it must also be available. Take it
+from the passwd entry if one was looked up for the uid; otherwise it is a
+disaster. */
+
+if (uid_set && !gid_set)
+  {
+  if (upw != NULL)
+    {
+    gid = upw->pw_gid;
+    gid_set = TRUE;
+    }
+  else
+    log_write(0, LOG_PANIC_DIE, "User set without group for %s director",
       dblock->name);
   }
 
-/* If the seteuid option is on and a uid is available, use seteuid() to become
-the local user. This is necessary in order to read .forward files that are in
-NFS-mounted home directories. The macros expand to -1 on systems without
-setuid() configured, but in those cases this code is never obeyed, as the
-option is locked out in the init check.
+/* If the seteuid option is on and a uid is available (either explicit or as a
+result of check_local_user), use seteuid() to become the local user. This is
+necessary in order to read .forward files that are in NFS-mounted home
+directories. The macros expand to -1 on systems without setuid() configured,
+but in those cases this code is never obeyed, as the option is locked out in
+the init check.
 
 Even if the seteuid option is not on, we need to do this when the security
 level is 1 or 3, because in those cases this code is running seteuid to exim.
 But only when a uid is available. */
 
-if ((uid >= 0 || ob->check_local_user) &&
+if ((uid_set || ob->check_local_user) &&
     (ob->use_seteuid || security_level == 1 || security_level == 3))
   {
   saved_euid = geteuid();
   saved_egid = getegid();
   if (saved_euid != root_uid) mac_seteuid(root_uid);
-  
-  if (uid >= 0)
+
+  if (uid_set)
     {
+    if (ob->initgroups)
+      {
+      struct passwd *ppw = getpwuid(uid);
+      if (ppw != NULL) initgroups(ppw->pw_name, gid);
+      }
     mac_setegid(gid);
-    mac_seteuid(uid);  
+    mac_seteuid(uid);
     }
   else
-    {       
+    {
+    if (ob->initgroups) initgroups(pw->pw_name, pw->pw_gid);
     mac_setegid(pw->pw_gid);
     mac_seteuid(pw->pw_uid);
     }
-      
+
   DEBUG(2) debug_printf("%s director euid=%d egid=%d\n", dblock->name,
-    geteuid(), getegid());
+    (int)geteuid(), (int)getegid());
   restore_uid = TRUE;
   }
 
@@ -326,7 +402,7 @@ directory or to a specified directory), and in that case we try to do a bit
 better by statting the directory first. If it cannot be statted, assume there
 is some mounting problem, and defer the delivery. */
 
-if (directory != NULL)
+if (directory != NULL && stat_directory)
   {
   if (stat(directory, &statbuf) != 0)
     {
@@ -337,6 +413,7 @@ if (directory != NULL)
     yield = DEFER;
     goto RESTORE_UID;             /* skip forward */
     }
+  DEBUG(2) debug_printf("successful stat of %s\n", directory);
   }
 
 /* Now try to open the file for reading. If this fails with a non-existence
@@ -379,9 +456,9 @@ if (fstat(fileno(fwd), &statbuf) != 0)
 if ((statbuf.st_mode & S_IFMT) != S_IFREG ||
     (statbuf.st_mode & ob->modemask) != 0)
   {
-  addr->errno = ERRNO_BADFORWARD;
+  addr->basic_errno = ERRNO_BADFORWARD;
   addr->message =
-    string_sprintf("<%s> - bad mode (%x) for %s (%s director)\n",
+    string_sprintf("<%s> - bad mode (%o) for %s (%s director)",
       addr->orig, statbuf.st_mode, filename, dblock->name);
   yield = DEFER;
   goto CLOSE_RESTORE_UID;         /* skip forward */
@@ -396,9 +473,9 @@ else if (ob->owners == NULL) uid_ok = TRUE;
 
 if (!uid_ok && ob->owners != NULL)
   {
-  int p = 0;
-  while (ob->owners[p] >= 0)
-    if (ob->owners[p++] == statbuf.st_uid) { uid_ok = TRUE; break; }
+  int i;
+  for (i = 1; i <= (int)(ob->owners[0]); i++)
+    if (ob->owners[i] == statbuf.st_uid) { uid_ok = TRUE; break; }
   }
 
 /* If gid is to be checked, the local user's gid is always permissible. */
@@ -409,15 +486,15 @@ if (!gid_ok)
     { if (statbuf.st_gid == pw->pw_gid) gid_ok = TRUE; }
   if (!gid_ok && ob->owngroups != NULL)
     {
-    int p = 0;
-    while (ob->owngroups[p] >= 0)
-      if (ob->owngroups[p++] == statbuf.st_gid) { gid_ok = TRUE; break; }
+    int i;
+    for (i = 1; i <= (int)(ob->owngroups[0]); i++)
+      if (ob->owngroups[i] == statbuf.st_gid) { gid_ok = TRUE; break; }
     }
-  }   
+  }
 
 if (!uid_ok || !gid_ok)
   {
-  addr->errno = ERRNO_BADFORWARD;
+  addr->basic_errno = ERRNO_BADFORWARD;
   addr->message =
     string_sprintf("<%s> - bad %s for %s (%s director)",
       addr->orig, uid_ok? "group" : "owner", filename, dblock->name);
@@ -431,7 +508,7 @@ We read the file in one go in order to minimize the time we have it open. */
 filebuf = store_malloc(statbuf.st_size + 1);
 if (fread(filebuf, 1, statbuf.st_size, fwd) != statbuf.st_size)
   {
-  addr->errno = errno;
+  addr->basic_errno = errno;
   addr->message =
     string_sprintf("<%s> - error while reading forward file (%s director)\n",
     addr->orig, dblock->name);
@@ -442,49 +519,49 @@ filebuf[statbuf.st_size] = 0;
 
 
 /* If the filter option is set, the file is to be interpreted as a filter
-file instead of a straight list of addresses, if it starts with 
-"# Exim filter ..." (any capitilization, spaces optional). */
+file instead of a straight list of addresses, if it starts with
+"# Exim filter ..." (any capitilization, spaces optional). Check for this
+in all cases, so as to give a warning message if it is found when filtering
+is not enabled. */
 
-if (ob->filter)
+s = filebuf;
+tag = "# exim filter";
+while (isspace(*s)) s++;           /* Skips initial blank lines */
+for (; *tag != 0; s++, tag++)
   {
-  char *s = filebuf;
-  char *tag = "# exim filter";
-  while (isspace(*s)) s++;           /* Skips initial blank lines */ 
-  for (; *tag != 0; s++, tag++)
+  if (*tag == ' ')
     {
-    if (*tag == ' ')
-      {  
-      while (*s == ' ' || *s == '\t') s++;
-      s--; 
-      } 
-    else if (tolower(*s) != tolower(*tag)) break; 
-    }  
-  if (*tag == 0) is_filter = TRUE;
-  } 
-   
+    while (*s == ' ' || *s == '\t') s++;
+    s--;
+    }
+  else if (tolower(*s) != tolower(*tag)) break;
+  }
+if (*tag == 0) filter_header = TRUE;
 
 /* Filter interpretation is done by a general function that is also called from
-the filter testing option (-bf). Set up the value of extracted to be the same
-as it is from parse_extract_addresses(). */
+the filter testing option (-bf). The final argument specifies whether the log
+command can be used; this is OK if we have set the uid to a local user, but not
+otherwise. Set up the value of extracted to be the same as it is from
+parse_extract_addresses(). */
 
-if (is_filter)
+if (filter_header && ob->filter)
   {
-  deliver_home = directory; 
-  extracted = filter_interpret(filebuf, &generated, &delivered, &error)? 0 : -1;
-  deliver_home = NULL;
+  is_filter = TRUE;
+  extracted = filter_interpret(filebuf, &generated, &delivered, NULL, &error,
+    restore_uid, ob->forbid_filter_log, ob->rewrite)? 0 : -1;
   }
-   
+
 /* Otherwise it's a vanilla .forward file; call parse_extract_addresses()
 to get the values. The yield is 0=>OK, -1=>error, +1=> failed to open an
 :include: file. */
 
-else 
-  { 
+else
+  {
   /* There is a common function for use by forwarding and aliasing
   directors that extracts a list of addresses from a text string.
   Setting the fourth argument TRUE says that generating no addresses
   (from valid syntax) is no error.
-  
+
   The forward file may include :include: items, and we have to be
   careful about permissions for reading them. The extracting function
   will check that include files begin with a specified string, unless
@@ -493,16 +570,20 @@ else
   (b) we haven't used seteuid and there's no directory to check. If
   seteuid has been used, just try to read anything; otherwise restrict
   to the directory or lock out if none. */
-  
-  extracted = parse_extract_addresses(filebuf, &generated, &error, TRUE,
+
+  extracted = parse_extract_addresses(filebuf, &generated, &error,
+    TRUE,                                      /* no addresses => no error */
+    FALSE,                                     /* don't recognize :blackhole: */
+    ob->rewrite,                               /* rewrite if configured */
     ob->forbid_include? "*" :                  /* includes forbidden */
     restore_uid? NULL :                        /* if seteuid, try all */
     (directory == NULL)? "*" :                 /* if no directory, lock out */
-    directory);                                /* else restrain to directory */
-  } 
-  
+    directory,                                 /* else restrain to directory */
+    ob->skip_syntax_errors? &eblock : NULL);
+  }
+
 /* The store for holding the forward file is now finished with. */
- 
+
 store_free(filebuf);
 
 /* At this point we are finished with the .forward file. Close it, and, if
@@ -518,10 +599,11 @@ RESTORE_UID:
 if (restore_uid)
   {
   mac_seteuid(root_uid);
+  if (ob->initgroups) setgroups(0, NULL);
   mac_setegid(saved_egid);
   mac_seteuid(saved_euid);
   DEBUG(2) debug_printf("%s director restored euid=%d egid=%d\n", dblock->name,
-    geteuid(), getegid());
+    (int)geteuid(), (int)getegid());
   }
 
 /* If there has been an error, return the error value now. Subsequently we
@@ -534,36 +616,47 @@ if (yield != OK) return yield;
 
 if (extracted != 0)
   {
-  /* If extraction from a filter file failed, it is a "probably user error", to 
-  use a good old IBM term. Just defer delivery and let the user clean things 
+  /* If extraction from a filter file failed, it is a "probably user error", to
+  use a good old IBM term. Just defer delivery and let the user clean things
   up. */
-   
+
   if (is_filter)
     {
-    addr->errno = ERRNO_BADFORWARD;
+    addr->basic_errno = ERRNO_BADFORWARD;
     addr->message =
       string_sprintf("<%s> - error in filter file: %s", addr->orig,
         error);
     return DEFER;
     }
-    
+
   /* If extraction from a .forward file failed, freeze and yield ERROR if
   it was a missing :include: file and freeze_missing_include is TRUE. Other-
   wise just DEFER and hope things get fixed eventually. */
- 
+
   else
-    {       
-    addr->errno = ERRNO_BADFORWARD;
+    {
+    addr->basic_errno = ERRNO_BADFORWARD;
     addr->message =
-      string_sprintf("<%s> - error in forward file: %s", addr->orig,
+      string_sprintf("<%s> - error in forward file%s: %s", addr->orig,
+        filter_header? " (filtering not enabled)" : "",
         error);
     if (extracted > 0 && ob->freeze_missing_include)
-      { 
+      {
       addr->special_action = SPECIAL_FREEZE;
       return ERROR;
-      } 
+      }
     else return DEFER;
-    } 
+    }
+  }
+
+/* If skip_syntax_errors was set and there were syntax errors in the list,
+error messages will be present in eblock. Log them. */
+
+while (eblock != NULL)
+  {
+  log_write(0, LOG_MAIN, "%s director: syntax error in forward file %s: "
+    "%s in \"%s\"", dblock->name, filename, eblock->text1, eblock->text2);
+  eblock = eblock->next;
   }
 
 /* If this director has a local errors_to setting for where to send error
@@ -576,96 +669,153 @@ if (ob->errors_to != NULL)
   char *s = expand_string(ob->errors_to);
   if (s == NULL)
     {
-    log_write(LOG_MAIN, "%s director failed to expand %s", dblock->name,
-      ob->errors_to);
+    log_write(0, LOG_MAIN, "%s director failed to expand %s: %s", dblock->name,
+      ob->errors_to, expand_string_message);
     addr->special_action = SPECIAL_FREEZE;
     return ERROR;
     }
-    
+
   if (verify) errors_to = s; else
     {
-    char *snew;  
+    char *snew;
     if (verify_address(s, TRUE, TRUE, NULL, NULL, &snew, FALSE, FALSE) == OK)
       errors_to = snew;
-    }   
+    }
   }
 
 /* Add the new addresses to the list of new addresses, copying in the
-uid, gid and permission flags for use by pipes and files and autoreplies, 
-setting the parent, and or-ing its ignore_error flag. */
+uid, gid and permission flags for use by pipes and files and autoreplies,
+setting the parent, and or-ing its ignore_error flag.
+
+If the generated address is the same as one of its ancestors, and the
+check_ancestor flag is set, do not use this generated address, but replace it
+with a copy of the input address. This is to cope with cases where A is aliased
+to B and B has a .forward file pointing to A. We can't just pass on the old
+address by returning FAIL, because is must act as a general parent for
+generated addresses, and only get marked "done" when all its children are
+delivered. */
 
 while (generated != NULL)
   {
   address_item *next = generated;
   generated = next->next;
   next->parent = addr;
-  next->ignore_error |= addr->ignore_error; 
+  next->ignore_error |= addr->ignore_error;
   addr->child_count++;
   next->next = *addr_new;
   *addr_new = next;
 
-  if (errors_to != NULL) next->errors_address = errors_to;
-
-  if (next->orig[0] == '|' || next->orig[0] == '/' || next->orig[0] == '>')
+  if (ob->check_ancestor)
     {
-    next->director = dblock;
-    if (uid >= 0) next->uid = uid;
-      else if (ob->check_local_user) next->uid = pw->pw_uid;
-    if (gid >= 0) next->gid = gid;
-      else if (ob->check_local_user) next->gid = pw->pw_gid;
-    if (directory != NULL) next->home_dir = string_copy(directory);   
-    next->allow_pipe = !ob->forbid_pipe;
-    next->allow_file = !ob->forbid_file;
-    next->allow_reply = !ob->forbid_reply; 
+    address_item *parent;
+    for (parent = addr; parent != NULL; parent = parent->parent)
+      {
+      if (strcmp(next->orig, parent->orig) == 0)
+        {
+        DEBUG(9) debug_printf("generated parent ignored\n");
+        next->orig = string_copy(addr->orig);
+        break;
+        }
+      }
     }
 
-  DEBUG(2) debug_printf("%s director generated %s%s%s%s\n",
+  if (errors_to != NULL) next->errors_address = errors_to;
+
+  if (next->pfr)
+    {
+    next->director = dblock;
+    if (uid_set)
+      {
+      next->uid = uid;
+      next->uid_set = TRUE;
+      }
+    else if (ob->check_local_user)
+      {
+      next->uid = pw->pw_uid;
+      next->uid_set = TRUE;
+      }
+    if (gid_set)
+      {
+      next->gid = gid;
+      next->gid_set = TRUE;
+      }
+    else if (ob->check_local_user)
+      {
+      next->gid = pw->pw_gid;
+      next->gid_set = TRUE;
+      }
+    next->initgroups = ob->initgroups;
+    if (ob->home_dir != NULL) next->home_dir = ob->home_dir;
+      else if (directory != NULL) next->home_dir = string_copy(directory);
+    next->current_dir = ob->current_dir;
+    next->allow_pipe = !ob->forbid_pipe;
+    next->allow_file = !ob->forbid_file;
+    next->allow_reply = !ob->forbid_reply;
+    }
+
+  DEBUG(2) debug_printf("%s director generated %s%s%s%s%s\n",
     dblock->name,
     next->orig,
+    next->pfr? " (pipe, file, or autoreply)" : "",
     (errors_to != NULL)? " (errors to " : "",
     (errors_to != NULL)? errors_to : "",
     (errors_to != NULL)? ")" : "");
   }
-  
+
 /* If the filter interpreter returned "delivered" then we have succeeded
 in completely handling this address. Otherwise, we have to arrange for this
-address to be passed on to subsequent directors. Returning FAIL would appear to 
-be the answer, but it isn't, because successful delivery of the base address 
-gets it marked "done", so deferred generated addresses never get tried again. 
-We have to generate a new version of the base address, as if there were a 
+address to be passed on to subsequent directors. Returning FAIL would appear to
+be the answer, but it isn't, because successful delivery of the base address
+gets it marked "done", so deferred generated addresses never get tried again.
+We have to generate a new version of the base address, as if there were a
 "deliver" command in the filter file, with the original address as parent.
 However, we don't need to do this if there were no generated addresses. */
 
-if (is_filter) 
+if (is_filter)
   {
   address_item *next;
- 
-  if (delivered) return OK;
-  if (addr->child_count <= 0) return FAIL;
 
-  next = deliver_make_addr(addr->orig);
-  next->parent = addr;
-  next->ignore_error |= addr->ignore_error; 
-  addr->child_count++;
-  next->next = *addr_new;
-  *addr_new = next;
+  if (!delivered)
+    {
+    if (addr->child_count <= 0) return FAIL;
 
-  if (errors_to != NULL) next->errors_address = errors_to;
+    next = deliver_make_addr(addr->orig);
+    next->parent = addr;
+    next->ignore_error |= addr->ignore_error;
+    next->pfr = addr->pfr;
+    addr->child_count++;
+    next->next = *addr_new;
+    *addr_new = next;
 
-  DEBUG(2) debug_printf("%s director generated %s%s%s%s\n",
-    dblock->name,
-    next->orig,
-    (errors_to != NULL)? " (errors to " : "",
-    (errors_to != NULL)? errors_to : "",
-    (errors_to != NULL)? ")" : "");
- 
-  return OK;   
-  } 
+    if (errors_to != NULL) next->errors_address = errors_to;
+
+    DEBUG(2) debug_printf("%s director generated %s%s%s%s%s\n",
+      dblock->name,
+      next->orig,
+      next->pfr? " (pipe, file, or autoreply)" : "",
+      (errors_to != NULL)? " (errors to " : "",
+      (errors_to != NULL)? errors_to : "",
+      (errors_to != NULL)? ")" : "");
+    }
+
+  yield = OK;
+  }
 
 /* If the forward file generated no addresses, it is not an error. The
 director just fails. Compare aliasfile, which is different. */
 
-else return (addr->child_count <= 0)? FAIL : OK;
+else yield = (addr->child_count <= 0)? FAIL : OK;
+
+/* If the yield is OK, put the original address onto the succeed queue so
+that any retry items that get attached to it get processed. */
+
+if (yield == OK)
+  {
+  addr->next = *addr_succeed;
+  *addr_succeed = addr;
+  }
+
+return yield;
 }
 
 /* End of directors/forwardfile.c */
