@@ -2,14 +2,17 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
   char *pw_dir;
-  re_block *re_pw_dir; 
+  char *home_dir;
+  char *current_dir;
+  re_block *re_pw_dir;
+  BOOL initgroups;
 } localuser_director_options_block;
 
 /* Data for reading the private options. */
@@ -23,9 +26,9 @@ extern localuser_director_options_block localuser_director_option_defaults;
 
 /* The main and initialization entry points for the director */
 
-extern int localuser_director_entry(director_instance *, address_item *, 
-  address_item **, address_item **, address_item **, BOOL);
-   
-extern void localuser_director_init(director_instance *); 
+extern int localuser_director_entry(director_instance *, address_item *,
+  address_item **, address_item **, address_item **, address_item **, BOOL);
+
+extern void localuser_director_init(director_instance *);
 
 /* End of directors/localuser.h */
