@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -17,9 +17,10 @@ Warning: Linking some objects which contain exception information sections
         problems.
 
 As this may cause people to worry needlessly, include a dummy function here
-to stop the message from appearing. */
+to stop the message from appearing. Make it call itself to stop picky compilers
+complaining that it isn't used. */
 
-static void dummy(void) {}
+static void dummy(void) { dummy(); }
 
 
 /* Tables of information about which directors, routers and transports are
@@ -49,6 +50,10 @@ config.h file. */
 #include "routers/domainlist.h"
 #endif
 
+#ifdef ROUTER_IPLITERAL
+#include "routers/ipliteral.h"
+#endif
+
 #ifdef ROUTER_IPLOOKUP
 #include "routers/iplookup.h"
 #endif
@@ -86,52 +91,52 @@ config.h file. */
 
 director_info directors_available[] = {
 #ifdef DIRECTOR_ALIASFILE
-  { 
-  "aliasfile", 
-  aliasfile_director_entry, 
-  aliasfile_director_init, 
-  aliasfile_director_tidyup,
-  aliasfile_director_options, 
+  {
+  "aliasfile",
+  aliasfile_director_entry,
+  aliasfile_director_init,
+  NULL,    /* no tidyup entry */
+  aliasfile_director_options,
   &aliasfile_director_options_count,
   &aliasfile_director_option_defaults,
   sizeof(aliasfile_director_options_block)
-  },  
+  },
 #endif
 #ifdef DIRECTOR_FORWARDFILE
-  { 
-  "forwardfile", 
-  forwardfile_director_entry, 
+  {
+  "forwardfile",
+  forwardfile_director_entry,
   forwardfile_director_init,
   NULL,    /* no tidyup entry */
-  forwardfile_director_options, 
+  forwardfile_director_options,
   &forwardfile_director_options_count,
   &forwardfile_director_option_defaults,
   sizeof(forwardfile_director_options_block)
-  },  
+  },
 #endif
 #ifdef DIRECTOR_LOCALUSER
-  { 
-  "localuser", 
-  localuser_director_entry, 
-  localuser_director_init, 
+  {
+  "localuser",
+  localuser_director_entry,
+  localuser_director_init,
   NULL,    /* no tidyup entry */
-  localuser_director_options, 
+  localuser_director_options,
   &localuser_director_options_count,
   &localuser_director_option_defaults,
   sizeof(localuser_director_options_block)
-  },  
+  },
 #endif
 #ifdef DIRECTOR_SMARTUSER
-  { 
-  "smartuser", 
-  smartuser_director_entry, 
-  smartuser_director_init, 
+  {
+  "smartuser",
+  smartuser_director_entry,
+  smartuser_director_init,
   NULL,    /* no tidyup entry */
-  smartuser_director_options, 
+  smartuser_director_options,
   &smartuser_director_options_count,
   &smartuser_director_option_defaults,
   sizeof(smartuser_director_options_block)
-  },  
+  },
 #endif
 { "", NULL, NULL, NULL, NULL, 0, NULL, 0 }
 };
@@ -141,52 +146,64 @@ director_info directors_available[] = {
 router_info routers_available[] = {
 #ifdef ROUTER_DOMAINLIST
   {
-  "domainlist", 
-  domainlist_router_entry, 
-  domainlist_router_init, 
-  domainlist_router_tidyup, 
-  domainlist_router_options, 
+  "domainlist",
+  domainlist_router_entry,
+  domainlist_router_init,
+  NULL,    /* no tidyup entry */
+  domainlist_router_options,
   &domainlist_router_options_count,
   &domainlist_router_option_defaults,
   sizeof(domainlist_router_options_block)
   },
-#endif   
+#endif
+#ifdef ROUTER_IPLITERAL
+  {
+  "ipliteral",
+  ipliteral_router_entry,
+  ipliteral_router_init,
+  NULL,    /* no tidyup entry */
+  ipliteral_router_options,
+  &ipliteral_router_options_count,
+  &ipliteral_router_option_defaults,
+  sizeof(ipliteral_router_options_block)
+  },
+#endif
 #ifdef ROUTER_IPLOOKUP
   {
-  "iplookup", 
-  iplookup_router_entry, 
-  iplookup_router_init, 
+  "iplookup",
+  iplookup_router_entry,
+  iplookup_router_init,
   NULL,    /* no tidyup entry */
-  iplookup_router_options, 
+  iplookup_router_options,
   &iplookup_router_options_count,
   &iplookup_router_option_defaults,
   sizeof(iplookup_router_options_block)
   },
-#endif   
+#endif
 #ifdef ROUTER_LOOKUPHOST
   {
-  "lookuphost", 
-  lookuphost_router_entry, 
-  lookuphost_router_init, 
+  "lookuphost",
+  lookuphost_router_entry,
+  lookuphost_router_init,
   NULL,    /* no tidyup entry */
-  lookuphost_router_options, 
+  lookuphost_router_options,
   &lookuphost_router_options_count,
   &lookuphost_router_option_defaults,
   sizeof(lookuphost_router_options_block)
   },
-#endif    
+#endif
 #ifdef ROUTER_QUERYPROGRAM
   {
-  "queryprogram", 
-  queryprogram_router_entry, 
-  queryprogram_router_init, 
+  "queryprogram",
+  queryprogram_router_entry,
+  queryprogram_router_init,
   NULL,    /* no tidyup entry */
-  queryprogram_router_options, 
+  queryprogram_router_options,
   &queryprogram_router_options_count,
   &queryprogram_router_option_defaults,
   sizeof(queryprogram_router_options_block)
   },
-#endif    
+#endif
 { "", NULL, NULL, NULL, NULL, 0, NULL, 0 }
 };
 
@@ -204,7 +221,7 @@ transport_info transports_available[] = {
   &appendfile_transport_option_defaults,       /* private options defaults */
   sizeof(appendfile_transport_options_block),  /* size of private block */
   TRUE,                                        /* local flag */
-  NULL                                         /* no closedown entry */ 
+  NULL                                         /* no closedown entry */
   },
 #endif
 #ifdef TRANSPORT_AUTOREPLY
@@ -218,7 +235,7 @@ transport_info transports_available[] = {
   &autoreply_transport_option_defaults,        /* private options defaults */
   sizeof(autoreply_transport_options_block),   /* size of private block */
   TRUE,                                        /* local flag */
-  NULL                                         /* no closedown entry */ 
+  NULL                                         /* no closedown entry */
   },
 #endif
 #ifdef TRANSPORT_DEBUG
@@ -232,7 +249,7 @@ transport_info transports_available[] = {
   &debug_transport_option_defaults,            /* private options defaults */
   sizeof(debug_transport_options_block),       /* size of private block */
   TRUE,                                        /* local flag */
-  NULL                                         /* no closedown entry */ 
+  NULL                                         /* no closedown entry */
   },
 #endif
 #ifdef TRANSPORT_PIPE
@@ -246,7 +263,7 @@ transport_info transports_available[] = {
   &pipe_transport_option_defaults,             /* private options defaults */
   sizeof(pipe_transport_options_block),        /* size of private block */
   TRUE,                                        /* local flag */
-  NULL                                         /* no closedown entry */ 
+  NULL                                         /* no closedown entry */
   },
 #endif
 #ifdef TRANSPORT_SMTP
@@ -260,10 +277,10 @@ transport_info transports_available[] = {
   &smtp_transport_option_defaults,             /* private options defaults */
   sizeof(smtp_transport_options_block),        /* size of private block */
   FALSE,                                       /* local flag */
-  smtp_transport_closedown                     /* close down passed channel */ 
+  smtp_transport_closedown                     /* close down passed channel */
   },
 #endif
 { "", NULL, NULL, NULL, NULL, 0, NULL, 0, FALSE }
-};        
+};
 
 /* End of drtables.c */
