@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -19,7 +19,7 @@ to the output filename. This is also handled. If there are any other variants,
 the program won't cope.
 
 The first argument to the program is the name of the serial file; the second
-is the base name for the DBM file(s). When native db is in use, these must be 
+is the base name for the DBM file(s). When native db is in use, these must be
 different.
 
 Input lines beginning with # are ignored, as are blank lines. Entries begin
@@ -31,11 +31,7 @@ indented lines. */
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
-#ifdef	USE_FCNTL_H
 #include <fcntl.h>
-#else
-#include <sys/fcntl.h>
-#endif
 
 
 #define maxsize 10000
@@ -46,7 +42,10 @@ int main(int argc, char **argv)
 int started;
 int count = 0;
 int yield = 0;
+#ifndef USE_DB
 int is_db = 0;
+struct stat statbuf;
+#endif
 FILE *f;
 EXIM_DB *d;
 EXIM_DATUM key, content;
@@ -56,7 +55,6 @@ char  temp_dbmname[256];
 char  real_dbmname[256];
 char *buffer = malloc(maxsize);
 char *line = malloc(maxsize);
-struct stat statbuf;
 
 if (argc != 3)
   {
