@@ -2,12 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
 #include <ctype.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <fcntl.h>
@@ -15,21 +16,28 @@
 
 #include <sys/stat.h>
 
+#include <pwd.h>                  /* Because used in includes below */
+#include "../regexp/regexp.h"     /* Ditto */
+
 #include "macros.h"
 #include "dbhdr.h"
+#include "structs.h"
+#include "functions.h"
 
 #define type_retry 1
 #define type_wait 2
 #define type_reject 3
+#define type_serialize 4
 
 
 /* Utility program to interrogate the contents of an exim database file, with
 possible updating. The first argument is the spool directory. The second
 argument is the name of the database file. The available names are:
 
-  retry:            retry delivery information
-  wait-<transport>  message waiting information
-  reject            message rejection information
+  retry                  retry delivery information
+  wait-<transport>       message waiting information
+  reject                 message rejection information
+  serialize-<transport>  host serialization information
 
 In order not to hold the database lock any longer than is necessary, each
 operation on the database uses a separate open/close call. This is expensive,
@@ -112,7 +120,6 @@ return mktime(tm);
 
 int main(int argc, char **argv)
 {
-struct stat statbuf;
 int db_type = 0;
 char buffer[256];
 
@@ -124,21 +131,23 @@ if (argc != 3)
   printf("<database-name> = retry | wait-<transport> | reject\n");
   exit(1);
   }
-  
+
 if (strcmp(argv[2], "retry") == 0) db_type = type_retry;
   else if (strcmp(argv[2], "reject") == 0) db_type = type_reject;
   else if (strncmp(argv[2], "wait-", 5) == 0) db_type = type_wait;
+  else if (strncmp(argv[2], "serialize-", 10) == 0) db_type = type_serialize;
 
 if (db_type == 0)
   {
   printf("Usage: exim_fixdb <spool-directory> <database-name>\n");
-  printf("<database-name> = retry | wait-<transport> | reject\n");
+  printf("<database-name> = retry | wait-<transport> | reject | "
+    "serialize-<transport>\n");
   exit(1);
-  } 
-  
+  }
+
 /* Set up the spool directory in the global place. */
 
-spool_directory = argv[1]; 
+spool_directory = argv[1];
 
 /* Verify what we are working on */
 
@@ -153,7 +162,8 @@ for(;;)
   db_retry *retry;
   db_wait *wait;
   db_reject *reject;
-  int i;
+  db_serialize *serialize;
+  int i, length;
   char *t;
   char name[256], field[256], value[256];
 
@@ -217,7 +227,7 @@ for(;;)
         if (record == NULL) printf("not found\n"); else
           {
           time_t tt;
-          int length;
+          int length = 0;     /* Stops compiler warning */
 
           switch(db_type)
             {
@@ -228,7 +238,7 @@ for(;;)
             switch(fieldno)
               {
               case 0:
-              retry->errno = atoi(value);
+              retry->basic_errno = atoi(value);
               break;
 
               case 1:
@@ -270,6 +280,10 @@ for(;;)
             case type_reject:
             printf("Can't change contents of reject database record\n");
             break;
+
+            case type_serialize:
+            printf("Can't change contents of serialization database record\n");
+            break;
             }
 
           db_write(dbm, name, record, length);
@@ -301,19 +315,21 @@ for(;;)
     continue;
     }
 
-  record = db_read(dbm, name);
-  if (record == NULL) 
+  record = db_read_with_length(dbm, name, &length);
+  if (record == NULL)
     {
-    printf("record %s not found\n", name); 
-    name[0] = 0; 
-    } 
+    printf("record %s not found\n", name);
+    name[0] = 0;
+    }
   else
     {
+    int count_bad = 0;
     switch(db_type)
       {
       case type_retry:
       retry = (db_retry *)record;
-      printf("0 error number: %d %s\n", retry->errno, retry->text);
+      printf("0 error number: %d %s\n", retry->basic_errno,
+        string_printing(retry->text, FALSE));
       printf("1 extra data:   %d\n", retry->more_errno);
       printf("2 first failed: %s\n", print_time(retry->first_failed));
       printf("3 last try:     %s\n", print_time(retry->last_try));
@@ -325,10 +341,30 @@ for(;;)
       wait = (db_wait *)record;
       t = wait->text;
       printf("Sequence: %d\n", wait->sequence);
+      if (wait->count > WAIT_NAME_MAX)
+        {
+        printf("**** Data corrupted: count=%d=0x%x max=%d ****\n", wait->count,
+          wait->count, WAIT_NAME_MAX);
+        wait->count = WAIT_NAME_MAX;
+        count_bad = 1;
+        }
       for (i = 1; i <= wait->count; i++)
         {
         strncpy(value, t, MESSAGE_ID_LENGTH);
         value[MESSAGE_ID_LENGTH] = 0;
+        if (count_bad && value[0] == 0) break;
+        if ((int)strlen(value) != MESSAGE_ID_LENGTH ||
+            strspn(value, "0123456789"
+                          "abcdefghijklmnopqrstuvwxyz"
+                          "ABCDEFGHIJKLMNOPQRSTUVWXYZ-") != MESSAGE_ID_LENGTH)
+          {
+          int j;
+          printf("\n**** Data corrupted: bad character in message id ****\n");
+          for (j = 0; j < MESSAGE_ID_LENGTH; j++)
+            printf("%02x ", value[j]);
+          printf("\n");
+          break;
+          }
         printf("%s ", value);
         t += MESSAGE_ID_LENGTH;
         }
@@ -337,7 +373,16 @@ for(;;)
 
       case type_reject:
       reject = (db_reject *)record;
-      printf("%s\n", print_time(reject->time_stamp));
+      if (length < sizeof(db_retry))
+        printf("%s\n", print_time(reject->time_stamp));
+      else
+        printf("%s %c\n", print_time(reject->time_stamp),
+          reject->rejected_mail_from? 'T':'F');
+      break;
+
+      case type_serialize:
+      serialize = (db_serialize *)record;
+      printf("%s\n", print_time(serialize->time_stamp));
       break;
       }
 
