@@ -2,28 +2,30 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
 /* Utility program to tidy the contents of an exim database file. The
-arguments are two optional flags, followed by the spool directory name and the 
+arguments are two optional flags, followed by the spool directory name and the
 database name. The flags are
 
    -f         full tidy - for the waiting database, checks are made on the
               existence of messages
-               
-   -t <time>  expiry time for old records - default 30 days       
+
+   -t <time>  expiry time for old records - default 30 days
 
 The final argument is the name of the database file. The available names are:
 
-  retry:      retry delivery information       
+  retry:      retry delivery information
   wait-<t>:   message waiting information - <t> is transport name
-  reject:     message rejection information 
-  
+  reject:     message rejection information
+  serialize:  host serialization information
+
 This argument is required. */
 
 
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,13 +40,14 @@ This argument is required. */
 #define type_retry 1
 #define type_wait 2
 #define type_reject 3
+#define type_serialize 4
 
 
 
 typedef struct key_item {
   struct key_item *next;
   char   key[1];
-} key_item;    
+} key_item;
 
 
 
@@ -57,18 +60,19 @@ extern char *spool_directory;
 static void usage(void)
 {
 printf("Usage: exim_tidydb [-f] [-t <time>] <spool-directory> <database-name>\n");
-printf("<database-name> = retry | wait-<transport> | reject\n");
+printf("<database-name> = retry | wait-<transport> | reject | "
+  "serialize-<transport>\n");
 exit(1);
 }
 
 
 int main(int argc, char **argv)
 {
-struct stat statbuf; 
+struct stat statbuf;
 int  db_type = 0;
 int  maxkeep = 30 * 24 * 60 * 60;
-int  i, db_arg, oldest, path_len;
-int  keycount = 0;
+int  i, oldest, path_len;
+int db_arg = 0;
 key_item *keychain = NULL;
 BOOL full = FALSE;
 EXIM_DB *dbm;
@@ -84,12 +88,13 @@ for (i = 1; i < argc; i++)
     {
     spool_directory = argv[i++];
     if (i != argc-1) usage();
-    db_arg = i; 
+    db_arg = i;
     if (strcmp(argv[i], "retry") == 0) db_type = type_retry;
-      else if (strncmp(argv[i], "wait-", 5) == 0) db_type = type_wait;
-        else if (strcmp(argv[i], "reject") == 0) db_type = type_reject;
-          else usage();
-    }  
+    else if (strncmp(argv[i], "wait-", 5) == 0) db_type = type_wait;
+    else if (strcmp(argv[i], "reject") == 0) db_type = type_reject;
+    else if (strncmp(argv[i], "serialize-", 10) == 0) db_type = type_serialize;
+    else usage();
+    }
   else if (strcmp(argv[i], "-f") == 0) full = TRUE;
   else if (strcmp(argv[i], "-t") == 0)
     {
@@ -113,13 +118,13 @@ for (i = 1; i < argc; i++)
         }
       maxkeep += value;
       }
-    }   
+    }
   else usage();
-  } 
+  }
 
 
 /* Compute the oldest keep time */
-               
+
 oldest = time(NULL) - maxkeep;
 
 /* Database name missing */
@@ -139,8 +144,8 @@ if (dbm == NULL)
   {
   printf("Failed to open database file %s\n", argv[db_arg]);
   exit(1);
-  }   
-  
+  }
+
 
 /* Prepare for building file names */
 
@@ -159,10 +164,12 @@ while (key != NULL)
   key_item *k = (key_item *)malloc(sizeof(key_item) + (int)strlen(key));
   k->next = keychain;
   keychain = k;
-  strcpy(k->key, key);   
+  strcpy(k->key, key);
   key = db_scan(dbm, FALSE);
   }
-  
+
+printf("keys read\n");
+
 
 /* Now scan the collected keys and operate on the records. */
 
@@ -170,43 +177,51 @@ while (keychain != NULL)
   {
   db_generic *value;
   key = keychain->key;
-  keychain = keychain->next; 
+  keychain = keychain->next;
   value = db_read(dbm, key);
-  
+
   /* A continuation record may have been deleted or renamed already, so
-  non-existence is not serious. */ 
-  
+  non-existence is not serious. */
+
   if (value == NULL) continue;
-  
-  /* Delete if too old */ 
-       
-  if (value->time_stamp < oldest) 
+
+  /* Delete if too old */
+
+  if (value->time_stamp < oldest)
     {
-    printf("deleted %s (too old)\n", key); 
+    printf("deleted %s (too old)\n", key);
     db_delete(dbm, key);
-    free(value); 
-    continue; 
-    } 
-    
+    free(value);
+    continue;
+    }
+
   /* If full processing not requested, do no more. */
-   
+
   if (!full) { free(value); continue; }
-  
+
   /* Otherwise do database-specific tidying for wait databases */
-    
+
   if (db_type == type_wait)
-    { 
-    db_wait *wait = (db_wait *)value; 
-    BOOL update = FALSE; 
-    
+    {
+    db_wait *wait = (db_wait *)value;
+    BOOL update = FALSE;
+
+    /* Leave corrupt records alone */
+
+    if (wait->count > WAIT_NAME_MAX)
+      {
+      printf("**** Data for %s corrupted\n  count=%d=0x%x max=%d\n",
+        key, wait->count, wait->count, WAIT_NAME_MAX);
+      continue;
+      }
 
     /* Loop for renamed continuation records */
-    
+
     for (;;)
-      {  
+      {
       int offset;
-      int length = wait->count * MESSAGE_ID_LENGTH; 
-      
+      int length = wait->count * MESSAGE_ID_LENGTH;
+
       for (offset = length - MESSAGE_ID_LENGTH;
            offset >= 0; offset -= MESSAGE_ID_LENGTH)
         {
@@ -215,71 +230,71 @@ while (keychain != NULL)
         if (stat(buffer, &statbuf) != 0)
           {
           int left = length - offset - MESSAGE_ID_LENGTH;
-          if (left > 0) strncpy(wait->text + offset, 
+          if (left > 0) strncpy(wait->text + offset,
             wait->text + offset + MESSAGE_ID_LENGTH, left);
-          wait->count--;           
-          length -= MESSAGE_ID_LENGTH; 
+          wait->count--;
+          length -= MESSAGE_ID_LENGTH;
           update = TRUE;
-          }  
-        } 
-         
+          }
+        }
+
       /* If record is empty and the main record, either delete it or rename
       the next continuation, repeating if that is also empty. */
-        
+
       if (wait->count == 0 && strchr(key, ':') == NULL)
         {
         while (wait->count == 0 && wait->sequence > 0)
           {
           char newkey[256];
-          db_generic *newvalue; 
+          db_generic *newvalue;
           sprintf(newkey, "%s:%d", key, wait->sequence - 1);
           newvalue = db_read(dbm, newkey);
           if (newvalue != NULL)
             {
             free(value);
-            value = newvalue;  
-            wait = (db_wait *)newvalue; 
+            value = newvalue;
+            wait = (db_wait *)newvalue;
             db_delete(dbm, newkey);
-            printf("renamed %s\n", newkey); 
-            update = TRUE; 
-            }   
-          else wait->sequence--;   
-          }  
-        
+            printf("renamed %s\n", newkey);
+            update = TRUE;
+            }
+          else wait->sequence--;
+          }
+
         /* If we have ended up with an empty main record, delete it
         and break the loop. Otherwise the new record will be scanned. */
-          
+
         if (wait->count == 0 && wait->sequence == 0)
           {
           db_delete(dbm, key);
-          printf("deleted %s (empty)\n", key);  
-          update = FALSE; 
-          break; 
-          }  
-        }  
-        
+          printf("deleted %s (empty)\n", key);
+          update = FALSE;
+          break;
+          }
+        }
+
       /* If not an empty main record, break the loop */
 
       else break;
-      }   
-      
+      }
+
     /* Re-write the record if required */
-     
+
     if (update)
       {
-      printf("updated %s\n", key); 
-      db_write(dbm, key, wait, sizeof(db_wait) + 
+      printf("updated %s\n", key);
+      db_write(dbm, key, wait, sizeof(db_wait) +
         wait->count * MESSAGE_ID_LENGTH);
-      }   
+      }
     }
-     
+
   /* Recover the store */
-     
-  free(value); 
+
+  free(value);
   }
 
 db_close(dbm);
-return 0;    
+return 0;
 }
 
 /* End of exim_tidydb.c */
