@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -33,6 +33,10 @@ enum {
   vtype_string,         /* value is address of string */
   vtype_stringptr,      /* value is address of pointer to string */
   vtype_msgbody,        /* as stringptr, but read when first required */
+  vtype_localpart,      /* extract local part from string */
+  vtype_domain,         /* extract domain from string */
+  vtype_recipients,     /* extract recipients from chain of addresses */
+                        /* (not currently in use) */
   vtype_todbsdin,       /* value not used; generate BSD inbox tod */
   vtype_todf,           /* value not used; generate full tod */
   vtype_todl,           /* value not used; generate log tod */
@@ -54,15 +58,25 @@ static var_entry var_table[] = {
   { "domain",              vtype_stringptr,   &deliver_domain },
   { "home",                vtype_stringptr,   &deliver_home },
   { "host",                vtype_stringptr,   &deliver_host },
+  { "key",                 vtype_stringptr,   &lookup_key },
   { "local_part",          vtype_stringptr,   &deliver_localpart },
+  { "local_part_prefix",   vtype_stringptr,   &deliver_localpart_prefix },
+  { "local_part_suffix",   vtype_stringptr,   &deliver_localpart_suffix },
   { "message_body",        vtype_msgbody,     &message_body },
-  { "message_id",          vtype_string,      &message_id_external },
+  { "message_id",          vtype_stringptr,   &message_id },
+  { "message_precedence",  vtype_stringptr,   &message_precedence },
   { "message_size",        vtype_int,         &message_size },
+  { "original_domain",     vtype_stringptr,   &deliver_domain_orig },
+  { "original_local_part", vtype_stringptr,   &deliver_localpart_orig },
   { "primary_hostname",    vtype_stringptr,   &primary_hostname },
   { "received_protocol",   vtype_stringptr,   &received_protocol },
+  { "recipients_count",    vtype_int,         &recipients_count },
   { "reply_address",       vtype_reply,       NULL },
   { "return_path",         vtype_stringptr,   &return_path },
+  { "route_option",        vtype_stringptr,   &route_option },
   { "sender_address",      vtype_stringptr,   &sender_address },
+  { "sender_address_domain", vtype_domain,    &sender_address },
+  { "sender_address_local_part", vtype_localpart, &sender_address },
   { "sender_fullhost",     vtype_stringptr,   &sender_fullhost },
   { "sender_host_address", vtype_stringptr,   &sender_host_address },
   { "sender_host_name",    vtype_stringptr,   &sender_host_name },
@@ -86,14 +100,25 @@ static char var_buffer[256];
 *             Pick out a name from a string      *
 *************************************************/
 
-/* The 3rd argument is pointing at the first alphabetic character.
-The yield is its revised value. If the name is too long, it is
-silently truncated. */
+/* If the name is too long, it is silently truncated.
 
-static char *read_name(char *name, int max, char *s)
+Arguments:
+  name      points to a buffer into which to put the name
+  max       is the length of the buffer
+  s         points to the first alphabetic character of the name
+  extras    chars other than alphanumerics to permit
+
+Returns:    pointer to the first character after the name
+
+Note: The test for *s != 0 in the while loop is necessary because
+strchr() yields non-NULL if the character is zero (which is not something
+I expected). */
+
+static char *
+read_name(char *name, int max, char *s, char *extras)
 {
 int ptr = 0;
-while (isalnum(*s) || *s == '_')
+while (*s != 0 && (isalnum(*s) || strchr(extras, *s) != NULL))
   {
   if (ptr < max-1) name[ptr++] = *s;
   s++;
@@ -112,9 +137,19 @@ return s;
 abbreviations) might not be the complete header name because headers can
 contain any printing characters in their names, except ':'. This function is
 called to read the rest of the name, chop h[eader]_ off the front, and put ':'
-on the end. */
+on the end, if the name was terminated by white space.
 
-static char *read_header_name(char *name, int max, char *s)
+Arguments:
+  name      points to a buffer in which the name read so far exists
+  max       is the length of the buffer
+  s         points to the first character after the name so far, i.e. the
+            first non-alphameric character after $header_xxxxx
+
+Returns:    a pointer to the first character after the header name
+*/
+
+static char *
+read_header_name(char *name, int max, char *s)
 {
 int prelen = strchr(name, '_') - name + 1;
 int ptr = strlen(name) - prelen;
@@ -136,10 +171,15 @@ return s;
 *           Pick out a number from a string      *
 *************************************************/
 
-/* The 2nd argument is pointing at the first digit.
-The yield is its revised value. */
+/* Arguments:
+  n     points to an integer into which to put the number
+  s     points to the first digit of the number
 
-static char *read_number(int *n, char *s)
+Returns:  a pointer to the character after the last digit
+*/
+
+static char *
+read_number(int *n, char *s)
 {
 *n = 0;
 while (isdigit(*s)) *n = *n * 10 + (*s++ - '0');
@@ -154,9 +194,18 @@ return s;
 
 /* This function is also used by the search routines when given a multi-level
 key to search for. The yield is in dynamic store; NULL means that the key
-was not found. */
+was not found.
 
-char *expand_getkeyed(char *key, char *s)
+Arguments:
+  key       points to the name of the subkey
+  s         points to the string from which to extract the subfield
+
+Returns:    NULL if the subfield was not found, or
+            a pointer to the subfield's data
+*/
+
+char *
+expand_getkeyed(char *key, char *s)
 {
 int length = (int)strlen(key);
 while (isspace(*s)) s++;
@@ -205,38 +254,7 @@ while (*s != 0)
         {
         int ch = *(++data);
         if (ch == 0 || ch == '\"') break;
-        if (ch == '\\')
-          {
-          ch = *(++data);
-          if (isdigit(ch) && ch != '8' && ch != '9')
-            {
-            ch -= '0';
-            if (isdigit(data[1]) && data[1] != '8' && data[1] != '9')
-              {
-              ch = ch * 8 + *(++data) - '0';
-              if (isdigit(data[1]) && data[1] != '8' && data[1] != '9')
-                ch = ch * 8 + *(++data) - '0';
-              }
-            }
-          else switch (ch)
-            {
-            case 0:    data--;  /* Fall through */
-            case '\\': ch = '\\'; break;
-            case 'n':  ch = '\n'; break;
-            case 'r':  ch = '\r'; break;
-            case 't':  ch = '\t'; break;
-            case 'x':
-            ch = 0;
-            if (isxdigit(data[1]))
-              {
-              ch = ch * 16 +
-                strchr(hex_digits, tolower(*(++data))) - hex_digits;
-              if (isxdigit(data[1])) ch = ch * 16 +
-                strchr(hex_digits, tolower(*(++data))) - hex_digits;
-              }
-            break;
-            }
-          }
+        if (ch == '\\') ch = string_interpret_escape(&data);
         yield[i++] = ch;
         }
       yield[i] = 0;
@@ -269,9 +287,21 @@ return NULL;
 can search it using a binary chop. The "choplen" variable is
 nothing to do with the binary chop. It can be set non-zero to
 cause chars at the end of the returned string to be disregarded.
-It should already be zero on entry. */
+It should already be zero on entry.
 
-static char *find_variable(char *name, int *choplen)
+Arguments:
+  name      the name of the variable being sought
+  choplen   a pointer to an int which is set if characters at the end
+            of the returned data are to be ignored (typically '\n' at the
+            end of header lines); it should normally be set zero before
+            calling this function
+
+Returns:    NULL if the variable does not exist, or
+            a pointer to the variable's contents
+*/
+
+static char *
+find_variable(char *name, int *choplen)
 {
 int first = 0;
 int last = var_table_size;
@@ -279,7 +309,7 @@ int last = var_table_size;
 while (last > first)
   {
   header_line *h;
-  char *s;
+  char *s, *domain, *localpart;
   char **ss;
   int middle = (first + last)/2;
   int c = strcmp(name, var_table[middle].name);
@@ -297,6 +327,23 @@ while (last > first)
     s = *((char **)(var_table[middle].value));
     return (s == NULL)? "" : s;
 
+    case vtype_localpart:                      /* Get local part from address */
+    s = *((char **)(var_table[middle].value));
+    if (s == NULL) return "";
+    domain = strrchr(s, '@');
+    if (domain == NULL) return s;
+    localpart = domain - 1;
+    while (localpart > s && localpart[-1] != ':') localpart--;
+    strncpy(var_buffer, localpart, domain - localpart);
+    var_buffer[domain - localpart] = 0;
+    return var_buffer;
+
+    case vtype_domain:                         /* Get domain from address */
+    s = *((char **)(var_table[middle].value));
+    if (s == NULL) return "";
+    domain = strrchr(s, '@');
+    return (domain == NULL)? "" : domain + 1;
+
     case vtype_msgbody:                        /* Pointer to msgbody string */
     ss = (char **)(var_table[middle].value);
     if (*ss == NULL && deliver_datafile >= 0)  /* Read body when needed */
@@ -306,14 +353,14 @@ while (last > first)
       if (len > message_size) len = message_size;
       *ss = body = store_malloc(len+1);
       body[0] = 0;
-      lseek(deliver_datafile, 0, SEEK_SET);
+      lseek(deliver_datafile, data_start_offset, SEEK_SET);
       len = read(deliver_datafile, body, len);
       if (len >= 0) body[len] = 0;
-      while (*body != 0) 
+      while (*body != 0)
         {
         if (*body == '\n') *body = ' ';
         body++;
-        }  
+        }
       }
     return (*ss == NULL)? "" : *ss;
 
@@ -334,9 +381,30 @@ while (last > first)
       if (h->type == htype_from && s == NULL) s = strchr(h->text, ':') + 1;
       }
     if (s == NULL) return "";
-    while (isspace(*s)) s++; 
+    while (isspace(*s)) s++;
     if (choplen != NULL) *choplen = 1;     /* Disregard final \n in header */
     return s;
+
+    /****** This code was for an idea that didn't quite make it. Keep it
+    around in case it gets revived one day. ******/
+
+    #ifdef NEVER
+    case vtype_recipients:                     /* Get recipients list */
+      {
+      address_item *addr;
+      int size = 128;
+      int ptr = 0;
+      s = store_malloc(size);
+      for (addr = (address_item *)(var_table[middle].value); addr != NULL;
+           addr = addr->next)
+        {
+        if (ptr != 0) s = string_cat(s, &size, &ptr, " ", 1);
+        s = string_cat(s, &size, &ptr, addr->orig, (int)strlen(addr->orig));
+        }
+      s[ptr] = 0;                             /* string_cat leaves room */
+      }
+    return s;
+    #endif    /* NEVER */
     }
 
   else if (c > 0) first = middle + 1;
@@ -353,15 +421,25 @@ return NULL;
 *          Find the value of a header            *
 *************************************************/
 
-/* NULL is returned if the header does not exist. The common case
-involves only one header, so no need to get store. Use a flag to
-distinguish so the caller can free it. It's set FALSE on entry. */
+/*
+Arguments:
+  name       the name of the header, without the leading $header_ or $h_
+  freestore  a pointer to a BOOL which is set TRUE if the result is a
+             concatenation of several headers in a new bit of store which
+             the caller must subsequently free; it should be set FALSE on
+             entry
 
-static char *find_header(char *name, BOOL *freestore)
+Returns:     NULL if the header does not exist, else
+             a pointer to the header's contents
+*/
+
+static char *
+find_header(char *name, BOOL *freestore)
 {
 int len = (int)strlen(name);
 char *yield = NULL;
 header_line *h;
+
 for (h = header_list; h != NULL; h = h->next)
   {
   if (h->type != htype_old)
@@ -391,10 +469,23 @@ return yield;
 
 
 /*************************************************
-*            Read a condition                    *
+*        Read and evaluate a condition           *
 *************************************************/
 
-static char *eval_condition(char *s, BOOL *yield)
+/*
+Arguments:
+  s        points to the start of the condition text
+  yield    points to a BOOL to hold the result of the condition test;
+           if NULL, we are just reading through a condition that is
+           part of an "or" combination to check syntax, or in a state
+           where the answer isn't required
+
+Returns:   a pointer to the first character after the condition, or
+           NULL after an error
+*/
+
+static char *
+eval_condition(char *s, BOOL *yield)
 {
 BOOL testfor = TRUE;
 char name[256];
@@ -405,57 +496,111 @@ for (;;)
   if (*s == '!') { testfor = !testfor; s++; } else break;
   }
 
-s = read_name(name, 256, s);
+s = read_name(name, 256, s, "_");
 
 /* def: tests for a non-zero or non-NULL variable */
 
 if (strcmp(name, "def") == 0 && *s == ':')
   {
   char *value;
-  s = read_name(name, 256, s+1);
+  s = read_name(name, 256, s+1, "_");
   value = find_variable(name, NULL);
   if (value == NULL)
     {
     expand_string_message = string_sprintf("unknown variable: %s", name);
     return NULL;
     }
-  *yield = (value[0] != 0 && strcmp(value, "0") != 0) == testfor;
+  if (yield != NULL)
+    *yield = (value[0] != 0 && strcmp(value, "0") != 0) == testfor;
   return s;
   }
-  
+
 /* exists: tests for file existence */
 
 else if (strcmp(name, "exists") == 0)
   {
-  char *filename; 
-  struct stat statbuf; 
+  char *filename;
+  struct stat statbuf;
   while (isspace(*s)) s++;
   if (*s != '{') goto COND_FAILED_CURLY;
   filename = expand_string_internal(s+1, TRUE, &s, FALSE);
-  if (filename == NULL) goto COND_FAILED;
+  if (filename == NULL) return NULL;
   if (*s++ != '}') goto COND_FAILED_CURLY;
-  *yield = (stat(filename, &statbuf) == 0) == testfor;
-  store_free(filename); 
-  return s;  
-  }  
+  if (yield != NULL)
+    *yield = (stat(filename, &statbuf) == 0) == testfor;
+  store_free(filename);
+  return s;
+  }
 
 /* eq: tests for string equality */
 
-else if (strcmp(name, "eq") == 0)
+/* match: does a regular expression match and sets up the numerical
+variables if it succeeds */
+
+else if (strcmp(name, "eq") == 0 || strcmp(name, "match") == 0)
   {
   int i;
+  regexp *re;
+  char *rerror;
   char *sub[2];
+
   for (i = 0; i < 2; i++)
     {
     while (isspace(*s)) s++;
     if (*s != '{') goto COND_FAILED_CURLY;
-    sub[i] = expand_string_internal(s+1, TRUE, &s, FALSE);
-    if (sub[i] == NULL) goto COND_FAILED;
+    sub[i] = expand_string_internal(s+1, TRUE, &s, yield == NULL);
+    if (sub[i] == NULL) return NULL;
     if (*s++ != '}') goto COND_FAILED_CURLY;
     }
-  *yield = (strcmp(sub[0], sub[1]) == 0) == testfor;
-  store_free(sub[0]);
-  store_free(sub[1]);
+
+  /* Result not required */
+
+  if (yield == NULL)
+    {
+    store_free(sub[0]);
+    store_free(sub[1]);
+    return s;
+    }
+
+  /* Straight comparison */
+
+  if (name[0] == 'e')
+    {
+    *yield = (strcmp(sub[0], sub[1]) == 0) == testfor;
+    store_free(sub[0]);
+    store_free(sub[1]);
+    return s;
+    }
+
+  /* Regular expression match */
+
+  rerror = "";
+  regcomp_error_pointer = &rerror;
+  re = regcomp(sub[1]);
+  regcomp_error_pointer = NULL;
+  if (re == NULL)
+    {
+    expand_string_message = string_sprintf("regular expression error in "
+      "\"%s\": %s", sub[1], rerror);
+    return NULL;
+    }
+
+  *yield = regexec(re, sub[0]);
+  if (*yield)
+    {
+    expand_nmax = 0;
+    for (; expand_nmax < NSUBEXP; expand_nmax++)
+      {
+      expand_nstring[expand_nmax] = re->startp[expand_nmax];
+      expand_nlength[expand_nmax] = re->endp[expand_nmax] -
+        expand_nstring[expand_nmax];
+      }
+    expand_nmax--;
+    }
+
+  free(re);   /* NB *not* store_free, as got by regcomp */
+
+  store_free(sub[1]);  /* Can't free sub[0]; pointers point to it */
   return s;
   }
 
@@ -463,35 +608,49 @@ else if (strcmp(name, "eq") == 0)
 
 else if (strcmp(name, "or") == 0 || strcmp(name, "and") == 0)
   {
+  BOOL temp;
+  BOOL *ptr = (yield == NULL)? NULL : &temp;
   BOOL and = strcmp(name, "and") == 0;
   int comb = and;
+
   while (isspace(*s)) s++;
   if (*s++ != '{') goto COND_FAILED_CURLY;
 
   for (;;)
     {
-    BOOL temp;
     while (isspace(*s)) s++;
     if (*s != '{') break;
-    s = eval_condition(s+1, &temp);
+    s = eval_condition(s+1, ptr);
     if (s == NULL) return NULL;
     while (isspace(*s)) s++;
     if (*s++ != '}') goto COND_FAILED_CURLY;
-    if (and) comb &= temp; else comb |= temp;
+    if (yield != NULL)
+      {
+      if (and)
+        {
+        comb &= temp;
+        if (!comb) ptr = NULL;  /* once false, don't evaluate any more */
+        }
+      else
+        {
+        comb |= temp;
+        if (comb) ptr = NULL;   /* once true, don't evaluate any more */
+        }
+      }
     }
 
   if (*s++ != '}') goto COND_FAILED_CURLY;
-  *yield = (comb == testfor);
+  if (yield != NULL) *yield = (comb == testfor);
   return s;
   }
 
-
 /* Unknown type of condition */
 
-COND_FAILED:
-expand_string_message = "unknown condition";
+expand_string_message = string_sprintf("unknown condition \"%s\"", name);
 return NULL;
 
+/* Bracketing error */
+
 COND_FAILED_CURLY:
 expand_string_message = "missing or misplaced { or }";
 return NULL;
@@ -515,6 +674,8 @@ sequences are:
                          extracts keyed substring; null if not found
    ${if cond {s1} {s2}}  conditional; the yielded string is expanded
                          {s2} can be replaced by "fail" or be omitted
+
+One-key+file lookups:
    ${lookup{key}search-type{file}{found}{not-found}}
                          the key, file, & strings are expanded; $value available
                          and {not-found} can be replaced by "fail" or be
@@ -523,15 +684,25 @@ sequences are:
                          which case a subfield is extracted from the found
                          string, which must consist of key=value pairs.
 
+Database-query lookups:
+   ${lookup search-type{query}{found}{not-found}}
+                         the query and strings are expanded; $value available
+                         and {not-found} can be replaced by "fail" or be
+                         omitted.
+
 Operators:
    lc                      lowercase the string
    length_<n>              take first n chars only
-   l_<n>                   ditto  
+   l_<n>                   ditto
+   substr_<m>_<n>          substring n chars from offset m
+   s_<m>_<n>               ditto
+   expand                  expands the string one more time
 
 Conditions:
    !cond                   negates condition
    def:variable            variable is defined and not empty
-   exists {string}         file exists 
+   exists {string}         file exists
+   match {string}{re}      regular expression match
    eq {string1}{string2}   strings are equal, case included
    or {{cond1}{cond2}...}  as it says
    and {{cond1}{cond2}...} ditto
@@ -544,10 +715,24 @@ variable expand_string_forcedfail, which is TRUE in the latter case.
 
 The skipping flag is set true when expanding a substring that isn't actually
 going to be used (after "if" or "lookup") and it prevents lookups from
-happening lower down. */
+happening lower down.
 
-static char *expand_string_internal(char *string, BOOL ket_ends, char **left,
-  BOOL skipping)
+Arguments:
+  string         the string to be expanded
+  ket_ends       true if expansion is to stop at }
+  left           if not NULL, a pointer to the first character after the
+                 expansion is placed here (typically used with ket_ends)
+  skipping       TRUE for recursive calls when the value isn't actually going
+                 to be used (to allow for optimisation)
+
+Returns:         NULL if expansion fails:
+                   expand_string_forcefail is set TRUE if failure was forced
+                   expand_string_message contains a textual error message
+                 a pointer to the expanded string on success
+*/
+
+static char *
+expand_string_internal(char *string, BOOL ket_ends, char **left, BOOL skipping)
 {
 int ptr = 0;
 int size = sizeof(string) + 50;
@@ -599,7 +784,7 @@ while (*s != 0)
     {
     BOOL freestore = FALSE;
     int choplen = 0;
-    s = read_name(name, 256, s);
+    s = read_name(name, 256, s, "_");
     if (strncmp(name, "header_", 7) == 0 || strncmp(name, "h_", 2) == 0)
       {
       s = read_header_name(name, 256, s);
@@ -666,15 +851,29 @@ while (*s != 0)
     expand_string_message = "letter or digit expected after ${";
     goto EXPAND_FAILED;
     }
-  s = read_name(name, 256, s);
+  s = read_name(name, 256, s, "_");
 
-  /* Handle conditionals */
+  /* Handle conditionals - preserve the values of the numerical expansion
+  variables in case they get changed by a regular expression match in the
+  condition. If not, they retain their external settings. At the end
+  of this "if" section, they get restored to their previous values. */
 
   if (strcmp(name, "if") == 0)
     {
-    BOOL cond;
+    BOOL cond = TRUE;
     char *sub1, *sub2;
-    s = eval_condition(s, &cond);
+    char *save_expand_nstring[EXPAND_MAXN+1];
+    int save_expand_nlength[EXPAND_MAXN+1];
+    int save_expand_nmax = expand_nmax;
+    int i;
+
+    for (i = 0; i <= expand_nmax; i++)
+      {
+      save_expand_nstring[i] = expand_nstring[i];
+      save_expand_nlength[i] = expand_nlength[i];
+      }
+
+    s = eval_condition(s, skipping? NULL : &cond);
     if (s == NULL) goto EXPAND_FAILED;  /* message already set */
 
     /* The condition must be followed by one or two substrings,
@@ -683,7 +882,7 @@ while (*s != 0)
 
     while (isspace(*s)) s++;
     if (*s != '{') goto EXPAND_FAILED_CURLY;
-    sub1 = expand_string_internal(s+1, TRUE, &s, FALSE);
+    sub1 = expand_string_internal(s+1, TRUE, &s, skipping);
     if (sub1 == NULL) goto EXPAND_FAILED;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
 
@@ -691,18 +890,18 @@ while (*s != 0)
     if (*s == '}') sub2 = NULL;
     else if (*s == '{')
       {
-      sub2 = expand_string_internal(s+1, TRUE, &s, FALSE);
+      sub2 = expand_string_internal(s+1, TRUE, &s, skipping);
       if (sub2 == NULL) goto EXPAND_FAILED;
       if (*s++ != '}') goto EXPAND_FAILED_CURLY;
       }
     else
       {
-      s = read_name(name, 256, s);
+      s = read_name(name, 256, s, "_");
       if (strcmp(name, "fail") == 0)
         {
         if (!cond)
           {
-          expand_string_message = "if failed and \"fail\" requested";
+          expand_string_message = "\"if\" failed and \"fail\" requested";
           expand_string_forcedfail = TRUE;
           goto EXPAND_FAILED;
           }
@@ -727,27 +926,61 @@ while (*s != 0)
 
     store_free(sub1);
     if (sub2 != NULL) store_free(sub2);
+
+    /* Restore external setting of expansion variables */
+
+    expand_nmax = save_expand_nmax;
+    for (i = 0; i <= expand_nmax; i++)
+      {
+      expand_nstring[i] = save_expand_nstring[i];
+      expand_nlength[i] = save_expand_nlength[i];
+      }
+
     continue;
     }
 
   /* Handle database lookups. If "skipping" is TRUE, we are expanding an
   internal string that isn't actually going to be used. All we need to
-  do is check the syntax, so don't do a lookup at all. */
+  do is check the syntax, so don't do a lookup at all. Preserve the values
+  of the numerical expansion variables in case they get changed by a partial
+  lookup. If not, they retain their external settings. At the end
+  of this "lookup" section, they get restored to their previous values. */
 
   if (strcmp(name, "lookup") == 0)
     {
-    void *handle;
-    char *key, *filename, *sub1, *sub2;
     int stype;
+    int pv = 0;
+    void *handle;
+    char *ss, *key, *filename, *sub1, *sub2;
+    char *lookup_errmsg = NULL;
+    char *save_lookup_value = lookup_value;
 
-    /* Get the key we are to look up. */
+    char *save_expand_nstring[EXPAND_MAXN+1];
+    int save_expand_nlength[EXPAND_MAXN+1];
+    int save_expand_nmax = expand_nmax;
+    int expand_setup = 0;
+    int i;
+
+    for (i = 0; i <= expand_nmax; i++)
+      {
+      save_expand_nstring[i] = expand_nstring[i];
+      save_expand_nlength[i] = expand_nlength[i];
+      }
+
+    lookup_value = NULL;
+
+    /* Get the key we are to look up for single-key+file style lookups.
+    Otherwise set the key NULL pro-tem. */
 
     while (isspace(*s)) s++;
-    if (*s != '{') goto EXPAND_FAILED_CURLY;
-    key = expand_string_internal(s+1, TRUE, &s, FALSE);
-    if (key == NULL) goto EXPAND_FAILED;
-    if (*s++ != '}') goto EXPAND_FAILED_CURLY;
-    while (isspace(*s)) s++;
+    if (*s == '{')
+      {
+      key = expand_string_internal(s+1, TRUE, &s, FALSE);
+      if (key == NULL) goto EXPAND_FAILED;
+      if (*s++ != '}') goto EXPAND_FAILED_CURLY;
+      while (isspace(*s)) s++;
+      }
+    else key = NULL;
 
     /* Find out the type of database */
 
@@ -756,26 +989,49 @@ while (*s != 0)
       expand_string_message = "missing lookup type";
       goto EXPAND_FAILED;
       }
-    s = read_name(name, 256, s);
+    s = read_name(name, 256, s, "-_");
+    ss = name;
 
-    if (strcmp(name, "lsearch") == 0) stype = stype_lsearch;
-    else if (strcmp(name, "dbm") == 0) stype = stype_dbm;
-    else if (strcmp(name, "nis") == 0) 
+    if (strncmp(ss, "partial", 7) == 0)
+      {
+      ss += 7;
+      if (isdigit (*ss))
+        while (isdigit(*ss)) pv = pv*10 + *ss++ - '0';
+      else pv = 2;
+      if (*ss++ != '-')
+        {
+        expand_string_message = string_sprintf("unknown lookup type \"%s\"",
+          name);
+        goto EXPAND_FAILED;
+        }
+      }
+
+    if (strcmp(ss, "lsearch") == 0) stype = stype_lsearch;
+    else if (strcmp(ss, "dbm") == 0) stype = stype_dbm;
+    else if (strcmp(ss, "nis") == 0)
       {
       if (have_nis) stype = stype_nis; else
         {
         expand_string_message = "lookup type \"nis\" unavailable";
-        goto EXPAND_FAILED;  
-        }  
-      } 
-    else if (strcmp(name, "nis0") == 0) 
+        goto EXPAND_FAILED;
+        }
+      }
+    else if (strcmp(ss, "nis0") == 0)
       {
       if (have_nis) stype = stype_nis0; else
         {
         expand_string_message = "lookup type \"nis0\" unavailable";
-        goto EXPAND_FAILED;  
-        }  
-      } 
+        goto EXPAND_FAILED;
+        }
+      }
+    else if (strcmp(ss, "nisplus") == 0)
+      {
+      if (have_nisplus) stype = stype_nisplus; else
+        {
+        expand_string_message = "lookup type \"nisplus\" unavailable";
+        goto EXPAND_FAILED;
+        }
+      }
     else
       {
       expand_string_message = string_sprintf("unknown lookup type \"%s\"",
@@ -783,7 +1039,38 @@ while (*s != 0)
       goto EXPAND_FAILED;
       }
 
-    /* Get the file name */
+    /* Check that a key was provided for those lookup types that need it,
+    and was not supplied for those that use the query style, and that
+    "partial" was provided only for a non-query lookup. */
+
+    if (stype < stype_querystyle)
+      {
+      if (key == NULL)
+        {
+        expand_string_message = string_sprintf("missing {key} for single-"
+          "key \"%s\" lookup", name);
+        goto EXPAND_FAILED;
+        }
+      }
+    else
+      {
+      if (pv > 0)
+        {
+        expand_string_message = string_sprintf("\"partial\" is not permitted "
+          "for lookup type \"%s\"", ss);
+        goto EXPAND_FAILED;
+        }
+
+      if (key != NULL)
+        {
+        expand_string_message = string_sprintf("a single key was given for "
+          "lookup type \"%s\", which is not a single-key lookup type", name);
+        goto EXPAND_FAILED;
+        }
+      }
+
+    /* Get the next string in quotes and expand it. It is the file name for
+    single-key+file lookups, and the whole query otherwise. */
 
     while (isspace(*s)) s++;
     if (*s != '{') goto EXPAND_FAILED_CURLY;
@@ -792,21 +1079,41 @@ while (*s != 0)
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
     while (isspace(*s)) s++;
 
+    /* If this isn't a single-key+file lookup, re-arrange the variables
+    to be appropriate for the search_ functions. */
+
+    if (key == NULL)
+      {
+      key = filename;
+      filename = NULL;
+      }
+
     /* If skipping, don't do the next bit - it has the result of
-    leaving lookup_value == NULL, as if the entry was not found. */
+    leaving lookup_value == NULL, as if the entry was not found. The
+    search_close() function is not called here, leaving any files open.
+    This is deliberate; at suitable places in higher logic, search_tidyup()
+    is called to tidy all open files. This can save opening the same file
+    several times.
+
+    Request that a partial search sets up $1 and maybe $2 by passing
+    expand_setup containing zero. If its value changes, reset expand_nmax,
+    since new variables will have been set. Note that at the end of this
+    "lookup" section, the old numeric variables are restored. */
 
     if (!skipping)
       {
       handle = search_open(filename, stype, 0, NULL, NULL,
-        &expand_string_message, &expand_tree);
+        &expand_string_message);
       if (handle == NULL) goto EXPAND_FAILED;
-      lookup_value = search_find(handle, filename, key, stype);
+      lookup_value = search_find(handle, filename, key, stype,
+        pv, &expand_setup, &lookup_errmsg);
+      if (expand_setup > 0) expand_nmax = expand_setup;
       }
 
-    /* The key and file name strings are no longer needed */
+    /* The file name string is no longer needed; the key might be because
+    numerical variable values may point into it. */
 
-    store_free(key);
-    store_free(filename);
+    if (filename != NULL) store_free(filename);
 
     /* Expand the first substring; $value will get the looked up value. */
 
@@ -839,12 +1146,15 @@ while (*s != 0)
 
     else
       {
-      s = read_name(name, 256, s);
+      s = read_name(name, 256, s, "_");
       if (strcmp(name, "fail") == 0)
         {
         if (lookup_value == NULL)
           {
-          expand_string_message = "lookup failed and \"fail\" requested";
+          expand_string_message =
+            string_sprintf("lookup failed and \"fail\" requested%s%s",
+              (lookup_errmsg == NULL)? "" : ": ",
+              (lookup_errmsg == NULL)? "" : lookup_errmsg);
           expand_string_forcedfail = TRUE;
           goto EXPAND_FAILED;
           }
@@ -860,16 +1170,22 @@ while (*s != 0)
     while (isspace(*s)) s++;
     if (*s++ != '}') goto EXPAND_FAILED_CURLY;
 
-    /* Free store and continue */
+    /* Restore external setting of expansion variables */
 
-    store_free(sub1);
-    if (sub2 != NULL) store_free(sub2);
-    if (lookup_value != NULL)
+    expand_nmax = save_expand_nmax;
+    for (i = 0; i <= expand_nmax; i++)
       {
-      store_free(lookup_value);
-      lookup_value = NULL;
+      expand_nstring[i] = save_expand_nstring[i];
+      expand_nlength[i] = save_expand_nlength[i];
       }
 
+    /* Free store and continue */
+
+    store_free(key);
+    store_free(sub1);
+    if (sub2 != NULL) store_free(sub2);
+    if (lookup_value != NULL) store_free(lookup_value);
+    lookup_value = save_lookup_value;
     continue;
     }
 
@@ -908,8 +1224,26 @@ while (*s != 0)
     if (sub == NULL) goto EXPAND_FAILED;
     s++;
 
+    /* expand expands another time */
+
+    if (strcmp(name, "expand") == 0)
+      {
+      char *expanded = expand_string_internal(sub, FALSE, NULL, FALSE);
+      if (expanded == NULL)
+        {
+        expand_string_message =
+          string_sprintf("internal expansion of \"%s\" failed: %s", sub,
+            expand_string_message);
+        goto EXPAND_FAILED;
+        }
+      yield = string_cat(yield, &size, &ptr, expanded, (int)strlen(expanded));
+      store_free(expanded);
+      store_free(sub);
+      continue;
+      }
+
     /* lc lowercases */
-     
+
     if (strcmp(name, "lc") == 0)
       {
       int count = 0;
@@ -919,33 +1253,45 @@ while (*s != 0)
       store_free(sub);
       continue;
       }
-      
+
     /* length_n or l_n takes just the first n characters or the whole
-    string, whichever is the shorter. */
-    
-    if (strncmp(name, "length_", 7) == 0 || strncmp(name, "l_", 2) == 0)
+    string, whichever is the shorter; substr_m_n or s_m_n take n characters
+    from offset m; l_n is synonymous with s_0_n. */
+
+    if (strncmp(name, "length_", 7) == 0 || strncmp(name, "l_", 2) == 0 ||
+        strncmp(name, "substr_", 7) == 0 || strncmp(name, "s_", 2) == 0)
       {
-      int len = 0; 
-      int sublen = (int)strlen(sub); 
+      int len = 0;
+      int offset = 0;
+      int *pn = (name[0] == 's')? &offset : &len;
+      int sublen = (int)strlen(sub);
       char *num = strchr(name, '_') + 1;
+
       while (*num != 0)
         {
-        if (!isdigit(*num))
+        if (*num == '_' && pn == &offset)
+          {
+          pn = &len;
+          num++;
+          }
+        else if (!isdigit(*num))
           {
           expand_string_message =
             string_sprintf("non-digit after underscore in \"%s\"", name);
           goto EXPAND_FAILED;
-          }       
-        len = len*10 + *num++ - '0';
-        }  
-      if (sublen > len) sub[len] = 0; else len = sublen;
-      yield = string_cat(yield, &size, &ptr, sub, len);
+          }
+        else *pn = (*pn)*10 + *num++ - '0';
+        }
+
+      if (sublen < offset + len) len = sublen - offset;
+      if (len > 0)
+        yield = string_cat(yield, &size, &ptr, sub + offset, len);
       store_free(sub);
-      continue; 
-      }   
+      continue;
+      }
 
     /* Unknown operator */
-     
+
     expand_string_message =
       string_sprintf("unknown expansion operator \"%s\"", name);
     goto EXPAND_FAILED;
@@ -1003,10 +1349,19 @@ store_free(yield);
 return NULL;
 }
 
-/* This is the external function call. */
 
-char *expand_string(char *string)
-  { return expand_string_internal(string, FALSE, NULL, FALSE); }
+/* This is the external function call.
+
+Argument: the string to be expanded
+Returns:  the expanded string, or NULL if expansion failed
+*/
+
+char *
+expand_string(char *string)
+{
+lookup_value = NULL;
+return expand_string_internal(string, FALSE, NULL, FALSE);
+}
 
 
 
@@ -1017,9 +1372,24 @@ char *expand_string(char *string)
 *************************************************/
 
 #ifdef STAND_ALONE
+
+void
+regerror(char *s)
+{
+if (regcomp_error_pointer == NULL)
+  log_write(0, LOG_PANIC_DIE, "regular expression error: %s%s%s", s,
+    (regexp_compiling == NULL)? "" : " while compiling ",
+    (regexp_compiling == NULL)? "" : regexp_compiling);
+else *regcomp_error_pointer = string_copy(s);
+}
+
+
 int main(void)
 {
 char buffer[256];
+debug_level = 1;
+debug_file = stderr;
+big_buffer = malloc(big_buffer_size);
 
 printf("Testing string expansion\n");
 
