@@ -2,15 +2,14 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
-/* This aren't macros, but they are the sort of general definition
-that fits in this file and we need to have them defined early. */
+/* This isn't a macro, but it's the sort of general definition
+that fits in this file and we need to have it defined early. */
 
 typedef int BOOL;
-typedef int IP_ADDRESS;
 
 /* Define which ends of pipes are for reading and writing, as some systems
 don't make both fd's two-way. */
@@ -18,10 +17,10 @@ don't make both fd's two-way. */
 #define pipe_read  0
 #define pipe_write 1
 
-/* The EDQUOT error code isn't universally available, though it is widespread. 
-There is a particular shambles in SunOS5, where it did not exist originally, 
-but got installed with a particular patch for Solaris 2.4. There is a 
-configuration variable for specifying what the system's "over quota" error is, 
+/* The EDQUOT error code isn't universally available, though it is widespread.
+There is a particular shambles in SunOS5, where it did not exist originally,
+but got installed with a particular patch for Solaris 2.4. There is a
+configuration variable for specifying what the system's "over quota" error is,
 but if it is not set, default to EDQUOT if it exists, otherwise ENOSPC. */
 
 #ifndef ERRNO_QUOTA
@@ -32,6 +31,10 @@ but if it is not set, default to EDQUOT if it exists, otherwise ENOSPC. */
 #endif
 #endif
 
+/* Simplify taking the larger of two values */
+
+#define mac_max(a, b) ((a) > (b) ? (a) : (b))
+
 /* The way of finding out the maximum file descriptor various between OS.
 Most have sysconf(), but a few don't. */
 
@@ -43,7 +46,7 @@ Most have sysconf(), but a few don't. */
   #define mac_maxfd (NOFILE - 1)
 #else
   #define mac_maxfd 255;    /* just in case */
-#endif            
+#endif
 
 /* Some generalities */
 
@@ -52,7 +55,7 @@ Most have sysconf(), but a few don't. */
 #define FALSE         0
 #define TRUE          1
 
-/* The size of buffer to get for constructing log entries. Make it big 
+/* The size of buffer to get for constructing log entries. Make it big
 enough to hold all the headers from a normal kind of message. */
 
 #define LOG_BUFFER_SIZE 8192
@@ -62,15 +65,11 @@ sprinting into before getting the correct store. */
 
 #define BIG_BUFFER_SIZE 1024
 
-/* The maximum size of header line we are prepared to process */
-
-#define HEADER_MAXLENGTH 8192
-
 /* A limit to the length of a sender address */
 
 #define SENDER_ADDRESS_MAXLENGTH 512
 
-/* The length of message identification strings. This is the id used internally 
+/* The length of message identification strings. This is the id used internally
 by exim. The external version for use in Received: strings has a leading 'E'
 added to ensure it starts with a letter. */
 
@@ -103,10 +102,10 @@ enum { tod_log, tod_full, tod_bsdin };
 /* For identifying which kind of driver deferred */
 
 enum {
-  DTYPE_DIRECTOR, 
+  DTYPE_DIRECTOR,
   DTYPE_ROUTER,
   DTYPE_TRANSPORT
-};     
+};
 
 /* Error numbers for generating error messages */
 
@@ -117,10 +116,11 @@ enum {
   ERRMESS_BADNOADDRESS,     /* Bad address via -t, leaving none */
   ERRMESS_SPOOLWRITE,       /* Write to spool failed */
   ERRMESS_MESSAGEREAD,      /* Read from input file failed */
-  ERRMESS_VLONGHEADER,      /* Excessively long header */ 
-  ERRMESS_TOOBIG            /* Message too big */ 
+  ERRMESS_VLONGHEADER,      /* Excessively long header */
+  ERRMESS_TOOBIG,           /* Message too big */
+  ERRMESS_TOOMANYRECIP      /* Too many recipients */
 };
-  
+
 /* Error handling styles - set by option, and apply only when receiving
 a local message. */
 
@@ -136,9 +136,9 @@ enum {
   OK,                       /* Success */
   DEFER,                    /* Failed soft */
   FAIL,                     /* Failed hard */
-  FORCEFAIL,                /* Failed, and don't pass on */ 
-  ISLOCAL,                  /* Remote address turned out to be local */ 
-  PASS,                     /* Pass to next - routers only */ 
+  FORCEFAIL,                /* Failed, and don't pass on */
+  ISLOCAL,                  /* Remote address turned out to be local */
+  PASS,                     /* Pass to next - routers only */
   ERROR,                    /* Soft failed with internal or config error */
   PANIC                     /* Hard failed with internal error */
 };
@@ -165,14 +165,25 @@ to conflict with system errno values. */
 #define ERRNO_BADALIAS     (-17)
 #define ERRNO_SMTPCLOSED   (-18)
 #define ERRNO_SMTPFORMAT   (-19)
-#define ERRNO_NOHOSTS      (-20)
-#define ERRNO_SPOOLFORMAT  (-21)
+#define ERRNO_SPOOLFORMAT  (-20)
+#define ERRNO_NOTABSOLUTE  (-21)
+#define ERRNO_EXIMQUOTA    (-22)   /* Exim-imposed quota */
+
+/* These must be last, so all retry deferments can easily be identified */
+
+#define ERRNO_RETRY_BASE   (-22)   /* Base to test against */
+#define ERRNO_DRETRY       (-22)   /* Not time for directing */
+#define ERRNO_RRETRY       (-23)   /* Not time for routing */
+#define ERRNO_LRETRY       (-24)   /* Not time for local delivery */
+#define ERRNO_HRETRY       (-25)   /* Not time for any remote host */
+#define ERRNO_LOCAL_ONLY   (-26)   /* Local-only delivery */
 
 /* Special actions to take after failure or deferment. */
 
 enum {
   SPECIAL_NONE,             /* No special action */
-  SPECIAL_FREEZE            /* Freeze message */
+  SPECIAL_FREEZE,           /* Freeze message */
+  SPECIAL_FAIL              /* Fail the delivery */
 };
 
 
@@ -193,25 +204,44 @@ used so they can easily be seen in the spool file. */
 /* Offsets into the tables of names for headers (normal & resent) */
 
 enum { hn_bcc, hn_cc, hn_date, hn_from, hn_msgid, hn_sender, hn_to,
-  hn_replyto };
+  hn_replyto, hn_subject };
 
 /* Types of item in options lists */
 
-enum { opt_stringptr, opt_lcstringptr, opt_transportptr, opt_int, 
-  opt_octint, opt_mkint, opt_time, opt_bool, opt_uid, opt_gid, 
-  opt_uidlist, opt_gidlist, opt_expand_uid, opt_expand_gid, 
-  opt_searchtype, opt_local_smtp };
-  
-/* Search types for alias files etc. */
+enum { opt_stringptr, opt_lcstringptr, opt_transportptr, opt_int,
+  opt_octint, opt_mkint, opt_fixed, opt_time, opt_bool, opt_bool_verify,
+  opt_uid, opt_gid, opt_uidlist, opt_gidlist, opt_expand_uid, opt_expand_gid,
+  opt_searchtype, opt_local_batch };
 
-enum { stype_lsearch, stype_dbm, stype_nis, stype_nis0, stype_nisplus }; 
+/* There's a high-ish bit which is used to flag duplicate options, kept
+for compatibility, which shouldn't be output. Also used for hidden options
+that are automatically maintained from others. Another high bit is used to
+flag driver options that although private (so as to be settable only on some
+drivers), are stored in the instance block so as to be accessible from outside.
+*/
+
+#define opt_hidden  0x100
+#define opt_public  0x200
+#define opt_mask    0x0ff
+
+/* Verify types when directing and routing */
+
+enum { v_none, v_sender, v_recipient };
+
+/* Search types for alias files etc. The single-key+file types must
+come first, with all the "query" types last. Start the values at a printing
+characters, as they are concatenated with file names in the caching tree. */
+
+enum {
+  stype_lsearch='0', stype_dbm, stype_nis, stype_nis0,   /* key+file types */
+  stype_querystyle, stype_nisplus };                     /* query types */
 
 /* Status values for host_item blocks. Require hstatus_unusable and
 hstatus_unusable_expired to be last. */
 
 enum { hstatus_unknown, hstatus_usable, hstatus_unusable,
        hstatus_unusable_expired };
-       
+
 /* Reasons why a host is unusable (for clearer log messages) */
 
 enum { hwhy_unknown, hwhy_retry, hwhy_failed, hwhy_deferred };
@@ -220,17 +250,28 @@ enum { hwhy_unknown, hwhy_retry, hwhy_failed, hwhy_deferred };
 
 enum { lk_pass, lk_byname, lk_bydns, lk_bydns_a, lk_bydns_mx };
 
+/* Values for the self_code fields */
+
+enum { self_freeze, self_defer, self_send, self_local,
+  self_fail, self_forcefail };
+
 /* Flags for rewrite rules */
 
-#define rewrite_sender  0x01
-#define rewrite_from    0x02
-#define rewrite_to      0x04
-#define rewrite_cc      0x08
-#define rewrite_bcc     0x10
-#define rewrite_replyto 0x20
-#define rewrite_envfrom 0x40
-#define rewrite_envto   0x80
-#define rewrite_all     0xFF
+#define rewrite_sender       0x001
+#define rewrite_from         0x002
+#define rewrite_to           0x004
+#define rewrite_cc           0x008
+#define rewrite_bcc          0x010
+#define rewrite_replyto      0x020
+#define rewrite_all_headers  0x03F  /* all header flags */
+
+#define rewrite_envfrom      0x040
+#define rewrite_envto        0x080
+#define rewrite_all_envelope 0x0C0  /* all envelope flags */
+
+#define rewrite_all      (rewrite_all_headers | rewrite_all_envelope)
+
+#define rewrite_whole   0x100   /* option bit for headers */
 
 /* Flags for log_write() */
 
@@ -240,29 +281,43 @@ enum { lk_pass, lk_byname, lk_bydns, lk_bydns_a, lk_bydns_mx };
 #define LOG_PROCESS     8    /* Write to the process log */
 #define LOG_REJECT     16    /* Write to the reject log, with headers */
 #define LOG_RECIPIENTS 32    /* Add recipients to the message */
+#define LOG_CONFIG     64    /* Add "Exim configuration error:\n" */
+#define LOG_CONFIG2   128    /* Add "Exim configuration error for" */
 
 /* Returns from host_find_by{name,dns}() */
 
 enum {
   HOST_FIND_FAILED,     /* failed to find the host */
-  HOST_FIND_AGAIN,      /* could not resolve at this time */ 
+  HOST_FIND_AGAIN,      /* could not resolve at this time */
   HOST_FOUND,           /* found host */
-  HOST_FOUND_LOCAL      /* found, but MX points to local host */ 
-};   
+  HOST_FOUND_LOCAL      /* found, but MX points to local host */
+};
 
 /* Actions applied to specific messages. */
 
 enum { MSG_DELIVER, MSG_FREEZE, MSG_REMOVE, MSG_THAW, MSG_ADD_RECIPIENT,
-       MSG_MARK_DELIVERED, MSG_EDIT_SENDER, MSG_EDIT_BODY };
+       MSG_MARK_ALL_DELIVERED, MSG_MARK_DELIVERED, MSG_EDIT_SENDER,
+       MSG_EDIT_BODY };
 
-/* Options for local batched SMTP deliveries. */
+/* Options for local batched SMTP deliveries. Must be in the same order
+as for non-SMTP local batching below. */
 
 enum {
   local_smtp_off,       /* not doing batched SMTP */
   local_smtp_one,       /* each address separate */
   local_smtp_domain,    /* batch identical domains */
   local_smtp_all        /* batch all addresses */
-};      
+};
+
+/* Options for non-SMTP local batched deliveries - must be in the same
+order as for SMTP local batching above. */
+
+enum {
+  local_batch_off,       /* not doing batched delivery */
+  local_batch_one,       /* each address separate */
+  local_batch_domain,    /* batch identical domains */
+  local_batch_all        /* batch all addresses */
+};
 
 /* Returns from the spool_read_header() function */
 
@@ -271,6 +326,15 @@ enum {
   spool_read_notopen,   /* open failed */
   spool_read_enverror,  /* error in the envelope */
   spool_read_hdrerror   /* error in the headers */
-};      
+};
+
+/* Options for transport_write_message */
+
+#define topt_add_return_path    0x01
+#define topt_add_delivery_date  0x02
+#define topt_add_envelope_to    0x04
+#define topt_from_hack          0x08
+#define topt_use_crlf           0x10
+#define topt_smtp_dots          0x20
 
 /* End of macros.h */
