@@ -2,21 +2,23 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
+/* Header for the domainlist router */
 
-/* Private structure for the private options. */
+/* Structure for the private options. */
 
 typedef struct {
-  void *handle;
   int   search_type;
+  int   partial_match;
   int   modemask;
   int  *owners;
-  int  *owngroups;   
-  char *route_file; 
+  int  *owngroups;
+  char *route_file;
+  char *route_query;
   char *route_list;
-  re_block *re_list; 
+  re_block *re_list;
 } domainlist_router_options_block;
 
 /* Data for reading the private options. */
@@ -28,13 +30,10 @@ extern int domainlist_router_options_count;
 
 extern domainlist_router_options_block domainlist_router_option_defaults;
 
-/* The main, initialization, and tidyup entry points for the router */
+/* The main and initialization entry points for the router */
 
-extern int domainlist_router_entry(router_instance *, address_item *, 
+extern int domainlist_router_entry(router_instance *, address_item *,
   address_item **, address_item **);
-   
-extern void domainlist_router_init(router_instance *); 
-
-extern void domainlist_router_tidyup(router_instance *);
+extern void domainlist_router_init(router_instance *);
 
 /* End of routers/domainlist.h */
