@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for reading spool files. */
@@ -20,10 +20,17 @@
 can get replaced during delivery because of header rewriting. The file has
 to opened with write access so that we can get an exclusive lock, but in
 fact it won't be written to. Just in case there's a major disaster (e.g.
-overwriting some other file descriptor with the value of this one), open it 
-with append. */
+overwriting some other file descriptor with the value of this one), open it
+with append.
 
-BOOL spool_open_datafile(char *id)
+Argument: the id of the message
+Returns:  TRUE if file successfully opened and locked
+
+Side effect: deliver_datafile is set to the fd of the open file.
+*/
+
+BOOL
+spool_open_datafile(char *id)
 {
 struct stat statbuf;
 flock_t lock_data;
@@ -36,10 +43,10 @@ if (deliver_datafile < 0)
   {
   if (errno == ENOENT)
     {
-    log_write(LOG_MAIN, "Spool file %s-D not found", id);
+    log_write(0, LOG_MAIN, "Spool file %s-D not found", id);
     return FALSE;
     }
-  log_write(LOG_MAIN, "Spool error for %s: %s", spoolname, strerror(errno));
+  log_write(0, LOG_MAIN, "Spool error for %s: %s", spoolname, strerror(errno));
   return FALSE;
   }
 
@@ -49,9 +56,9 @@ lock_data.l_whence = lock_data.l_start = lock_data.l_len = 0;
 if (fcntl(deliver_datafile, F_SETLK, &lock_data) < 0)
   {
   if (message_id[0] == 0 || strcmp(message_id, id) != 0)
-    log_write(LOG_MAIN, "Spool file %s-D is locked", id);
+    log_write(5, LOG_MAIN, "Spool file %s-D is locked", id);
   else
-    log_write(LOG_MAIN, "Spool file is locked");
+    log_write(5, LOG_MAIN, "Spool file is locked");
   close(deliver_datafile);
   deliver_datafile = -1;
   return FALSE;
@@ -80,21 +87,37 @@ save the right flag before recursing with the same buffer.
 
 Once the tree is read, we re-construct the balance fields by scanning the tree.
 I forgot to write them out originally, and the compatible fix is to do it this
-way. Another recursing function does the necessary. */
+way. This initial local recursing function does the necessary.
 
+Arguments:
+  node      tree node
 
-static int count_below(tree_node *node, BOOL countleft)
+Returns:    maximum depth below the node, including the node itself
+*/
+
+static int
+count_below(tree_node *node)
 {
 int nleft, nright;
 if (node == NULL) return 0;
-nleft = count_below(node->left, TRUE);
-nright = count_below(node->right, FALSE);
+nleft = count_below(node->left);
+nright = count_below(node->right);
 node->balance = (nleft > nright)? 1 : ((nright > nleft)? 2 : 0);
-return 1 + (countleft? nleft : nright);
+return 1 + ((nleft > nright)? nleft : nright);
 }
 
+/* This is the real function...
 
-static BOOL read_nonrecipients_tree(tree_node **connect, FILE *f, char *buffer)
+Arguments:
+  connect   pointer to the root of the tree
+  f         FILE to read data from
+  buffer    contains next input line; further lines read into it
+
+Returns:    FALSE on format error
+*/
+
+static BOOL
+read_nonrecipients_tree(tree_node **connect, FILE *f, char *buffer)
 {
 tree_node *node;
 int n = (int)strlen(buffer);
@@ -121,7 +144,7 @@ if (right)
   }
 else node->right = NULL;
 
-(void) count_below(*connect, TRUE);
+(void) count_below(*connect);
 return TRUE;
 }
 
@@ -133,26 +156,29 @@ return TRUE;
 *************************************************/
 
 /* This function reads a spool header file and places the data into the
-appropriate global variables. It returns non-zero if there is any error
-while reading the file, or if the file fails to open. There are various
-values indicating what has gone wrong otherwise:
-
-  spool_read_OK        success
-  spool_read_notopen   open failed
-  spool_read_enverror  error in the envelope portion
-  spool_read_hdrdrror  error in the header portion  
-  
-The header portion is always read, but header structures are built only if
-read_headers is set true. It isn't, for example, while generating -bp output. 
+appropriate global variables. The header portion is always read, but header
+structures are built only if read_headers is set true. It isn't, for example,
+while generating -bp output.
 
 It may be possible for blocks of nulls (binary zeroes) to get written on the
 end of a file if there is a system crash during writing. It was observed on an
 earlier version of Exim that omitted to fsync() the files - this is thought to
 have been the cause of that incident, but in any case, this code must be robust
 against such an event, and if such a file is encountered, it must be treated as
-malformed. */
+malformed.
 
-int spool_read_header(char *name, BOOL read_headers)
+Arguments:
+  name          name of the header file, including the -H
+  read_headers  TRUE if in-store header structures are to be built
+
+Returns:        spool_read_OK        success
+                spool_read_notopen   open failed
+                spool_read_enverror  error in the envelope portion
+                spool_read_hdrdrror  error in the header portion
+*/
+
+int
+spool_read_header(char *name, BOOL read_headers)
 {
 FILE *f;
 int n;
@@ -176,19 +202,39 @@ errno = 0;
 
 DEBUG(5) debug_printf("Opened spool file %s\n", name);
 
-/* The first three lines of spool files are in a fixed format. The
-first contains the login, uid, and gid of the user who caused the
-file to be written. The second contains the mail address of the
-message's sender, enclosed in <>. The third contains the time the
-message was received, and the number of warning messages for delivery
-delays that have been sent. */
+/* The first line of a spool file contains the message id followed by -H (i.e.
+the file name), in order to make the file self-identifying. However, versions
+of Exim before 0.57 did not do this, so make this line optional for
+compatibility. Change made in October 1996. After a year or so, the optionality
+can be removed. Set data_start_offset to the length of the message id + 3
+because the identification will also be present in the data file if it is in
+the header file. */
 
-if (fgets(big_buffer, BIG_BUFFER_SIZE, f) == NULL) goto SPOOL_READ_ERROR;
-if (sscanf(big_buffer, "%s %d %d", originator, &originator_uid,
-  &originator_gid) != 3) goto SPOOL_FORMAT_ERROR;
+data_start_offset = 0;
+
+if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
+
+if ((int)strlen(big_buffer) == MESSAGE_ID_LENGTH + 3)
+  {
+  if (strncmp(big_buffer, name, MESSAGE_ID_LENGTH + 2) == 0)
+    {
+    data_start_offset = MESSAGE_ID_LENGTH + 3;
+    if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
+    }
+  }
+
+/* We now have the first proper data line in the big buffer. The first three
+data lines of spool files are in a fixed format. The first contains the login,
+uid, and gid of the user who caused the file to be written. The second contains
+the mail address of the message's sender, enclosed in <>. The third contains
+the time the message was received, and the number of warning messages for
+delivery delays that have been sent. */
+
+if (sscanf(big_buffer, "%s %d %d", originator, (int *)(&originator_uid),
+  (int *)(&originator_gid)) != 3) goto SPOOL_FORMAT_ERROR;
 originator_login = string_copy(originator);
 
-if (fgets(big_buffer, BIG_BUFFER_SIZE, f) == NULL) goto SPOOL_READ_ERROR;
+if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 n = (int)strlen(big_buffer);
 if (n < 3 || big_buffer[0] != '<' || big_buffer[n-2] != '>')
   goto SPOOL_FORMAT_ERROR;
@@ -196,12 +242,12 @@ sender_address = store_malloc(n-2);
 strncpy(sender_address, big_buffer+1, n-3);
 sender_address[n-3] = 0;
 
-if (fgets(big_buffer, BIG_BUFFER_SIZE, f) == NULL) goto SPOOL_READ_ERROR;
+if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 if (sscanf(big_buffer, "%d %d", &received_time, &warning_count) != 2)
   goto SPOOL_FORMAT_ERROR;
 
 DEBUG(5) debug_printf("user=%s uid=%d gid=%d sender=%s\n", originator_login,
-  originator_uid, originator_gid, sender_address);
+  (int)originator_uid, (int)originator_gid, sender_address);
 
 /* Now there may be a number of options lines, each starting with "-".
 First set default values for the options. */
@@ -215,7 +261,7 @@ user_null_sender = FALSE;
 
 for (;;)
   {
-  if (fgets(big_buffer, BIG_BUFFER_SIZE, f) == NULL) goto SPOOL_READ_ERROR;
+  if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
   if (big_buffer[0] != '-') break;
   if (strcmp(big_buffer, "-local\n") == 0) sender_local = TRUE;
     else if (strcmp(big_buffer, "-localerror\n") == 0)
@@ -227,6 +273,8 @@ for (;;)
       deliver_freeze = TRUE;
       deliver_frozen_at = atoi(big_buffer + 7);
       }
+    else if (strcmp(big_buffer, "-manual_thaw\n") == 0)
+      deliver_manual_thaw = TRUE;
     else if (strcmp(big_buffer, "-user_null_sender\n") == 0)
       user_null_sender = TRUE;
 
@@ -252,26 +300,26 @@ if (strncmp(big_buffer, "XX\n", 3) != 0 &&
 
 DEBUG(5)
   {
-  printf("Non-recipients:\n");
-  tree_print(tree_nonrecipients);
+  debug_printf("Non-recipients:\n");
+  tree_print(tree_nonrecipients, debug_file);
   }
 
 /* After reading the tree, the next line has not yet been read into the
 buffer. It contains the count of recipients which follow on separate lines. */
 
-if (fgets(big_buffer, BIG_BUFFER_SIZE, f) == NULL) goto SPOOL_READ_ERROR;
+if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 if (sscanf(big_buffer, "%d", &recipients_count) != 1) goto SPOOL_FORMAT_ERROR;
 
 DEBUG(5) debug_printf("recipients_count=%d\n", recipients_count);
 
-recipients_max = recipients_count;
+recipients_list_max = recipients_count;
 recipients_list = store_malloc(recipients_count * sizeof(char *));
 
 for (n = 0; n < recipients_count; n++)
   {
   int nn;
   char *receiver;
-  if (fgets(big_buffer, BIG_BUFFER_SIZE, f) == NULL) goto SPOOL_READ_ERROR;
+  if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
   nn = (int)strlen(big_buffer);
   if (nn < 2) goto SPOOL_FORMAT_ERROR;
   receiver = store_malloc(nn);
@@ -285,11 +333,11 @@ separated off from the previous data by a blank line. Each header is preceded
 by a count of its length and either a certain letter (for various identified
 headers), space (for a miscellaneous live header) or an asterisk (for a header
 that has been rewritten). Count the Received: headers. We read the headers
-always, in order to check on the format of the file, but only create a header 
+always, in order to check on the format of the file, but only create a header
 list if requested to do so. */
 
 inheader = TRUE;
-if (fgets(big_buffer, BIG_BUFFER_SIZE, f) == NULL) goto SPOOL_READ_ERROR;
+if (fgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 if (big_buffer[0] != '\n') goto SPOOL_FORMAT_ERROR;
 
 while ((n = fgetc(f)) != EOF)
@@ -303,16 +351,17 @@ while ((n = fgetc(f)) != EOF)
   ungetc(n, f);
   fscanf(f, "%d%c ", &n, flag);
   if (!isprint(flag[0])) goto SPOOL_FORMAT_ERROR;
+  message_size += n;
 
   if (read_headers)
-    { 
+    {
     h = store_malloc(sizeof(header_line) + n);
     h->next = NULL;
     h->type = flag[0];
     h->slen = n;
-    
+
     if (h->type == htype_received) received_count++;
-    
+
     if (header_list == NULL)
       {
       header_list = h;
@@ -324,7 +373,7 @@ while ((n = fgetc(f)) != EOF)
       h->prev = header_last;
       }
     header_last = h;
-     
+
     for (i = 0; i < n; i++)
       {
       int c = fgetc(f);
@@ -332,7 +381,21 @@ while ((n = fgetc(f)) != EOF)
       h->text[i] = c;
       }
     h->text[i] = 0;
-    } 
+
+    /* Make the Precedence value available in a variable. This isn't
+    a standard RFC 822 header, but it is in widespread use. */
+
+    if (header_checkname(h, "Precedence", 10))
+      {
+      char *s = h->text + 10;
+      while (isspace(*s)) s++;
+      s++;
+      while (isspace(*s)) s++;
+      message_precedence = string_copy(s);
+      }
+    }
+
+  /* Not requiring header data, just skip through the bytes */
 
   else for (i = 0; i < n; i++)
     {
