@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* General functions concerned with transportation, and generic options for all
@@ -14,15 +14,12 @@ transports. */
 
 
 /* Generic options for transports, all of which live inside transport_instance
-data blocks. There are in fact a couple of options which have generic entries
-in the structure, but are set only by certain transports. These are uid and
-gid, and they are handled by having private options with the same values,
-and copying them into the generic slots at the end of initialization. This
-means they can only be set by some transports, but the values are available at
-the top level. */
+data blocks and which therefore have the opt_public flag set. Note that there
+are other options living inside this structure which can be set only from
+certain transports. */
 
 optionlist optionlist_transports[] = {
-  { "driver",    opt_stringptr,
+  { "driver",    opt_stringptr|opt_public,
                  (void *)offsetof(transport_instance, driver_name) },
 };
 
@@ -39,7 +36,8 @@ instances according to its contents. Each transport has generic options and may
 also have its own private options. This function is only ever called when
 transports == NULL. We use generic code in readconf to do most of the work. */
 
-void transport_init(void)
+void
+transport_init(void)
 {
 transport_instance *t;
 
@@ -68,9 +66,14 @@ for (t = transports; t != NULL; t = t->next)
 
 /* This is called by the any transport that wants to apply timeouts
 to its activities. It then sets transport_chunk_timeout before calling
-transport_write_message() to apply timeouts to individual message chunks. */
+transport_write_message() to apply timeouts to individual message chunks.
 
-void transport_timeout_handler(int sig)
+Argument: the signal value (SIGALRM)
+Returns:  nothing
+*/
+
+void
+transport_timeout_handler(int sig)
 {
 transport_sigalrm_seen = TRUE;
 }
@@ -87,23 +90,34 @@ on some systems at least, if a quota is exceeded *during* the write, the yield
 is the number of bytes written and errno is not set. Therefore we always try to
 write a second time to output the remainder of the data after a non-negative
 return from write() (except after a timeout). This time, no bytes should get
-written, and a proper error should get put into errno. */
+written, and a proper error should get put into errno.
 
-static BOOL write_block(int fd, char *block, int len)
+Arguments:
+  fd        file descriptor to write to
+  block     block of bytes to write
+  len       number of bytes to write
+
+Returns:    TRUE on success, FALSE on failure (with errno preserved)
+*/
+
+static BOOL
+write_block(int fd, char *block, int len)
 {
 int i, rc;
 for (i = 0; i < 2; i++)
   {
+  DEBUG(9)
+    debug_printf("writing data block: timeout=%d\n", transport_chunk_timeout);
   if (transport_chunk_timeout > 0) alarm(transport_chunk_timeout);
   rc = write(fd, block, len);
   if (transport_chunk_timeout > 0) alarm(0);
   if (rc == len) return TRUE;
-  if (rc < 0) return FALSE;  
-  if (transport_sigalrm_seen) 
+  if (rc < 0) return FALSE;
+  if (transport_sigalrm_seen)
     {
-    errno = ETIMEDOUT; 
+    errno = ETIMEDOUT;
     return FALSE;
-    }  
+    }
   len -= rc;
   block += rc;
   }
@@ -131,12 +145,24 @@ in transport_chunk_timeout and sets up transport_timeout_handler() as the
 SIGALRM handler. A non-zero transport_chunk_timeout causes a timer to be set
 for each chunk of data written from here via write_block() above. If time runs
 out, then a write() fails and provokes an error return. The caller can then
-inspect transport_sigalrm_seen to check for a timeout. */
+inspect transport_sigalrm_seen to check for a timeout.
+
+Arguments:
+  fd         file descript to write to
+  chunk      pointer to data to write
+  len        length of data to write
+  from_hack  TRUE if the "from hack" is wanted
+  usr_crlf   TRUE if CR LF is wanted at the end of each line
+  smtp_dots  TRUE if the SMTP dot-escaping algorithm is wanted
+
+Returns:     TRUE on success, FALSE on failure (with errno preserved)
+*/
 
 static BOOL was_nl;     /* NL status flag */
 
-static BOOL write_chunk(int fd, char *chunk, int len, BOOL from_hack,
-  BOOL use_crlf, BOOL smtp_dots)
+static BOOL
+write_chunk(int fd, char *chunk, int len, BOOL from_hack, BOOL use_crlf,
+  BOOL smtp_dots)
 {
 char *start = chunk;
 char *end = chunk + len;
@@ -210,7 +236,7 @@ for (ptr = start; ptr < end; ptr++)
 /* Write out any remaining data in the buffer before returning. */
 
 return (len = outptr - deliver_out_buffer) <= 0 ||
-  write(fd, deliver_out_buffer, len);
+  write_block(fd, deliver_out_buffer, len);
 }
 
 
@@ -222,16 +248,21 @@ return (len = outptr - deliver_out_buffer) <= 0 ||
 
 /* This function writes the message to the given file descriptor. The headers
 are in the in-store data structure, and the rest of the message is in the open
-file descriptor deliver_datafile. Make sure we start it at the beginning. There
-are three options:
+file descriptor deliver_datafile. Make sure we start it at the beginning.
 
 . If add_return_path is TRUE, a "return-path:" header is added to the message,
-  containing the sender's address, unless it is null, or the user_null_sender 
-  flag is set (indicating an error message).
+  containing the sender's address, unless it is null, or the user_null_sender
+  flag is set (indicating an error message), in which case just "<>" is given.
+
+. If add_envelope_to is TRUE, a "envelope-to:" header is added to the message,
+  giving the top-level envelope address that caused this delivery to happen.
+
+. If add_delivery_date is TRUE, a "delivery-date:" header is added to the
+  message. It gives the time and date that delivery took place.
 
 . If from_hack is true, the "from hack" is done to lines starting with the text
   "From " (what a shoddy specification). The from hack is not applied to
-  headers or the prefix or suffix.
+  headers.
 
 . If use_crlf is true, newlines are turned into CRLF (SMTP output).
 
@@ -240,32 +271,87 @@ are three options:
 The yield is TRUE if all went well, and FALSE if not. Exit *immediately* after
 any writing or reading error, leaving the code in errno intact. Error exits
 can include timeouts for certain transports, which are requested by setting
-transport_chunk_timeout non-zero. */
+transport_chunk_timeout non-zero.
 
+Arguments:
+  addr                  (chain of) addresses (for extra headers)
+  fd                    file descriptor to write the message to
+  options               bit-wise options:
+    add_return_path       if TRUE, add a "return-path" header
+    add_envelope_to       if TRUE, add a "envelope-to" header
+    add_delivery_date     if TRUE, add a "delivery-date" header
+    from_hack             if TRUE, apply the "from hack"
+    use_crlf              if TRUE, turn NL into CR LF
+    smtp_dots             if TRUE, implement the SMTP dot escaping algorithm
+  errors_to             if not NULL, used for the return-path header
+  size_limit            if > 0, this is a limit to the size of message written;
+                          it is used when returning messages to their senders,
+                          and is approximate rather than exact, owing to chunk
+                          buffering
 
-BOOL transport_write_message(int fd, BOOL add_return_path,
-  BOOL add_delivery_date, BOOL from_hack, BOOL use_crlf, BOOL smtp_dots,
+Returns:                TRUE on success; FALSE (with errno) on failure
+*/
+
+BOOL
+transport_write_message(address_item *addr, int fd, int options,
   char *errors_to, int size_limit)
 {
 int written = 0;
 int len;
 header_line *h;
+BOOL from_hack = (options & topt_from_hack) != 0;
+BOOL use_crlf  = (options & topt_use_crlf)  != 0;
+BOOL smtp_dots = (options & topt_smtp_dots) != 0;
 
 /* Add return-path: if requested, and there is one. */
 
-if (add_return_path && (errors_to != NULL || 
-    (sender_address[0] != 0 && !user_null_sender)))
+if ((options & topt_add_return_path) != 0)
   {
   char buffer[SENDER_ADDRESS_MAXLENGTH + 20];
   sprintf(buffer, "Return-path: <%s>\n",
-    (errors_to != NULL)? errors_to : sender_address);
+    (errors_to != NULL)? errors_to : user_null_sender? "" : sender_address);
   len = strlen(buffer);
   if (!write_chunk(fd, buffer, len, FALSE, use_crlf, FALSE)) return FALSE;
   }
 
+/* Add envelope-to: if requested, picking up from all the addresses. */
+
+if ((options & topt_add_envelope_to) != 0)
+  {
+  int len;
+  address_item *p;
+  char buffer[256];
+  strcpy(buffer, "Envelope-to: ");
+  len = (int)strlen(buffer);
+
+  for (p = addr; p != NULL; p = p->next)
+    {
+    address_item *pp = p;
+    while (pp->parent != NULL) pp = pp->parent;
+
+    strcpy(buffer + len, pp->orig);
+    len += (int)strlen(pp->orig);
+
+    if (p->next == NULL)
+      {
+      strcat(buffer, "\n");
+      len += 1;
+      }
+    else
+      {
+      strcat(buffer, ",\n ");
+      len += 3;
+      }
+
+    if (!write_chunk(fd, buffer, len, FALSE, use_crlf, FALSE)) return FALSE;
+    buffer[0] = 0;
+    len = 0;
+    }
+  }
+
 /* Add delivery-date: if requested. */
 
-if (add_delivery_date)
+if ((options & topt_add_delivery_date) != 0)
   {
   char buffer[100];
   sprintf(buffer, "Delivery-date: %s\n", tod_stamp(tod_full));
@@ -284,6 +370,18 @@ for (h = header_list; h != NULL; h = h->next)
     return FALSE;
   }
 
+/* Add on any address-specific headers at the end, copying from all
+addresses that are being transported. */
+
+while (addr != NULL)
+  {
+  for (h = addr->extra_headers; h!= NULL; h = h->next)
+    if (!write_chunk(fd, h->text, h->slen, FALSE, use_crlf, smtp_dots))
+      return FALSE;
+  addr = addr->next;
+  }
+
+
 /* Separate headers from body with a blank line; set the static flag for
 the chunk writer. Note: use_crlf won't be in use when writing to a file, so we
 are only every trying to write one byte to a file. If this fails because of
@@ -293,10 +391,10 @@ if (use_crlf? (write(fd, "\r\n", 2) != 2) : (write(fd, "\n", 1) != 1))
   return FALSE;
 was_nl = TRUE;
 
-/* Ensure the body is positioned at the start of its file, then write it,
-applying the size limit if required. */
+/* Ensure the body is positioned at the start of its file, following the
+message id if present, then write it, applying the size limit if required. */
 
-lseek(deliver_datafile, 0, SEEK_SET);
+lseek(deliver_datafile, data_start_offset, SEEK_SET);
 while ((len = read(deliver_datafile,deliver_in_buffer,DELIVER_BUFFER_SIZE)) > 0)
   {
   if (!write_chunk(fd, deliver_in_buffer, len, from_hack, use_crlf, smtp_dots))
@@ -355,10 +453,17 @@ determine whether to add a message id to a given record. This shouldn't be
 too bad except in extreme cases. I can't figure out a *simple* way of doing
 better.
 
-Old records should eventually get swept up by the exim_tidydb utility. */
+Old records should eventually get swept up by the exim_tidydb utility.
 
+Arguments:
+  hostlist  list of hosts that this message could be sent to
+  tpname    name of the transport
 
-void transport_update_waiting(host_item *hostlist, char *tpname)
+Returns:    nothing
+*/
+
+void
+transport_update_waiting(host_item *hostlist, char *tpname)
 {
 char buffer[256];
 char *prevname = "";
@@ -371,7 +476,8 @@ sprintf(buffer, "wait-%s", tpname);
 dbm_file = db_open(buffer, O_RDWR|O_CREAT);
 if (dbm_file == NULL)
   {
-  log_write(LOG_MAIN, "Failed to open %s database", buffer);
+  log_write(0, LOG_MAIN, "Failed to open %s database: %s", buffer,
+    strerror(errno));
   return;
   }
 
@@ -487,25 +593,41 @@ db_close(dbm_file);
 
 /* This function is called by a remote transport which uses the previous
 function to remember which messages are waiting for which remote hosts. It's
-called after a successful delivery and it's job is to check whether there is
-another message waiting for the same host, and if so, spawn a new exim process
-to deliver it. However, it doesn't do this if the current continue sequence
-is greater than the maximum. */
+called after a successful delivery and its job is to check whether there is
+another message waiting for the same host. However, it doesn't do this if the
+current continue sequence is greater than the maximum supplied as an argument,
+or greater than the global batch_max, if that is set.
 
-BOOL transport_check_waiting(char *transport_name, char *hostname, int fd_in,
-   int fd_out, int batch_max)
+Arguments:
+  transport_name    name of the transport
+  hostname          name of the host
+  local_batch_max   maximum number of messages down one connection
+                      as set by the caller transport
+  new_message_id    set to the message id of a waiting message
+
+Returns:            TRUE if new_message_id set; FALSE otherwise
+*/
+
+BOOL
+transport_check_waiting(char *transport_name, char *hostname,
+  int local_batch_max, char *new_message_id)
 {
 db_wait *host_record;
-int host_length, path_len, pid, status;
+int host_length, path_len;
 EXIM_DB *dbm_file;
-char new_message_id[MESSAGE_ID_LENGTH + 1];
 char buffer[256];
 
-DEBUG(4) debug_printf("transport_check_waiting entered\n");
+DEBUG(4)
+  {
+  debug_printf("transport_check_waiting entered\n");
+  debug_printf("  sequence=%d local_max=%d global_max=%d\n",
+    continue_sequence, local_batch_max, batch_max);
+  }
 
 /* Do nothing if we have hit the batch maximum. */
 
-if (batch_max > 0 && continue_sequence >= batch_max)
+if ((local_batch_max > 0 && continue_sequence >= local_batch_max) ||
+    (batch_max >= 0 && continue_sequence >= batch_max))
   {
   DEBUG(4) debug_printf("batch max reached: returning\n");
   return FALSE;
@@ -517,7 +639,7 @@ sprintf(buffer, "wait-%s", transport_name);
 dbm_file = db_open(buffer, O_RDWR|O_CREAT);
 if (dbm_file == NULL)
   {
-  log_write(LOG_MAIN, "Failed to open %s database", buffer);
+  log_write(0, LOG_MAIN, "Failed to open %s database", buffer);
   return FALSE;
   }
 
@@ -531,6 +653,17 @@ if (host_record == NULL)
   return FALSE;
   }
 
+/* If the data in the record looks corrupt, just log something and
+don't try to use it. */
+
+if (host_record->count > WAIT_NAME_MAX)
+  {
+  db_close(dbm_file);
+  log_write(0, LOG_MAIN|LOG_PANIC, "smtp-wait database entry for %s has bad "
+    "count=%d (max=%d)", hostname, host_record->count, WAIT_NAME_MAX);
+  return FALSE;
+  }
+
 /* Scan the message ids in the record from the end towards the beginning,
 until one is found for which a spool file actually exists. If the record gets
 emptied, delete it and continue with any continuation records that may exist.
@@ -605,12 +738,10 @@ for (;;)
     host_length = host_record->count * MESSAGE_ID_LENGTH;
     }
 
-
   /* If we found an existing message, break the continuation loop. */
 
   if (found) break;
 
-
   /* If host_length <= 0 we have emptied a record and not found a good message,
   and there are no continuation records. Otherwise there is a continuation
   record to process. */
@@ -623,29 +754,54 @@ for (;;)
     }
   }
 
-
 /* Control gets here when an existing message has been encountered; its
 id is in new_message_id, and host_length is the revised length of the
 host record. If it is zero, the record has been removed. Update the
-record if required, and then close the database. */
+record if required, close the database, and return TRUE. */
 
 if (host_length > 0)
   {
   host_record->count = host_length/MESSAGE_ID_LENGTH;
   db_write(dbm_file, hostname, host_record, (int)sizeof(db_wait) + host_length);
   }
+
 db_close(dbm_file);
+return TRUE;
+}
 
 
-/* Now we need to fork a new exim process to deliver the message, and do
-a re-exec, both to get a clean delivery process, and to regain root privilege
-in cases where it has been given away. */
+
+/*************************************************
+*    Deliver waiting message down same socket    *
+*************************************************/
+
+/* Fork a new exim process to deliver the message, and do a re-exec, both to
+get a clean delivery process, and to regain root privilege in cases where it
+has been given away.
+
+Arguments:
+  transport_name  to pass to the new process
+  hostname        ditto
+  id              the new message to process
+  fd_in           the input socket
+  fd_out          the output socket
+
+Returns:          FALSE if fork fails; TRUE otherwise
+*/
+
+BOOL
+transport_pass_socket(char *transport_name, char *hostname, char *id,
+  int socket_fd)
+{
+pid_t pid;
+int status;
 
 if ((pid = vfork()) == 0)
   {
-  int fd; 
-  int i = 0;
-  char *argv[8];
+  int fd, i;
+  int fd_in = socket_fd;
+  int fd_out = dup(fd_in);
+  char *argv[11];
 
   /* Disconnect entirely from the parent process. */
 
@@ -653,14 +809,22 @@ if ((pid = vfork()) == 0)
 
   /* Set up the calling arguments */
 
+  i = 0;
   argv[i++] = exim_path;
   if (debug_level > 0)
     argv[i++] = string_sprintf("-d%d", debug_level);
+  if (batch_max >= 0)
+    argv[i++] = string_sprintf("-oB%d", batch_max);
+  if (config_changed)
+    {
+    argv[i++] = "-C";
+    argv[i++] = config_filename;
+    }
   argv[i++] = "-MC";
   argv[i++] = transport_name;
   argv[i++] = hostname;
   argv[i++] = string_sprintf("%d", continue_sequence + 1);
-  argv[i++] = new_message_id;
+  argv[i++] = id;
   argv[i++] = NULL;
 
   /* Arrange for the channel to be on stdin and stdout */
@@ -682,8 +846,12 @@ if ((pid = vfork()) == 0)
     close(fd_out);
     }
 
-  DEBUG(4) debug_printf("fork %s %s %s %s %s %s %s\n", argv[0], argv[1],
-    argv[2], argv[3], argv[4], argv[5], (argv[6] == NULL)? "" : argv[6]);
+  DEBUG(4) debug_printf("fork %s %s %s %s %s %s %s %s %s %s\n", argv[0],
+    argv[1], argv[2], argv[3], argv[4], argv[5],
+      (argv[6] == NULL)? "" : argv[6],
+      (argv[7] == NULL)? "" : argv[7],
+      (argv[8] == NULL)? "" : argv[8],
+      (argv[9] == NULL)? "" : argv[9]);
 
   execv(argv[0], argv);
 
@@ -700,10 +868,105 @@ this one. */
 
 if (pid > 0)
   {
-  while (wait(&status) != pid);
+  int rc;
+  while ((rc = wait(&status)) != pid && (rc >= 0 || errno != ECHILD));
   return TRUE;
   }
 else return FALSE;
 }
 
+
+
+/*************************************************
+*            Test host for serialization         *
+*************************************************/
+
+/* This function is called when a host is listed for serialization of
+connections. We open the relevant database and look for a record, which implies
+an existing connection. If not found, create one and return TRUE.
+
+Arguments:
+  transport_name    name of transport
+  hostname          name of host
+
+Returns:            TRUE if OK to proceed; FALSE otherwise
+*/
+
+
+BOOL
+transport_check_serialized(char *transport_name, char *hostname)
+{
+db_serialize *host_record;
+db_serialize new_record;
+EXIM_DB *dbm_file;
+char buffer[256];
+
+DEBUG(4) debug_printf("transport_check_serialized entered\n");
+
+/* Open and lock the waiting information database. */
+
+sprintf(buffer, "serialize-%s", transport_name);
+dbm_file = db_open(buffer, O_RDWR|O_CREAT);
+if (dbm_file == NULL)
+  {
+  log_write(0, LOG_MAIN, "Failed to open %s database", buffer);
+  return FALSE;
+  }
+
+/* See if there is a record for this host; if there is, we cannot proceed with
+the connection unless the record is very old. */
+
+host_record = db_read(dbm_file, hostname);
+if (host_record != NULL && time(NULL) - host_record->time_stamp < 6*60*60)
+  {
+  db_close(dbm_file);
+  DEBUG(4) debug_printf("outstanding connection to %s\n", hostname);
+  return FALSE;
+  }
+
+/* We can proceed - insert a new record or update the old one. At present
+the count field is not used; just set it to 1. */
+
+new_record.count = 1;
+db_write(dbm_file, hostname, &new_record, (int)sizeof(db_serialize));
+db_close(dbm_file);
+return TRUE;
+}
+
+
+
+/*************************************************
+*            Release host serialization          *
+*************************************************/
+
+/* This function is called when a serialized host's connection ends.
+We open the relevant database and delete it's record.
+
+Arguments:
+  transport_name    name of transport
+  hostname          name of host
+
+Returns:            nothing
+*/
+
+void
+transport_end_serialized(char *transport_name, char *hostname)
+{
+EXIM_DB *dbm_file;
+char buffer[256];
+
+DEBUG(4) debug_printf("transport_end_serialized entered\n");
+
+sprintf(buffer, "serialize-%s", transport_name);
+dbm_file = db_open(buffer, O_RDWR|O_CREAT);
+if (dbm_file == NULL)
+  {
+  log_write(0, LOG_MAIN, "Failed to open %s database", buffer);
+  return;
+  }
+
+db_delete(dbm_file, hostname);
+db_close(dbm_file);
+}
+
 /* End of transport.c */
