@@ -2,38 +2,37 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
   char *filename;
-  char *dirname; 
+  char *dirname;
   char *prefix;
   char *suffix;
-  char *expand_uid;
-  char *expand_gid;
-  char *create_file_string; 
-  int   local_smtp; 
-  int   uid;
-  int   gid;
+  char *create_file_string;
+  char *quota;
+  int   quota_value;
   int   mode;
-  int   dirmode; 
+  int   dirmode;
   int   lockfile_mode;
   int   lockfile_timeout;
   int   lock_retries;
   int   lock_interval;
-  int   create_file; 
-  BOOL  check_group; 
-  BOOL  create_directory; 
+  int   create_file;
+  BOOL  allow_symlink;
+  BOOL  check_group;
+  BOOL  create_directory;
   BOOL  notify_comsat;
   BOOL  require_lockfile;
-  BOOL  use_lockfile; 
+  BOOL  use_lockfile;
   BOOL  from_hack;
   BOOL  return_path_add;
-  BOOL  delivery_date_add; 
-  BOOL  file_must_exist; 
+  BOOL  delivery_date_add;
+  BOOL  envelope_to_add;
+  BOOL  file_must_exist;
 } appendfile_transport_options_block;
 
 /* Restricted creation options */
