@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -14,13 +14,18 @@
 /* Options specific to the autoreply transport. They must be in alphabetic
 order (note that "_" comes before the lower case letters). Those starting
 with "*" are not settable by the user but are used by the option-reading
-software for alternative value types. */
+software for alternative value types. Some options are publicly visible and so
+are stored in the driver instance block. These are flagged with opt_public. */
 
 optionlist autoreply_transport_options[] = {
-  { "*expand_group",     opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, expand_gid)) },
-  { "*expand_user",      opt_stringptr,
-      (void *)(offsetof(autoreply_transport_options_block, expand_uid)) },
+  { "*expand_group",     opt_stringptr | opt_hidden | opt_public,
+      (void *)(offsetof(transport_instance, expand_gid)) },
+  { "*expand_user",      opt_stringptr | opt_hidden | opt_public,
+      (void *)(offsetof(transport_instance, expand_uid)) },
+  { "*set_group",         opt_bool | opt_hidden | opt_public,
+      (void *)(offsetof(transport_instance, gid_set)) },
+  { "*set_user",          opt_bool | opt_hidden | opt_public,
+      (void *)(offsetof(transport_instance, uid_set)) },
   { "bcc",               opt_stringptr,
       (void *)(offsetof(autoreply_transport_options_block, bcc)) },
   { "cc",                opt_stringptr,
@@ -33,10 +38,12 @@ optionlist autoreply_transport_options[] = {
       (void *)(offsetof(autoreply_transport_options_block, file_optional)) },
   { "from",              opt_stringptr,
       (void *)(offsetof(autoreply_transport_options_block, from)) },
-  { "group",             opt_expand_gid,
-      (void *)(offsetof(autoreply_transport_options_block, gid)) },
+  { "group",             opt_expand_gid | opt_public,
+      (void *)(offsetof(transport_instance, gid)) },
   { "headers",           opt_stringptr,
       (void *)(offsetof(autoreply_transport_options_block, headers)) },
+  { "initgroups",        opt_bool | opt_public,
+      (void *)(offsetof(transport_instance, initgroups)) },
   { "log",               opt_stringptr,
       (void *)(offsetof(autoreply_transport_options_block, logfile)) },
   { "mode",              opt_octint,
@@ -51,8 +58,8 @@ optionlist autoreply_transport_options[] = {
       (void *)(offsetof(autoreply_transport_options_block, text)) },
   { "to",                opt_stringptr,
       (void *)(offsetof(autoreply_transport_options_block, to)) },
-  { "user",              opt_expand_uid,
-      (void *)(offsetof(autoreply_transport_options_block, uid)) },
+  { "user",              opt_expand_uid | opt_public,
+      (void *)(offsetof(transport_instance, uid)) },
 };
 
 /* Size of the options list. An extern variable has to be used so that its
@@ -64,24 +71,20 @@ int autoreply_transport_options_count =
 /* Default private options block for the autoreply transport. */
 
 autoreply_transport_options_block autoreply_transport_option_defaults = {
-  NULL,           /* expand_uid */
-  NULL,           /* expand_gid */
-  NULL,           /* from */ 
+  NULL,           /* from */
   NULL,           /* to */
   NULL,           /* cc */
   NULL,           /* bcc */
-  NULL,           /* subject */ 
-  NULL,           /* headers */ 
+  NULL,           /* subject */
+  NULL,           /* headers */
   NULL,           /* text */
   NULL,           /* file */
   NULL,           /* logfile */
   NULL,           /* oncelog */
-  -1,             /* uid */
-  -1,             /* gid */
-  0600,           /* mode */ 
-  FALSE,          /* file_expand */ 
-  FALSE,          /* file_optional */ 
-  FALSE           /* return message */ 
+  0600,           /* mode */
+  FALSE,          /* file_expand */
+  FALSE,          /* file_optional */
+  FALSE           /* return message */
 };
 
 
@@ -94,23 +97,19 @@ autoreply_transport_options_block autoreply_transport_option_defaults = {
 enable consistency checks to be done, or anything else that needs
 to be set up. */
 
-void autoreply_transport_init(transport_instance *tblock)
+void
+autoreply_transport_init(transport_instance *tblock)
 {
+/*
 autoreply_transport_options_block *ob =
   (autoreply_transport_options_block *)(tblock->options_block);
+*/
 
 /* If a fixed uid field is set, then a gid field must also be set. */
 
-if (ob->uid >= 0 && ob->gid < 0)
-  log_write(LOG_PANIC_DIE, "Exim configuration error:\n  "
+if (tblock->uid_set && !tblock->gid_set)
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "user set without group for the %s transport", tblock->name);
-
-/* Copy the uid and gid into the generic slots. */
-
-tblock->uid = ob->uid;
-tblock->gid = ob->gid;
-tblock->expand_uid = ob->expand_uid;
-tblock->expand_gid = ob->expand_gid;
 }
 
 
@@ -121,9 +120,21 @@ tblock->expand_gid = ob->expand_gid;
 *************************************************/
 
 /* If the expansion fails, the error is set up in the address. Expanded
-strings must be checked to ensure they contain only printing characters. */
+strings must be checked to ensure they contain only printing characters
+and white space. If not, the function fails.
 
-static char *checkexpand(char *s, address_item *addr, char *name, BOOL text)
+Arguments:
+   s         string to expand
+   addr      address that is being worked on
+   name      transport name, for error text
+   text      if TRUE, \n is allowed in the expanded string
+
+Returns:     expanded string if expansion succeeds;
+             NULL otherwise
+*/
+
+static char *
+checkexpand(char *s, address_item *addr, char *name, BOOL text)
 {
 char *t;
 char *ss = expand_string(s);
@@ -135,20 +146,20 @@ if (ss == NULL)
     "%s", s, name, expand_string_message);
   return NULL;
   }
-   
+
 for (t = ss; *t != 0; t++)
   {
   if (!isprint(*t) && *t != '\t' && (*t != '\n' || !text))
     {
-    s = string_printing(s, TRUE); 
+    s = string_printing(s, TRUE);
     addr->transport_return = FAIL;
     addr->message = string_sprintf("Expansion of \"%s\" in %s transport "
       "contains non-printing characters", s, name);
-    return NULL;   
+    return NULL;
     }
-  }         
-   
-return ss; 
+  }
+
+return ss;
 }
 
 
@@ -160,7 +171,8 @@ return ss;
 
 /* See local README for interface details. */
 
-void autoreply_transport_entry(
+void
+autoreply_transport_entry(
   transport_instance *tblock,      /* data for this instantiation */
   address_item *addr)              /* address we are working on */
 {
@@ -182,7 +194,7 @@ DEBUG(2) debug_printf("%s transport entered\n", tblock->name);
 /* Set up for the good case */
 
 addr->transport_return = OK;
-addr->errno = 0;
+addr->basic_errno = 0;
 
 /* If the address is pointing to a reply block, then take all the data
 from that block. It has typically been set up by a mail filter processing
@@ -191,49 +203,49 @@ it has to be expanded here. */
 
 if (addr->reply != NULL)
   {
-  DEBUG(9) debug_printf("taking data from address\n"); 
-  from = addr->reply->from; 
+  DEBUG(9) debug_printf("taking data from address\n");
+  from = addr->reply->from;
   to = addr->reply->to;
   cc = addr->reply->cc;
   bcc = addr->reply->bcc;
-  subject = addr->reply->subject; 
-  headers = addr->reply->headers; 
+  subject = addr->reply->subject;
+  headers = addr->reply->headers;
   text = addr->reply->text;
   file = addr->reply->file;
   logfile = addr->reply->logfile;
   oncelog = addr->reply->oncelog;
-  file_expand = addr->reply->file_expand; 
+  file_expand = addr->reply->file_expand;
   }
 else
   {
-  DEBUG(9) debug_printf("taking data from transport\n"); 
-  from = ob->from; 
+  DEBUG(9) debug_printf("taking data from transport\n");
+  from = ob->from;
   to = ob->to;
   cc = ob->cc;
   bcc = ob->bcc;
-  subject = ob->subject; 
-  headers = ob->headers; 
+  subject = ob->subject;
+  headers = ob->headers;
   text = ob->text;
   file = ob->file;
   logfile = ob->logfile;
   oncelog = ob->oncelog;
-  file_expand = ob->file_expand; 
+  file_expand = ob->file_expand;
 
-  if ((from  != NULL && 
+  if ((from  != NULL &&
         (from = checkexpand(from, addr, tblock->name, FALSE)) == NULL) ||
-      (to    != NULL && 
+      (to    != NULL &&
         (to = checkexpand(to, addr, tblock->name, FALSE)) == NULL) ||
-      (cc    != NULL && 
+      (cc    != NULL &&
         (cc = checkexpand(cc, addr, tblock->name, FALSE)) == NULL) ||
-      (bcc   != NULL && 
+      (bcc   != NULL &&
         (bcc = checkexpand(bcc, addr, tblock->name, FALSE)) == NULL) ||
-      (subject   != NULL && 
+      (subject   != NULL &&
         (subject = checkexpand(subject, addr, tblock->name, FALSE)) == NULL) ||
-      (headers != NULL && 
+      (headers != NULL &&
         (headers = checkexpand(headers, addr, tblock->name, TRUE)) == NULL) ||
-      (text  != NULL && 
+      (text  != NULL &&
         (text = checkexpand(text, addr, tblock->name, TRUE)) == NULL) ||
-      (file  != NULL && 
+      (file  != NULL &&
         (file = checkexpand(file, addr, tblock->name, FALSE)) == NULL) ||
       (logfile != NULL &&
         (logfile = checkexpand(logfile, addr, tblock->name, FALSE)) == NULL) ||
@@ -251,51 +263,51 @@ if (dont_deliver)
     tblock->name);
   addr->transport_return = OK;
   return;
-  }      
+  }
 
 
 /* If the oncelog field is set, we send want to send only one message to
-the given recipient(s). This works only on the "To" field. If there is no "To" 
-field the message is always sent. The "To" field is used as the key in a dbm 
-file; if it contains more than one recipient, the effect might not be quite as 
+the given recipient(s). This works only on the "To" field. If there is no "To"
+field the message is always sent. The "To" field is used as the key in a dbm
+file; if it contains more than one recipient, the effect might not be quite as
 envisaged. */
 
 if (oncelog != NULL && to != NULL)
   {
   EXIM_DATUM key_datum, result_datum;
-  dbm_file = EXIM_DBOPEN(oncelog, O_RDWR|O_CREAT, ob->mode); 
-  if (dbm_file == NULL) 
+  dbm_file = EXIM_DBOPEN(oncelog, O_RDWR|O_CREAT, ob->mode);
+  if (dbm_file == NULL)
     {
     addr->transport_return = DEFER;
     addr->message = string_sprintf("Failed to open %s file %s when sending "
-      "message from %s transport: %s", EXIM_DBTYPE, oncelog, tblock->name, 
+      "message from %s transport: %s", EXIM_DBTYPE, oncelog, tblock->name,
       strerror(errno));
-    return;   
-    } 
+    return;
+    }
 
   EXIM_DATUM_DATA(key_datum) = to;
   EXIM_DATUM_SIZE(key_datum) = (int)strlen(to) + 1;
- 
+
   if (EXIM_DBGET(dbm_file, key_datum, result_datum))
     {
-    DEBUG(9) debug_printf("message already sent to %s\n", to); 
+    DEBUG(9) debug_printf("message already sent to %s\n", to);
     log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
     if (log_fd >= 0)
       {
       char *ptr = log_buffer;
-      if (now == NULL) now = tod_stamp(tod_log); 
-      sprintf(ptr, "%s\n  previously sent to %s\n", now, to); 
+      if (now == NULL) now = tod_stamp(tod_log);
+      sprintf(ptr, "%s\n  previously sent to %s\n", now, to);
       while(*ptr) ptr++;
-      write(log_fd, log_buffer, ptr - log_buffer);   
-      }  
-    EXIM_DBCLOSE(dbm_file);   
+      write(log_fd, log_buffer, ptr - log_buffer);
+      }
+    EXIM_DBCLOSE(dbm_file);
     return;
-    }  
-  } 
+    }
+  }
 
 
 /* Ensure any requested file is available. */
-  
+
 if (file != NULL)
   {
   ff = fopen(file, "r");
@@ -304,20 +316,20 @@ if (file != NULL)
     addr->transport_return = DEFER;
     addr->message = string_sprintf("Failed to open file %s when sending "
       "message from %s transport: %s", file, tblock->name, strerror(errno));
-    return;   
-    } 
-  }   
+    return;
+    }
+  }
 
 /* Make a subprocess to send the message */
 
-pid = child_open(mailer_argv, NULL, 077, -1, -1, &fd,
+pid = child_open(mailer_argv, NULL, 077, NULL, NULL, &fd,
   (debug_file != NULL)? fileno(debug_file) : -1);
 
 /* Creation of child failed; defer this delivery. */
 
 if (pid < 0)
   {
-  DEBUG(2) debug_printf("Failed to create child process\n"); 
+  DEBUG(2) debug_printf("Failed to create child process\n");
   addr->transport_return = DEFER;
   addr->message = string_sprintf("Failed to create child process to send "
     "message from %s transport", tblock->name);
@@ -340,40 +352,40 @@ if (subject != NULL) fprintf(f, "Subject: %s\n", subject);
 always be one, but code defensively. */
 
 for (h = header_list; h != NULL; h = h->next)
-  if (h->type == htype_id) break; 
-  
-if (h != NULL) 
+  if (h->type == htype_id) break;
+
+if (h != NULL)
   {
   char *s = strchr(h->text, ':') + 1;
-  while (isspace(*s)) s++; 
+  while (isspace(*s)) s++;
   fprintf(f, "In-Reply-To: %s\n", s);
-  } 
+  }
 
 /* Add any specially requested headers */
 
 if (headers != NULL) fprintf(f, "%s\n", headers);
 fprintf(f, "\n");
 
-if (text != NULL) 
+if (text != NULL)
   {
   fprintf(f, "%s", text);
   if (text[(int)strlen(text)-1] != '\n') fprintf(f, "\n");
-  }  
+  }
 
 if (ff != NULL)
   {
-  while (fgets(big_buffer, BIG_BUFFER_SIZE, ff) != NULL)
+  while (fgets(big_buffer, big_buffer_size, ff) != NULL)
     {
     if (file_expand)
       {
       char *s = expand_string(big_buffer);
       fprintf(f, "%s", (s == NULL)? big_buffer : s);
-      if (s != NULL) store_free(s);       
-      }    
+      if (s != NULL) store_free(s);
+      }
     else fprintf(f, "%s", big_buffer);
-    } 
-  }     
-  
+    }
+  }
+
 /* Copy the original message if required, observing the return size
 limit. */
 
@@ -384,11 +396,11 @@ if (ob->return_message)
     struct stat statbuf;
     int max = (return_size_limit/DELIVER_BUFFER_SIZE + 1)*DELIVER_BUFFER_SIZE;
     if (fstat(deliver_datafile, &statbuf) == 0 && statbuf.st_size > max)
-      { 
+      {
       fprintf(f, "\n"
 "------ This is a copy of your message, including all the headers.\n"
 "------ The body of your message is %d characters long; only the first\n"
-"------ %d or so are included here.\n\n", statbuf.st_size, (max/1000)*1000);
+"------ %d or so are included here.\n\n", (int)statbuf.st_size, (max/1000)*1000);
       }
     else fprintf(f, "\n"
 "------ This is a copy of your message, including all the headers. ------\n\n");
@@ -397,30 +409,28 @@ if (ob->return_message)
 "------ This is a copy of your message, including all the headers. ------\n\n");
 
   fflush(f);
-  transport_write_message(fileno(f), FALSE, FALSE, FALSE, FALSE, FALSE, NULL,
-    return_size_limit);
-  }  
+  transport_write_message(addr, fileno(f), 0, NULL, return_size_limit);
+  }
 
-/* End the message and wait for the child process to end. */
+/* End the message and wait for the child process to end; no timeout. */
 
 fclose(f);
-rc = child_close(pid);
+rc = child_close(pid, 0);
 
-/* Update the "sent to" log whatever the yield. This errs on the side of 
+/* Update the "sent to" log whatever the yield. This errs on the side of
 missing out a message rather than risking sending more than one. */
 
 if (dbm_file != NULL)
   {
   EXIM_DATUM key_datum, value_datum;
-  if (now == NULL) now = tod_stamp(tod_log); 
+  if (now == NULL) now = tod_stamp(tod_log);
   EXIM_DATUM_DATA(key_datum) = to;
   EXIM_DATUM_SIZE(key_datum) = (int)strlen(to) + 1;
   EXIM_DATUM_DATA(value_datum) = now;
   EXIM_DATUM_SIZE(value_datum) = (int)strlen(now) + 1;
   EXIM_DBPUT(dbm_file, key_datum, value_datum);
-  EXIM_DBCLOSE(dbm_file); 
-  dbm_close(dbm_file); 
-  } 
+  EXIM_DBCLOSE(dbm_file);
+  }
 
 /* If sending failed, defer to try again - but if once is set the next
 try will skip, of course. */
@@ -431,65 +441,65 @@ if (rc != 0)
   addr->message = string_sprintf("Failed to send message from %s "
     "transport (%d)", tblock->name, rc);
   }
-  
+
 /* Log the sending of the message if successful and required. If the file
 fails to open, it's hard to know what to do. We cannot write to the Exim
 log from here, since we may be running under an unprivileged uid. We don't
-want to fail the delivery, since the message has been successfully sent. For 
-the moment, ignore open failures. Write the log entry as a single write() to a 
-file opened for appending, in order to avoid interleaving of output from 
-different processes. The log_buffer can be used exactly as for main log 
-writing. Oh, wouldn't it be nice if all C systems had ANSI-compliant sprintf() 
+want to fail the delivery, since the message has been successfully sent. For
+the moment, ignore open failures. Write the log entry as a single write() to a
+file opened for appending, in order to avoid interleaving of output from
+different processes. The log_buffer can be used exactly as for main log
+writing. Oh, wouldn't it be nice if all C systems had ANSI-compliant sprintf()
 funtions? For now, code without using the result of sprintf(). */
 
 if (logfile != NULL)
   {
-  if (now == NULL) now = tod_stamp(tod_log); 
-  if (log_fd < 0) 
+  if (now == NULL) now = tod_stamp(tod_log);
+  if (log_fd < 0)
     log_fd = open(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
   if (log_fd >= 0)
-    { 
+    {
     char *ptr = log_buffer;
-    DEBUG(9) debug_printf("logging message details\n"); 
-    sprintf(ptr, "%s\n", now); 
+    DEBUG(9) debug_printf("logging message details\n");
+    sprintf(ptr, "%s\n", now);
     while(*ptr) ptr++;
-    if (from != NULL) 
+    if (from != NULL)
       {
       sprintf(ptr, "  From: %s\n", from);
       while(*ptr) ptr++;
-      } 
-    if (to != NULL) 
+      }
+    if (to != NULL)
       {
       sprintf(ptr, "  To: %s\n", to);
       while(*ptr) ptr++;
-      } 
-    if (cc != NULL) 
+      }
+    if (cc != NULL)
       {
       sprintf(ptr, "  Cc: %s\n", cc);
       while(*ptr) ptr++;
-      } 
-    if (bcc != NULL) 
+      }
+    if (bcc != NULL)
       {
       sprintf(ptr, "  Bcc: %s\n", bcc);
       while(*ptr) ptr++;
-      } 
-    if (subject != NULL) 
+      }
+    if (subject != NULL)
       {
       sprintf(ptr, "  Subject: %s\n", subject);
       while(*ptr) ptr++;
-      } 
-    if (headers != NULL) 
+      }
+    if (headers != NULL)
       {
       sprintf(ptr, "  %s\n", headers);
       while(*ptr) ptr++;
-      } 
-    write(log_fd, log_buffer, ptr - log_buffer);   
+      }
+    write(log_fd, log_buffer, ptr - log_buffer);
     }
   else DEBUG(2) debug_printf("Failed to open log file %s for %s "
-    "transport: %s\n", logfile, tblock->name, strerror(errno));    
-  } 
+    "transport: %s\n", logfile, tblock->name, strerror(errno));
+  }
 
-if (log_fd >= 0) close(log_fd); 
+if (log_fd >= 0) close(log_fd);
 
 DEBUG(2) debug_printf("%s transport succeeded\n", tblock->name);
 }
