@@ -2,30 +2,26 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
-  char *expand_uid;
-  char *expand_gid;  
-  char *from; 
+  char *from;
   char *to;
   char *cc;
   char *bcc;
-  char *subject; 
-  char *headers; 
+  char *subject;
+  char *headers;
   char *text;
   char *file;
-  char *logfile;      
-  char *oncelog; 
-  int   uid; 
-  int   gid;
-  int   mode; 
-  BOOL  file_expand; 
-  BOOL  file_optional; 
-  BOOL  return_message; 
+  char *logfile;
+  char *oncelog;
+  int   mode;
+  BOOL  file_expand;
+  BOOL  file_optional;
+  BOOL  return_message;
 } autoreply_transport_options_block;
 
 /* Data for reading the private options. */
