@@ -2,31 +2,25 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 199h */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
-  char *cmd; 
-  char *path; 
+  char *cmd;
+  char *path;
   char *prefix;
-  char *suffix;  
-  char *expand_uid;
-  char *expand_gid;  
-  char *directory; 
-  int   local_smtp; 
-  int   uid; 
-  int   gid;
-  int   umask; 
-  int   max_output; 
-  BOOL  from_hack; 
-  BOOL  ignore_status; 
-  BOOL  pipe_as_creator; 
-  BOOL  return_path_add; 
-  BOOL  delivery_date_add; 
-  BOOL  return_output; 
-  BOOL  restrict_to_path; 
+  char *suffix;
+  int   umask;
+  int   max_output;
+  int   timeout;
+  BOOL  from_hack;
+  BOOL  ignore_status;
+  BOOL  return_path_add;
+  BOOL  delivery_date_add;
+  BOOL  envelope_to_add;
+  BOOL  restrict_to_path;
 } pipe_transport_options_block;
 
 /* Data for reading the private options. */
