@@ -2,10 +2,11 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
-/* Functions for maintaining binary balanced trees. */
+/* Functions for maintaining binary balanced trees and some associated
+functions as well. */
 
 
 #include "exim.h"
@@ -17,49 +18,77 @@
 *      Convert address for use as tree key       *
 *************************************************/
 
-/* An "address" that starts with / or | are left alone; otherwise the domain 
-portions of the address are forced into lower case. */
+/* The domain portion of the address is forced into lower case. This function
+should not be called for pipe or file or reply addresses.
+
+Argument: points to address string
+Returns:  points to prepared address string, in a fixed bit of store
+*/
 
 static char prepared_address[256];
 
-static char *address_prepare(char *s)
+static char *
+address_prepare(char *s)
 {
-if (s[0] == '/' || s[0] == '|') strcpy(prepared_address, s); else
+char *p = s + (int)strlen(s);
+while (p > s && p[-1] != '@') p--;
+if (p <= s) strcpy(prepared_address, s); else
   {
-  char *p = s + (int)strlen(s);
-  while (p > s && p[-1] != '@') p--;
-  if (p <= s) strcpy(prepared_address, s); else
-    {
-    char *t = prepared_address; 
-    char *pp = p - 2;
-    while (pp >= s && *pp != ':') pp--;
-    while (s < pp) *t++ = tolower(*s++);
-    while (s < p) *t++ = *s++;
-    while (*s) *t++ = tolower(*s++);  
-    *t = 0;
-    }  
-  }   
+  char *t = prepared_address;
+  char *pp = p - 2;
+  while (pp >= s && *pp != ':') pp--;
+  while (s < pp) *t++ = tolower(*s++);
+  while (s < p) *t++ = *s++;
+  while (*s) *t++ = tolower(*s++);
+  *t = 0;
+  }
 return prepared_address;
 }
 
 
 
 /*************************************************
-* Add entry to non-recipients or duplicates tree *
+*        Add entry to non-recipients tree        *
 *************************************************/
 
-void tree_add_nonrecipient(char *s)
+/* Duplicates are just discarded.
+
+Arguments:
+  s       string to add
+  pfr     true for a pipe/file/reply address
+
+Returns:  nothing
+*/
+
+void
+tree_add_nonrecipient(char *s, BOOL pfr)
 {
 tree_node *node = store_malloc(sizeof(tree_node) + (int)strlen(s));
-strcpy(node->name, address_prepare(s));
+strcpy(node->name, pfr? s : address_prepare(s));
 node->data.ptr = NULL;
 if (!tree_insertnode(&tree_nonrecipients, node)) store_free(node);
 }
 
-void tree_add_duplicate(char *s)
+
+
+/*************************************************
+*        Add entry to duplicates tree            *
+*************************************************/
+
+/* Duplicates are just discarded.
+
+Argument:
+  s       string to add
+  pfr     TRUE for pipe/file/reply address
+
+Returns:  nothing
+*/
+
+void
+tree_add_duplicate(char *s, BOOL pfr)
 {
 tree_node *node = store_malloc(sizeof(tree_node) + (int)strlen(s));
-strcpy(node->name, address_prepare(s));
+strcpy(node->name, pfr? s : address_prepare(s));
 node->data.ptr = NULL;
 if (!tree_insertnode(&tree_duplicates, node)) store_free(node);
 }
@@ -70,11 +99,20 @@ if (!tree_insertnode(&tree_duplicates, node)) store_free(node);
 *            Search an address tree              *
 *************************************************/
 
-/* The key has to be prepared in order to cope with upper case bits. */
+/* The key has to be prepared in order to cope with upper case bits.
 
-tree_node *tree_search_addr(tree_node *p, char *name)
+Arguments:
+  p         pointer to root of tree
+  name      key to search for
+  pfr       true for a pipe/file/reply address
+
+Returns:    pointer to node or NULL if not found
+*/
+
+tree_node *
+tree_search_addr(tree_node *p, char *name, BOOL pfr)
 {
-return tree_search(p, address_prepare(name));
+return tree_search(p, pfr? name : address_prepare(name));
 }
 
 
@@ -84,13 +122,21 @@ return tree_search(p, address_prepare(name));
 *    Add entry to unusable addresses tree        *
 *************************************************/
 
-/* Duplicates are simply discarded. */
+/* Duplicates are simply discarded.
 
-void tree_add_unusable(char *s, int why)
+Argument:    the host item
+Returns:     nothing
+*/
+
+void
+tree_add_unusable(host_item *h)
 {
-tree_node *node = store_malloc(sizeof(tree_node) + (int)strlen(s));
+tree_node *node;
+char s[256];
+sprintf(s, "T:%s:%s", h->name, h->address);
+node = store_malloc(sizeof(tree_node) + (int)strlen(s));
 strcpy(node->name, s);
-node->data.val = why;
+node->data.val = h->why;
 if (!tree_insertnode(&tree_unusable, node)) store_free(node);
 }
 
@@ -102,29 +148,39 @@ if (!tree_insertnode(&tree_unusable, node)) store_free(node);
 
 /* This function writes out a tree in a form in which it can
 easily be re-read. It is used for writing out the non-recipients
-tree onto the spool, for retrieval at the next retry time. 
+tree onto the spool, for retrieval at the next retry time.
 
 The format is as follows:
 
    . If the tree is empty, write one line containing XX.
-   
+
    . Otherwise, each node is written, preceded by two letters
      (Y/N) indicating whether it has left or right children.
-     
-   . The left subtree (if any) then follows, then the right subtree.
-   
-*/ 
 
-static void write_tree(tree_node *p, FILE *f)
+   . The left subtree (if any) then follows, then the right subtree.
+
+First, there's an internal recursive subroutine.
+
+Arguments:
+  p          current node
+  f          FILE to write to
+
+Returns:     nothing
+*/
+
+static void
+write_tree(tree_node *p, FILE *f)
 {
-fprintf(f, "%c%c %s\n", 
-  (p->left == NULL)? 'N':'Y', (p->right == NULL)? 'N':'Y', p->name); 
+fprintf(f, "%c%c %s\n",
+  (p->left == NULL)? 'N':'Y', (p->right == NULL)? 'N':'Y', p->name);
 if (p->left != NULL) write_tree(p->left, f);
 if (p->right != NULL) write_tree(p->right, f);
-}    
+}
 
+/* This is the top-level function, with the same arguments. */
 
-void tree_write(tree_node *p, FILE *f)
+void
+tree_write(tree_node *p, FILE *f)
 {
 if (p == NULL)
   {
@@ -135,7 +191,7 @@ write_tree(p, f);
 }
 
 
- 
+
 
 
 /***********************************************************
@@ -164,11 +220,17 @@ C does not permit additions and subtractions on void pointers. */
 *************************************************/
 
 /* The node->name field must (obviously) be set, but the other
-fields need not be initialized. The yield is TRUE if the node is
-successfully inserted, and FALSE otherwise (duplicate node
-found). */
+fields need not be initialized.
 
-int tree_insertnode(tree_node **treebase, tree_node *node)
+Arguments:
+  treebase   pointer to the root of the tree
+  node       the note to insert, with name field set
+
+Returns:     TRUE if node inserted; FALSE if not (duplicate)
+*/
+
+int
+tree_insertnode(tree_node **treebase, tree_node *node)
 {
 tree_node *p = *treebase;
 tree_node **q, *r, *s, **t;
@@ -216,12 +278,9 @@ for (;;)
 that is the place at which the new node must be inserted. */
 
 *q = node;
-node->left = NULL;
-node->right = NULL;
-node->balance = 0;
 
 /* Set up s as the potential re-balancing point, and r as the
-next node after it along the route*/
+next node after it along the route. */
 
 s = *t;
 r = (strcmp(node->name, s->name) > 0)? s->right : s->left;
@@ -229,7 +288,9 @@ r = (strcmp(node->name, s->name) > 0)? s->right : s->left;
 /* Adjust balance factors along the route from s to node. */
 
 p = r;
+
 while (p != node)
+  {
   if (strcmp(node->name, p->name) < 0)
     {
     p->balance = tree_lbal;
@@ -240,6 +301,7 @@ while (p != node)
     p->balance = tree_rbal;
     p = p->right;
     }
+  }
 
 /* Now the World-Famous Balancing Act */
 
@@ -249,7 +311,9 @@ if (s->balance == 0) s->balance = (char)a;        /* The tree has grown higher *
   else if (s->balance != (char)a) s->balance = 0; /* It's become more balanced */
 else                                              /* It's got out of balance */
   {
-  if (r->balance == (char)a)     /* Perform a single rotation */
+  /* Perform a single rotation */
+
+  if (r->balance == (char)a)
     {
     p = r;
     if (a == tree_rbal)
@@ -266,10 +330,16 @@ else                                              /* It's got out of balance */
     r->balance = 0;
     }
 
-  else                          /* Perform a double rotation */
+  /* Perform a double rotation There was an occasion when the balancing
+  factors were screwed up by a bug in the code that reads a tree from
+  the spool. In case this ever happens again, check for changing p to NULL
+  and don't do it. It is better to have an unbalanced tree than a crash. */
+
+  else
     {
     if (a == tree_rbal)
       {
+      if (r->left == NULL) return TRUE;   /* Bail out if tree corrupt */
       p = r->left;
       r->left = p->right;
       p->right = r;
@@ -278,6 +348,7 @@ else                                              /* It's got out of balance */
       }
     else
       {
+      if (r->right == NULL) return TRUE;  /* Bail out if tree corrupt */
       p = r->right;
       r->right = p->left;
       p->left = r;
@@ -304,11 +375,16 @@ return TRUE;     /* Successful insertion */
 *          Search tree for node by name          *
 *************************************************/
 
-/* If wantdeleted is non-zero, we return a deleted node if
-found. Otherwise return NULL for deleted as well as for
-non-existent nodes. */
+/*
+Arguments:
+  p         root of tree
+  name      key to search for
 
-tree_node *tree_search(tree_node *p, char *name)
+Returns:    pointer to node, or NULL if not found
+*/
+
+tree_node *
+tree_search(tree_node *p, char *name)
 {
 while (p != NULL)
   {
@@ -325,16 +401,21 @@ return NULL;
 *************************************************/
 
 /* This disposes of an entire tree. We never free parts, because we never
-actually remove anything from a tree. */
+actually remove anything from a tree.
 
-void tree_free(tree_node *p)
+Argument: the root of the tree
+Returns:  nothing
+*/
+
+void
+tree_free(tree_node *p)
 {
 if (p != NULL)
   {
   if (p->left != NULL) tree_free(p->left);
   if (p->right != NULL) tree_free(p->right);
   store_free(p);
-  } 
+  }
 }
 
 
@@ -350,26 +431,41 @@ is used only in debugging circumstances. */
 #define tree_printlinesize 132   /* line size for printing */
 static char tree_printline[tree_printlinesize];
 
-static void tree_printsub(tree_node *p, int pos, int barswitch)
+/* Internal recursive subroutine.
+
+Arguments:
+  p          tree node
+  pos        amount of indenting & vertical bars to pring
+  barswitch  if TRUE print | at the pos value
+
+Returns:     nothing
+*/
+
+static void
+tree_printsub(tree_node *p, int pos, int barswitch, FILE *f)
 {
 int i;
-if (p->right != NULL) tree_printsub(p->right, pos+2, 1);
-for (i = 0; i <= pos-1; i++) fputc(tree_printline[i], stdout);
-fprintf(stdout, "-->%s [%d]\n", p->name, p->balance);
+if (p->right != NULL) tree_printsub(p->right, pos+2, 1, f);
+for (i = 0; i <= pos-1; i++) fputc(tree_printline[i], f);
+fprintf(f, "-->%s [%d]\n", p->name, p->balance);
 tree_printline[pos] = barswitch? '|' : ' ';
 if (p->left != NULL)
   {
   tree_printline[pos+2] = '|';
-  tree_printsub(p->left, pos+2, 0);
+  tree_printsub(p->left, pos+2, 0, f);
   }
 }
 
-void tree_print(tree_node *p)
+/* The external function, with just a tree node argument. */
+
+void
+tree_print(tree_node *p, FILE *f)
 {
 int i;
+if (f == NULL) return;
 for (i = 0; i < tree_printlinesize; i++) tree_printline[i] = ' ';
-if (p == NULL) printf("Empty Tree\n"); else tree_printsub(p, 0, 0);
-printf("---- End of tree ----\n");
+if (p == NULL) fprintf(f, "Empty Tree\n"); else tree_printsub(p, 0, 0, f);
+fprintf(f, "---- End of tree ----\n");
 }
 
 /* End of tree.c */
