@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 1996 */
+/* Copyright (c) University of Cambridge 1995 - 1997 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Function for setting up the version string. */
@@ -10,7 +10,7 @@
 #include "exim.h"
 
 
-#define THIS_VERSION  "0.53"
+#define THIS_VERSION  "1.62"
 
 
 /* The header file cnumber.h contains a single line containing the
@@ -18,12 +18,13 @@ compilation number, making it easy to have it updated automatically.
 Hence the fudgery below to get the number turned into a string, since
 we can't use #include inside a macro argument list */
 
-void version_init(void)
+void
+version_init(void)
 {
 int i = 0;
 char today[20];
 
-int cnumber = 
+int cnumber =
 #include "cnumber.h"
 ;
 
